@@ -1,14 +1,16 @@
 import collections
 import contextlib
 import os
+import tempfile
 from typing import (
-    Iterator,
-    Mapping,
     List,
     Dict,
     Optional,
     TYPE_CHECKING,
+    Tuple,
+    FrozenSet,
 )
+from collections.abc import Iterator, Mapping
 
 from debputy.build_support.build_context import BuildContext
 from debputy.build_support.buildsystem_detection import (
@@ -16,11 +18,13 @@ from debputy.build_support.buildsystem_detection import (
 )
 from debputy.commands.debputy_cmd.context import CommandContext
 from debputy.manifest_parser.base_types import BuildEnvironmentDefinition
-from debputy.plugin.debputy.to_be_api_types import BuildRule
+from debputy.packages import BinaryPackage
+from debputy.plugins.debputy.to_be_api_types import BuildRule
 from debputy.util import (
     _error,
     _info,
     _non_verbose_info,
+    generated_content_dir,
 )
 
 if TYPE_CHECKING:
@@ -28,20 +32,23 @@ if TYPE_CHECKING:
 
 
 @contextlib.contextmanager
-def in_build_env(build_env: BuildEnvironmentDefinition):
-    remove_unnecessary_env()
+def in_build_env(
+    build_env: BuildEnvironmentDefinition,
+    *,
+    env_is_for_clean: bool = False,
+) -> Iterator[None]:
     # Should possibly be per build
-    with _setup_build_env(build_env):
+    with _setup_build_env(build_env, env_is_for_clean=env_is_for_clean):
         yield
 
 
-def _set_stem_if_absent(stems: List[Optional[str]], idx: int, stem: str) -> None:
+def _set_stem_if_absent(stems: list[str | None], idx: int, stem: str) -> None:
     if stems[idx] is None:
         stems[idx] = stem
 
 
 def assign_stems(
-    build_rules: List[BuildRule],
+    build_rules: list[BuildRule],
     manifest: "HighLevelManifest",
 ) -> None:
     if not build_rules:
@@ -52,11 +59,11 @@ def assign_stems(
 
     debs = {p.name for p in manifest.all_packages if p.package_type == "deb"}
     udebs = {p.name for p in manifest.all_packages if p.package_type == "udeb"}
-    deb_only_builds: List[int] = []
-    udeb_only_builds: List[int] = []
-    by_name_only_builds: Dict[str, List[int]] = collections.defaultdict(list)
+    deb_only_builds: list[int] = []
+    udeb_only_builds: list[int] = []
+    by_name_only_builds: dict[str, list[int]] = collections.defaultdict(list)
     stems = [rule.name for rule in build_rules]
-    reserved_stems = set(n for n in stems if n is not None)
+    reserved_stems = {n for n in stems if n is not None}
 
     for idx, rule in enumerate(build_rules):
         stem = stems[idx]
@@ -94,7 +101,9 @@ def assign_stems(
 def perform_builds(
     context: CommandContext,
     manifest: "HighLevelManifest",
+    build_system_install_dirs: list[tuple[str, frozenset[BinaryPackage]]],
 ) -> None:
+    prune_unnecessary_env()
     build_rules = manifest.build_rules
     if build_rules is not None:
         if not build_rules:
@@ -133,6 +142,10 @@ def perform_builds(
                     _error(
                         f"An error occurred during build/install at {step_ref} (defined at {build_rule.attribute_path.path}): {str(e)}"
                     )
+            dest_dir = build_rule.install_dest_dir()
+            if dest_dir is not None:
+                build_system_install_dirs.append((dest_dir, build_rule.for_packages))
+
             _info(f"Completed build for {step_ref}.")
 
     else:
@@ -145,13 +158,18 @@ def perform_builds(
                     build_context,
                     manifest,
                 )
+                dest_dir = build_system.install_dest_dir()
+                if dest_dir is not None:
+                    build_system_install_dirs.append(
+                        (dest_dir, build_system.for_packages)
+                    )
 
             _non_verbose_info("Upstream builds completed successfully")
         else:
             _info("No build system was detected from the current plugin set.")
 
 
-def remove_unnecessary_env() -> None:
+def prune_unnecessary_env() -> None:
     vs = [
         "XDG_CACHE_HOME",
         "XDG_CONFIG_DIRS",
@@ -163,12 +181,15 @@ def remove_unnecessary_env() -> None:
     for v in vs:
         if v in os.environ:
             del os.environ[v]
-
-    # FIXME: Add custom HOME + XDG_RUNTIME_DIR
+    os.environ["HOME"] = "/non-existent"
 
 
 @contextlib.contextmanager
-def _setup_build_env(build_env: BuildEnvironmentDefinition) -> Iterator[None]:
+def _setup_build_env(
+    build_env: BuildEnvironmentDefinition,
+    *,
+    env_is_for_clean: bool = False,
+) -> Iterator[None]:
     env_backup = dict(os.environ)
     env = dict(env_backup)
     had_delta = False
@@ -177,7 +198,19 @@ def _setup_build_env(build_env: BuildEnvironmentDefinition) -> Iterator[None]:
         _set_env(env)
         had_delta = True
     _info("Updated environment to match build")
-    yield
+    if env_is_for_clean:
+        yield
+    else:
+        orig_home = env_backup["HOME"]
+        env["HOME"] = generated_content_dir(subdir_key="dpty_buildtime_home")
+        with tempfile.TemporaryDirectory() as xdg_runtime_dir:
+            env["XDG_RUNTIME_DIR"] = xdg_runtime_dir
+            yield
+        try:
+            del env["XDG_RUNTIME_DIR"]
+        except KeyError:
+            pass
+        env["HOME"] = orig_home
     if had_delta or env != env_backup:
         _set_env(env_backup)
 
