@@ -9,16 +9,11 @@ import textwrap
 import traceback
 from tempfile import TemporaryDirectory
 from typing import (
-    List,
-    Dict,
     Any,
-    Tuple,
-    Optional,
     NoReturn,
     NamedTuple,
     Literal,
     cast,
-    FrozenSet,
 )
 from collections.abc import Sequence
 
@@ -69,7 +64,7 @@ except ImportError:
 
 from debputy.version import __version__
 from debputy.filesystem_scan import (
-    FSROOverlay,
+    OSFSROOverlay,
     FSRootDir,
 )
 from debputy.plugin.api.impl_types import (
@@ -882,7 +877,7 @@ def assemble(
     *,
     debug_materialization: bool = False,
 ) -> None:
-    source_fs = FSROOverlay.create_root_dir("..", ".")
+    source_fs = OSFSROOverlay.create_root_dir("..", ".")
     source_version = manifest.source_version()
     is_native = "-" not in source_version
     is_dh_rrr_only_mode = integration_mode == INTEGRATION_MODE_DH_DEBPUTY_RRR
