@@ -6,17 +6,14 @@ import os
 from typing import (
     Optional,
     Tuple,
-    Mapping,
     FrozenSet,
     Set,
     Union,
-    Sequence,
-    Iterable,
-    Callable,
     Dict,
     TYPE_CHECKING,
     Literal,
 )
+from collections.abc import Mapping, Sequence, Iterable, Callable
 
 from debian.debian_support import DpkgArchTable
 
@@ -27,7 +24,7 @@ from debputy.architecture_support import (
 )
 from debputy.dh.dh_assistant import read_dh_addon_sequences
 from debputy.exceptions import DebputyRuntimeError
-from debputy.filesystem_scan import FSROOverlay
+from debputy.filesystem_scan import OSFSROOverlay
 from debputy.highlevel_manifest import HighLevelManifest
 from debputy.highlevel_manifest_parser import YAMLManifestParser
 from debputy.integration_detection import determine_debputy_integration_mode
@@ -53,6 +50,8 @@ from debputy.util import (
     setup_logging,
     PRINT_COMMAND,
     change_log_level,
+    _warn,
+    TRACE_LOG,
 )
 
 if TYPE_CHECKING:
@@ -89,6 +88,22 @@ class Command:
     requested_plugins_only: bool = False
 
 
+def _host_dpo_to_dbo(opt_and_profiles: "DebBuildOptionsAndProfiles", v: str) -> bool:
+
+    if (
+        v in opt_and_profiles.deb_build_profiles
+        and v not in opt_and_profiles.deb_build_options
+    ):
+        val = os.environ.get("DEB_BUILD_OPTIONS", "") + " " + v
+        _warn(
+            f'Copying "{v}" into DEB_BUILD_OPTIONS: It was in DEB_BUILD_PROFILES but not in DEB_BUILD_OPTIONS'
+        )
+        os.environ["DEB_BUILD_OPTIONS"] = val.lstrip()
+        # Note: It will not be immediately visible since `DebBuildOptionsAndProfiles` caches the result
+        return True
+    return False
+
+
 class CommandContext:
     def __init__(
         self,
@@ -104,20 +119,20 @@ class CommandContext:
         self._debputy_plugin_feature_set: PluginProvidedFeatureSet = (
             PluginProvidedFeatureSet()
         )
-        self._debian_dir = FSROOverlay.create_root_dir("debian", "debian")
-        self._mtime: Optional[int] = None
-        self._source_variables: Optional[Mapping[str, str]] = None
-        self._substitution: Optional[Substitution] = None
-        self._requested_plugins: Optional[Sequence[str]] = None
+        self._debian_dir = OSFSROOverlay.create_root_dir("debian", "debian")
+        self._mtime: int | None = None
+        self._source_variables: Mapping[str, str] | None = None
+        self._substitution: Substitution | None = None
+        self._requested_plugins: Sequence[str] | None = None
         self._plugins_loaded = False
-        self._dctrl_parser: Optional[DctrlParser] = None
-        self.debputy_integration_mode: Optional[DebputyIntegrationMode] = None
-        self._dctrl_data: Optional[
-            Tuple[
+        self._dctrl_parser: DctrlParser | None = None
+        self.debputy_integration_mode: DebputyIntegrationMode | None = None
+        self._dctrl_data: None | (
+            tuple[
                 "SourcePackage",
                 Mapping[str, "BinaryPackage"],
             ]
-        ] = None
+        ) = None
         self._package_set: Literal["both", "arch", "indep"] = "both"
 
     @property
@@ -149,17 +164,25 @@ class CommandContext:
     def dctrl_parser(self) -> DctrlParser:
         parser = self._dctrl_parser
         if parser is None:
-            packages: Union[Set[str], FrozenSet[str]] = frozenset()
+            packages: set[str] | frozenset[str] = frozenset()
             if hasattr(self.parsed_args, "packages"):
                 packages = self.parsed_args.packages
 
+            instance = DebBuildOptionsAndProfiles(environ=os.environ)
+
+            dirty = _host_dpo_to_dbo(instance, "nodoc")
+            dirty = _host_dpo_to_dbo(instance, "nocheck") or dirty
+
+            if dirty:
+                instance = DebBuildOptionsAndProfiles(environ=os.environ)
+
             parser = DctrlParser(
                 packages,  # -p/--package
                 set(),  # -N/--no-package
                 # binary-indep and binary-indep (dpkg BuildDriver integration only)
                 self._package_set == "indep",
                 self._package_set == "arch",
-                deb_options_and_profiles=DebBuildOptionsAndProfiles.instance(),
+                deb_options_and_profiles=instance,
                 dpkg_architecture_variables=dpkg_architecture_table(),
                 dpkg_arch_query_table=DpkgArchTable.load_arch_table(),
             )
@@ -182,7 +205,7 @@ class CommandContext:
             self._requested_plugins = self._resolve_requested_plugins()
         return self._requested_plugins
 
-    def required_plugins(self) -> Set[str]:
+    def required_plugins(self) -> set[str]:
         return set(getattr(self.parsed_args, "required_plugins") or [])
 
     @property
@@ -190,7 +213,7 @@ class CommandContext:
         return self.dctrl_parser.deb_options_and_profiles
 
     @property
-    def deb_build_options(self) -> Mapping[str, Optional[str]]:
+    def deb_build_options(self) -> Mapping[str, str | None]:
         return self.deb_build_options_and_profiles.deb_build_options
 
     def _create_substitution(
@@ -278,7 +301,7 @@ class CommandContext:
 
     def _parse_dctrl(
         self,
-    ) -> Tuple[
+    ) -> tuple[
         "SourcePackage",
         Mapping[str, "BinaryPackage"],
     ]:
@@ -337,12 +360,12 @@ class CommandContext:
             self.debputy_integration_mode = integration_mode
         return integration_mode
 
-    def set_log_level_for_build_subcommand(self) -> Optional[int]:
+    def set_log_level_for_build_subcommand(self) -> int | None:
         parsed_args = self.parsed_args
-        log_level: Optional[int] = None
+        log_level: int | None = None
         if os.environ.get("DH_VERBOSE", "") != "":
             log_level = PRINT_COMMAND
-        if parsed_args.debug_mode or os.environ.get("DEBPUTY_DEBUG", "") != "":
+        if parsed_args.debug_mode:
             log_level = logging.DEBUG
         if log_level is not None:
             change_log_level(log_level)
@@ -351,7 +374,7 @@ class CommandContext:
     def manifest_parser(
         self,
         *,
-        manifest_path: Optional[str] = None,
+        manifest_path: str | None = None,
     ) -> YAMLManifestParser:
         substitution = self.substitution
         dctrl_parser = self.dctrl_parser
@@ -378,7 +401,7 @@ class CommandContext:
     def parse_manifest(
         self,
         *,
-        manifest_path: Optional[str] = None,
+        manifest_path: str | None = None,
     ) -> HighLevelManifest:
         substitution = self.substitution
         manifest_required = False
@@ -420,7 +443,7 @@ class SubcommandBase(CommandBase):
         name: str,
         *,
         aliases: Sequence[str] = tuple(),
-        help_description: Optional[str] = None,
+        help_description: str | None = None,
     ) -> None:
         self.name = name
         self.aliases = aliases
@@ -456,12 +479,12 @@ class GenericSubCommand(SubcommandBase):
         handler: Callable[[CommandContext], None],
         *,
         aliases: Sequence[str] = tuple(),
-        help_description: Optional[str] = None,
-        configure_handler: Optional[Callable[[argparse.ArgumentParser], None]] = None,
+        help_description: str | None = None,
+        configure_handler: Callable[[argparse.ArgumentParser], None] | None = None,
         require_substitution: bool = True,
         requested_plugins_only: bool = False,
         log_only_to_stderr: bool = False,
-        default_log_level: Union[int, Callable[[CommandContext], int]] = logging.INFO,
+        default_log_level: int | Callable[[CommandContext], int] = logging.INFO,
     ) -> None:
         super().__init__(name, aliases=aliases, help_description=help_description)
         self._handler = handler
@@ -501,10 +524,10 @@ class GenericSubCommand(SubcommandBase):
             assert callable(default_log_level)
             level = default_log_level(context)
         change_log_level(level)
-        if level > logging.DEBUG and (
-            context.parsed_args.debug_mode or os.environ.get("DEBPUTY_DEBUG", "") != ""
-        ):
+        if level > logging.DEBUG and context.parsed_args.debug_mode:
             change_log_level(logging.DEBUG)
+        if level > TRACE_LOG and os.environ.get("DEBPUTY_DEBUG", "") == "trace":
+            change_log_level(TRACE_LOG)
         return self._handler(context)
 
 
@@ -520,9 +543,9 @@ class DispatchingCommandMixin(CommandBase):
         dest: str,
         *,
         aliases: Sequence[str] = tuple(),
-        help_description: Optional[str] = None,
+        help_description: str | None = None,
         metavar: str = "command",
-        default_subcommand: Optional[str] = None,
+        default_subcommand: str | None = None,
     ) -> "DispatcherCommand":
         ds = DispatcherCommand(
             name,
@@ -537,16 +560,16 @@ class DispatchingCommandMixin(CommandBase):
 
     def register_subcommand(
         self,
-        name: Union[str, Sequence[str]],
+        name: str | Sequence[str],
         *,
-        help_description: Optional[str] = None,
-        argparser: Optional[
-            Union[ArgparserConfigurator, Sequence[ArgparserConfigurator]]
-        ] = None,
+        help_description: str | None = None,
+        argparser: None | (
+            ArgparserConfigurator | Sequence[ArgparserConfigurator]
+        ) = None,
         require_substitution: bool = True,
         requested_plugins_only: bool = False,
         log_only_to_stderr: bool = False,
-        default_log_level: Union[int, Callable[[CommandContext], int]] = logging.INFO,
+        default_log_level: int | Callable[[CommandContext], int] = logging.INFO,
     ) -> Callable[[CommandHandler], GenericSubCommand]:
         if isinstance(name, str):
             cmd_name = name
@@ -601,17 +624,17 @@ class DispatcherCommand(SubcommandBase, DispatchingCommandMixin):
         dest: str,
         *,
         aliases: Sequence[str] = tuple(),
-        help_description: Optional[str] = None,
+        help_description: str | None = None,
         metavar: str = "command",
-        default_subcommand: Optional[str] = None,
+        default_subcommand: str | None = None,
     ) -> None:
         super().__init__(name, aliases=aliases, help_description=help_description)
-        self._aliases: Dict[str, SubcommandBase] = {}
-        self._subcommands: Dict[str, SubcommandBase] = {}
+        self._aliases: dict[str, SubcommandBase] = {}
+        self._subcommands: dict[str, SubcommandBase] = {}
         self._dest = dest
         self._metavar = metavar
         self._default_subcommand = default_subcommand
-        self._argparser: Optional[argparse.ArgumentParser] = None
+        self._argparser: argparse.ArgumentParser | None = None
 
     def add_subcommand(self, subcommand: SubcommandBase) -> None:
         all_names = [subcommand.name]
