@@ -6,15 +6,13 @@ import sys
 import textwrap
 from itertools import chain
 from typing import (
-    Sequence,
     Union,
     Tuple,
-    Iterable,
     Any,
     Optional,
     Type,
-    Callable,
 )
+from collections.abc import Sequence, Iterable, Callable
 
 from debputy.analysis.analysis_util import flatten_ppfs
 from debputy.commands.debputy_cmd.context import (
@@ -25,17 +23,20 @@ from debputy.commands.debputy_cmd.context import (
 from debputy.commands.debputy_cmd.output import (
     _stream_to_pager,
     _output_styling,
-    OutputStylingBase,
+    IOBasedOutputStyling,
 )
 from debputy.exceptions import DebputySubstitutionError
 from debputy.filesystem_scan import build_virtual_fs
 from debputy.manifest_parser.declarative_parser import (
     BASIC_SIMPLE_TYPES,
 )
-from debputy.manifest_parser.parser_data import ParserContextData
-from debputy.manifest_parser.parser_doc import render_rule
-from debputy.manifest_parser.tagging_types import TypeMapping
-from debputy.manifest_parser.util import unpack_type, AttributePath
+from debputy.manifest_parser.parser_doc import (
+    render_rule,
+    render_multiline_documentation,
+    render_type_mapping,
+    render_source_type,
+)
+from debputy.manifest_parser.util import unpack_type
 from debputy.packager_provided_files import detect_all_packager_provided_files
 from debputy.plugin.api.doc_parsing import parser_type_name
 from debputy.plugin.api.example_processing import (
@@ -51,7 +52,6 @@ from debputy.plugin.api.impl_types import (
     PluginProvidedDiscardRule,
     AutomaticDiscardRuleExample,
     MetadataOrMaintscriptDetector,
-    PluginProvidedTypeMapping,
     DebputyPluginMetadata,
     DeclarativeInputParser,
 )
@@ -59,9 +59,6 @@ from debputy.plugin.api.parser_tables import (
     SUPPORTED_DISPATCHABLE_TABLE_PARSERS,
     OPARSER_MANIFEST_ROOT,
 )
-from debputy.plugin.api.spec import (
-    TypeMappingExample,
-)
 from debputy.substitution import Substitution
 from debputy.util import _error, _warn, manifest_format_doc
 
@@ -170,9 +167,9 @@ def _ppf_flags(ppf: PackagerProvidedFileClassSpec) -> str:
     argparser=TEXT_ONLY_FORMAT,
 )
 def _plugin_cmd_list_uppf(context: CommandContext) -> None:
-    ppf_table = context.load_plugins().packager_provided_files
+    plugin_feature_set = context.load_plugins()
     all_ppfs = detect_all_packager_provided_files(
-        ppf_table,
+        plugin_feature_set,
         context.debian_dir,
         context.binary_packages(),
     )
@@ -197,7 +194,7 @@ def _plugin_cmd_list_uppf(context: CommandContext) -> None:
 
     with _stream_to_pager(context.parsed_args) as (fd, fo):
         if used_ppfs:
-            headers: Sequence[Union[str, Tuple[str, str]]] = [
+            headers: Sequence[str | tuple[str, str]] = [
                 "File",
                 "Matched Stem",
                 "Installed Into",
@@ -219,7 +216,7 @@ def _plugin_cmd_list_uppf(context: CommandContext) -> None:
             )
 
         if inactive_ppfs:
-            headers: Sequence[Union[str, Tuple[str, str]]] = [
+            headers: Sequence[str | tuple[str, str]] = [
                 "UNUSED FILE",
                 "Matched Stem",
                 "Installed Into",
@@ -252,7 +249,7 @@ def _plugin_cmd_list_ppf(context: CommandContext) -> None:
     ppfs: Iterable[PackagerProvidedFileClassSpec]
     ppfs = context.load_plugins().packager_provided_files.values()
     with _stream_to_pager(context.parsed_args) as (fd, fo):
-        headers: Sequence[Union[str, Tuple[str, str]]] = [
+        headers: Sequence[str | tuple[str, str]] = [
             "Stem",
             "Installed As",
             ("Mode", ">"),
@@ -327,7 +324,7 @@ def _render_manifest_variable_flag(variable: PluginProvidedManifestVariable) ->
     return ",".join(flags)
 
 
-def _render_list_filter(v: Optional[bool]) -> str:
+def _render_list_filter(v: bool | None) -> str:
     if v is None:
         return "N/A"
     return "shown" if v else "hidden"
@@ -439,7 +436,7 @@ def _plugin_cmd_list_manifest_rules(context: CommandContext) -> None:
 
     # Type hint to make the chain call easier for the type checker, which does not seem
     # to derive to this common base type on its own.
-    base_type = Iterable[Tuple[Union[str, Type[Any]], DispatchingParserBase[Any]]]
+    base_type = Iterable[tuple[Union[str, type[Any]], DispatchingParserBase[Any]]]
 
     parser_generator = feature_set.manifest_parser_generator
     table_parsers: base_type = parser_generator.dispatchable_table_parsers.items()
@@ -486,32 +483,13 @@ def _plugin_cmd_list_automatic_discard_rules(context: CommandContext) -> None:
         )
 
 
-def _render_manifest_variable_value(v: Optional[str]) -> str:
+def _render_manifest_variable_value(v: str | None) -> str:
     if v is None:
         return "(N/A: Cannot resolve the variable)"
     v = v.replace("\n", "\\n").replace("\t", "\\t")
     return v
 
 
-def _render_multiline_documentation(
-    documentation: str,
-    *,
-    first_line_prefix: str = "Documentation: ",
-    following_line_prefix: str = " ",
-) -> None:
-    current_prefix = first_line_prefix
-    for line in documentation.splitlines(keepends=False):
-        if line.isspace():
-            if not current_prefix.isspace():
-                print(current_prefix.rstrip())
-                current_prefix = following_line_prefix
-            else:
-                print()
-            continue
-        print(f"{current_prefix}{line}")
-        current_prefix = following_line_prefix
-
-
 @plugin_show_cmds.register_subcommand(
     ["manifest-variables"],
     help_description="Plugin provided manifest variables (such as `{{path:FOO}}`)",
@@ -530,7 +508,7 @@ def _plugin_cmd_show_manifest_variables(context: CommandContext) -> None:
     fo = _output_styling(context.parsed_args, sys.stdout)
     if variable_name.startswith("{{") and variable_name.endswith("}}"):
         variable_name = variable_name[2:-2]
-    variable: Optional[PluginProvidedManifestVariable]
+    variable: PluginProvidedManifestVariable | None
     if variable_name.startswith("env:") and len(variable_name) > 4:
         env_var = variable_name[4:]
         variable = PluginProvidedManifestVariable(
@@ -578,7 +556,8 @@ def _plugin_cmd_show_manifest_variables(context: CommandContext) -> None:
             binary_value = None
 
     doc = variable.variable_reference_documentation or "No documentation provided"
-    _render_multiline_documentation(doc)
+    for line in render_multiline_documentation(doc):
+        print(line)
 
     if source_value == binary_value:
         print(f"Resolved: {_render_manifest_variable_value(source_value)}")
@@ -603,7 +582,7 @@ def _plugin_cmd_show_manifest_variables(context: CommandContext) -> None:
 
 def _determine_ppf(
     context: CommandContext,
-) -> Tuple[PackagerProvidedFileClassSpec, bool]:
+) -> tuple[PackagerProvidedFileClassSpec, bool]:
     feature_set = context.load_plugins()
     ppf_name = context.parsed_args.ppf_name
     try:
@@ -634,7 +613,7 @@ def _determine_ppf(
         if "/" not in basename:
             debian_dir = build_virtual_fs([basename])
             all_ppfs = detect_all_packager_provided_files(
-                feature_set.packager_provided_files,
+                feature_set,
                 debian_dir,
                 context.binary_packages(),
             )
@@ -789,7 +768,7 @@ def lookup_pmr_rule(
 
     matched = []
 
-    base_type = Iterable[Tuple[Union[str, Type[Any]], DispatchingParserBase[Any]]]
+    base_type = Iterable[tuple[Union[str, type[Any]], DispatchingParserBase[Any]]]
     parser_generator = feature_set.manifest_parser_generator
     table_parsers: base_type = parser_generator.dispatchable_table_parsers.items()
     object_parsers: base_type = parser_generator.dispatchable_object_parsers.items()
@@ -862,6 +841,7 @@ def lookup_pmr_rule(
 def _plugin_cmd_show_manifest_rule(context: CommandContext) -> None:
     feature_set = context.load_plugins()
     parsed_args = context.parsed_args
+    fo = _output_styling(parsed_args, sys.stdout)
 
     try:
         pmr_rule = lookup_pmr_rule(feature_set, parsed_args.pmr_rule_name)
@@ -873,6 +853,7 @@ def _plugin_cmd_show_manifest_rule(context: CommandContext) -> None:
             pmr_rule.rule_name,
             pmr_rule.parser,
             pmr_rule.plugin_metadata,
+            fo,
             is_root_rule=pmr_rule.is_root_rule,
         )
     )
@@ -897,7 +878,7 @@ def _plugin_cmd_show_manifest_rule(context: CommandContext) -> None:
 
 
 def _render_discard_rule_example(
-    fo: OutputStylingBase,
+    fo: IOBasedOutputStyling,
     discard_rule: PluginProvidedDiscardRule,
     example: AutomaticDiscardRuleExample,
 ) -> None:
@@ -996,7 +977,10 @@ def _render_discard_rule(
     )
     print()
     doc = discard_rule.reference_documentation or "No documentation provided"
-    _render_multiline_documentation(doc, first_line_prefix="", following_line_prefix="")
+    for line in render_multiline_documentation(
+        doc, first_line_prefix="", following_line_prefix=""
+    ):
+        print(line)
 
     if len(discard_rule.examples) > 1:
         print()
@@ -1041,20 +1025,6 @@ def _plugin_cmd_show_automatic_discard_rules(context: CommandContext) -> None:
     _render_discard_rule(context, discard_rule)
 
 
-def _render_source_type(t: Any) -> str:
-    _, origin_type, args = unpack_type(t, False)
-    if origin_type == Union:
-        at = ", ".join(_render_source_type(st) for st in args)
-        return f"One of: {at}"
-    name = BASIC_SIMPLE_TYPES.get(t)
-    if name is not None:
-        return name
-    try:
-        return t.__name__
-    except AttributeError:
-        return str(t)
-
-
 @plugin_list_cmds.register_subcommand(
     "type-mappings",
     help_description="Registered type mappings/descriptions",
@@ -1068,7 +1038,7 @@ def _plugin_cmd_list_type_mappings(context: CommandContext) -> None:
             [
                 (
                     target_type.__name__,
-                    _render_source_type(type_mapping.mapped_type.source_type),
+                    render_source_type(type_mapping.mapped_type.source_type),
                     type_mapping.plugin_metadata.plugin_name,
                 )
                 for target_type, type_mapping in type_mappings.items()
@@ -1088,6 +1058,7 @@ def _plugin_cmd_list_type_mappings(context: CommandContext) -> None:
 def _plugin_cmd_show_type_mappings(context: CommandContext) -> None:
     type_mapping_name = context.parsed_args.type_mapping
     type_mappings = context.load_plugins().mapped_types
+    fo = _output_styling(context.parsed_args, sys.stdout)
 
     matches = []
     for type_ in type_mappings:
@@ -1134,95 +1105,21 @@ def _plugin_cmd_show_type_mappings(context: CommandContext) -> None:
         )
 
     match = matches[0]
-    _render_type(context, type_mappings[match])
-
-
-def _render_type_example(
-    context: CommandContext,
-    fo: OutputStylingBase,
-    parser_context: ParserContextData,
-    type_mapping: TypeMapping[Any, Any],
-    example: TypeMappingExample,
-) -> Tuple[str, bool]:
-    attr_path = AttributePath.builtin_path()["CLI Request"]
-    v = _render_value(example.source_input)
-    try:
-        type_mapping.mapper(
-            example.source_input,
-            attr_path,
-            parser_context,
-        )
-    except RuntimeError:
-        if context.parsed_args.debug_mode:
-            raise
-        fo.print(
-            fo.colored("Broken example: ", fg="red")
-            + f"Provided example input ({v})"
-            + " caused an exception when parsed. Please file a bug against the plugin."
-            + " Use --debug/DEBPUTY_DEBUG=1 to see the stack trace"
-        )
-        return fo.colored(v, fg="red") + " [Example value could not be parsed]", True
-    return fo.colored(v, fg="green"), False
-
-
-def _render_type(
-    context: CommandContext,
-    pptm: PluginProvidedTypeMapping,
-) -> None:
-    fo = _output_styling(context.parsed_args, sys.stdout)
-    type_mapping = pptm.mapped_type
-    target_type = type_mapping.target_type
-    ref_doc = pptm.reference_documentation
-    desc = ref_doc.description if ref_doc is not None else None
-    examples = ref_doc.examples if ref_doc is not None else tuple()
-
-    fo.print(fo.colored(f"# Type Mapping: {target_type.__name__}", style="bold"))
-    fo.print()
-    if desc is not None:
-        _render_multiline_documentation(
-            desc, first_line_prefix="", following_line_prefix=""
-        )
-    else:
-        fo.print("No documentation provided.")
-
-    context.parse_manifest()
-
     manifest_parser = context.manifest_parser()
 
-    if examples:
-        had_issues = False
-        fo.print()
-        fo.print(fo.colored("## Example values", style="bold"))
-        fo.print()
-        for no, example in enumerate(examples, start=1):
-            v, i = _render_type_example(
-                context, fo, manifest_parser, type_mapping, example
-            )
-            fo.print(f" * {v}")
-            if i:
-                had_issues = True
-    else:
-        had_issues = False
+    pptm = type_mappings[match]
+    fo.print(
+        render_type_mapping(
+            pptm,
+            fo,
+            manifest_parser,
+            recover_from_broken_examples=context.parsed_args.debug_mode,
+        )
+    )
 
     fo.print()
     fo.print(f"Provided by plugin: {pptm.plugin_metadata.plugin_name}")
 
-    if had_issues:
-        fo.print()
-        fo.print(
-            fo.colored(
-                "Examples had issues. Please file a bug against the plugin", fg="red"
-            )
-        )
-        fo.print()
-        fo.print("Use --debug/DEBPUTY_DEBUG=1 to see the stacktrace")
-
-
-def _render_value(v: Any) -> str:
-    if isinstance(v, str) and '"' not in v:
-        return f'"{v}"'
-    return str(v)
-
 
 def ensure_plugin_commands_are_loaded() -> None:
     # Loading the module does the heavy lifting
