@@ -15,24 +15,21 @@ import textwrap
 from contextlib import ExitStack, suppress
 from tempfile import mkstemp
 from typing import (
-    Iterable,
     List,
     Optional,
     Set,
     Dict,
-    Sequence,
     Tuple,
-    Iterator,
     Literal,
     TypeVar,
     FrozenSet,
     cast,
     Any,
     Union,
-    Mapping,
     AbstractSet,
     TYPE_CHECKING,
 )
+from collections.abc import Iterable, Sequence, Iterator, Mapping
 
 import debian.deb822
 from debian.changelog import Changelog
@@ -139,9 +136,9 @@ def generate_md5sums_file(
 
 
 def install_or_generate_conffiles(
-    ctrl_root: Union[FSPath, FSControlRootDir],
+    ctrl_root: FSPath | FSControlRootDir,
     fs_root: VirtualPath,
-    reserved_packager_provided_files: Dict[str, List[PackagerProvidedFile]],
+    reserved_packager_provided_files: dict[str, list[PackagerProvidedFile]],
 ) -> None:
     provided_conffiles_file = resolve_reserved_provided_file(
         "conffiles",
@@ -270,7 +267,7 @@ def _find_and_analyze_systemd_service_files(
         f"./usr/lib/systemd/{systemd_service_dir}",
         f"./lib/systemd/{systemd_service_dir}",
     ]
-    aliases: Dict[str, List[str]] = collections.defaultdict(list)
+    aliases: dict[str, list[str]] = collections.defaultdict(list)
     seen = set()
     all_files = []
 
@@ -309,10 +306,10 @@ def _prune_dch_file(
     package: BinaryPackage,
     path: VirtualPath,
     is_changelog: bool,
-    keep_versions: Optional[Set[str]],
+    keep_versions: set[str] | None,
     *,
     trim: bool = True,
-) -> Tuple[bool, Optional[Set[str]]]:
+) -> tuple[bool, set[str] | None]:
     # TODO: Process `d/changelog` once
     # Note we cannot assume that changelog_file is always `d/changelog` as you can have
     # per-package changelogs.
@@ -363,7 +360,7 @@ def _prune_dch_file(
 
     with (
         path.replace_fs_path_content() as fs_path,
-        open(fs_path, "wt", encoding="utf-8") as fd,
+        open(fs_path, "w", encoding="utf-8") as fd,
     ):
         for entry in kept_entries:
             fd.write(str(entry))
@@ -392,7 +389,7 @@ def _prune_dch_file(
             ) as binnmu_changelog,
             open(
                 binnmu_changelog.fs_path,
-                "wt",
+                "w",
                 encoding="utf-8",
             ) as binnmu_fd,
         ):
@@ -450,13 +447,13 @@ _UPSTREAM_CHANGELOG_NAMES = {
     f"{n}{s}": (sw, nw)
     for (nw, n), (sw, s) in itertools.product(
         enumerate(["changelog", "changes", "history"], start=1),
-        enumerate(["", ".txt", ".md", ".rst"], start=1),
+        enumerate(["", ".txt", ".md", ".rst", ".org"], start=1),
     )
 }
 _NONE_TUPLE = (None, (0, 0))
 
 
-def _detect_upstream_changelog(names: Iterable[str]) -> Optional[str]:
+def _detect_upstream_changelog(names: Iterable[str]) -> str | None:
     matches = []
     for name in names:
         match_priority = _UPSTREAM_CHANGELOG_NAMES.get(name.lower())
@@ -513,14 +510,14 @@ def install_upstream_changelog(
 class _ElfInfo:
     path: VirtualPath
     fs_path: str
-    is_stripped: Optional[bool] = None
-    build_id: Optional[str] = None
-    dbgsym: Optional[FSPath] = None
+    is_stripped: bool | None = None
+    build_id: str | None = None
+    dbgsym: FSPath | None = None
 
 
 def _elf_static_lib_walk_filter(
     fs_path: VirtualPath,
-    children: List[VP],
+    children: list[VP],
 ) -> bool:
     if (
         fs_path.name == ".build-id"
@@ -540,7 +537,7 @@ def _elf_static_lib_walk_filter(
 
 
 @contextlib.contextmanager
-def _all_elf_files(fs_root: VirtualPath) -> Iterator[Dict[str, _ElfInfo]]:
+def _all_elf_files(fs_root: VirtualPath) -> Iterator[dict[str, _ElfInfo]]:
     all_elf_files = find_all_elf_files(
         fs_root,
         walk_filter=_elf_static_lib_walk_filter,
@@ -595,13 +592,13 @@ def _find_all_static_libs(
 
 
 @contextlib.contextmanager
-def _all_static_libs(fs_root: FSPath) -> Iterator[List[str]]:
+def _all_static_libs(fs_root: FSPath) -> Iterator[list[str]]:
     all_static_libs = list(_find_all_static_libs(fs_root))
     if not all_static_libs:
         yield []
         return
     with ExitStack() as cm_stack:
-        resolved: List[str] = [
+        resolved: list[str] = [
             cm_stack.enter_context(p.replace_fs_path_content()) for p in all_static_libs
         ]
         yield resolved
@@ -610,7 +607,7 @@ def _all_static_libs(fs_root: FSPath) -> Iterator[List[str]]:
 _FILE_BUILD_ID_RE = re.compile(rb"BuildID(?:\[\S+\])?=([A-Fa-f0-9]+)")
 
 
-def _resolve_build_ids(elf_info: Dict[str, _ElfInfo]) -> None:
+def _resolve_build_ids(elf_info: dict[str, _ElfInfo]) -> None:
     static_cmd = ["file", "-00", "-N"]
     if detect_fakeroot():
         static_cmd.append("--no-sandbox")
@@ -670,7 +667,7 @@ def _make_debug_file(
     return dbgsym
 
 
-def _strip_binary(strip: str, options: List[str], paths: Iterable[str]) -> None:
+def _strip_binary(strip: str, options: list[str], paths: Iterable[str]) -> None:
     # We assume the paths are obtained via `p.replace_fs_path_content()`,
     # which is the case at the time of written and should remain so forever.
     it = iter(paths)
@@ -709,7 +706,7 @@ def _attach_debug(objcopy: str, elf_binary: VirtualPath, dbgsym: FSPath) -> None
             )
 
 
-@functools.lru_cache()
+@functools.lru_cache
 def _has_tool(tool: str) -> bool:
     return shutil.which(tool) is not None
 
@@ -717,7 +714,7 @@ def _has_tool(tool: str) -> bool:
 def _run_dwz(
     dctrl: BinaryPackage,
     dbgsym_fs_root: FSPath,
-    unstripped_elf_info: List[_ElfInfo],
+    unstripped_elf_info: list[_ElfInfo],
 ) -> None:
     if not unstripped_elf_info or dctrl.is_udeb or not _has_tool("dwz"):
         return
@@ -762,7 +759,7 @@ def relocate_dwarves_into_dbgsym_packages(
     dbgsym_fs_root: VirtualPath,
     *,
     run_dwz: bool = False,
-) -> List[str]:
+) -> list[str]:
     # FIXME: hardlinks
     with _all_static_libs(package_fs_root) as all_static_files:
         if all_static_files:
@@ -858,7 +855,7 @@ def cross_package_control_files(
     errors = []
     combined_shlibs = ShlibsContent()
     shlibs_dir = None
-    shlib_dirs: List[str] = []
+    shlib_dirs: list[str] = []
     shlibs_local = manifest.debian_dir.get("shlibs.local")
     if shlibs_local and shlibs_local.is_file:
         with shlibs_local.open() as fd:
@@ -918,7 +915,7 @@ def cross_package_control_files(
         if shlibs_dir is None:
             shlibs_dir = generated_content_dir(subdir_key="_shlibs_materialization_dir")
         generated_shlibs_local = os.path.join(shlibs_dir, "shlibs.local")
-        with open(generated_shlibs_local, "wt", encoding="utf-8") as fd:
+        with open(generated_shlibs_local, "w", encoding="utf-8") as fd:
             combined_shlibs.write_to(fd)
         _info(f"Generated {generated_shlibs_local} for dpkg-shlibdeps")
 
@@ -934,12 +931,12 @@ def cross_package_control_files(
 
 def _relevant_service_definitions(
     service_rule: ServiceRule,
-    service_managers: Union[List[str], FrozenSet[str]],
+    service_managers: list[str] | frozenset[str],
     by_service_manager_key: Mapping[
-        Tuple[str, str, str, str], Tuple[ServiceManagerDetails, ServiceDefinition[Any]]
+        tuple[str, str, str, str], tuple[ServiceManagerDetails, ServiceDefinition[Any]]
     ],
-    aliases: Mapping[str, Sequence[Tuple[str, str, str, str]]],
-) -> Iterable[Tuple[Tuple[str, str, str, str], ServiceDefinition[Any]]]:
+    aliases: Mapping[str, Sequence[tuple[str, str, str, str]]],
+) -> Iterable[tuple[tuple[str, str, str, str], ServiceDefinition[Any]]]:
     as_keys = (key for key in aliases[service_rule.service])
 
     pending_queue = {
@@ -948,7 +945,7 @@ def _relevant_service_definitions(
         if key in by_service_manager_key
         and service_rule.applies_to_service_manager(key[-1])
     }
-    relevant_names: Dict[Tuple[str, str, str, str], ServiceDefinition[Any]] = {}
+    relevant_names: dict[tuple[str, str, str, str], ServiceDefinition[Any]] = {}
     seen_keys = set()
 
     if not pending_queue:
@@ -1096,7 +1093,7 @@ def setup_control_files(
     binary_package_data: "BinaryPackageData",
     manifest: "HighLevelManifest",
     dbgsym_fs_root: VirtualPath,
-    dbgsym_ids: List[str],
+    dbgsym_ids: list[str],
     package_metadata_context: PackageProcessingContext,
     *,
     allow_ctrl_file_management: bool = True,
@@ -1237,7 +1234,7 @@ def setup_control_files(
     if generated_triggers:
         assert allow_ctrl_file_management
         dest_file = os.path.join(control_output_fs_path, "triggers")
-        with open(dest_file, "at", encoding="utf-8") as fd:
+        with open(dest_file, "a", encoding="utf-8") as fd:
             fd.writelines(
                 textwrap.dedent(
                     f"""\
@@ -1270,7 +1267,7 @@ def setup_control_files(
 def _generate_snippet(
     control_output_dir: str,
     script: str,
-    maintscript_snippets: Dict[str, MaintscriptSnippetContainer],
+    maintscript_snippets: dict[str, MaintscriptSnippetContainer],
 ) -> None:
     debputy_snippets = maintscript_snippets.get(script)
     if debputy_snippets is None:
@@ -1286,7 +1283,7 @@ def _generate_snippet(
     if not full_content:
         return
     filename = os.path.join(control_output_dir, script)
-    with open(filename, "wt") as fd:
+    with open(filename, "w") as fd:
         fd.write("#!/bin/sh\nset -e\n\n")
         fd.write(full_content)
         os.chmod(fd.fileno(), 0o755)
@@ -1343,7 +1340,7 @@ def _generate_dbgsym_control_file_if_relevant(
     dbgsym_fs_root: VirtualPath,
     dbgsym_control_dir: FSControlRootDir,
     dbgsym_ids: str,
-    multi_arch: Optional[str],
+    multi_arch: str | None,
     dctrl: str,
     extra_common_params: Sequence[str],
 ) -> None:
@@ -1416,7 +1413,7 @@ def _generate_dbgsym_control_file_if_relevant(
     os.chmod(os.path.join(dbgsym_control_fs_path, "control"), 0o644)
 
 
-def _all_parent_directories_of(directories: Iterable[str]) -> Set[str]:
+def _all_parent_directories_of(directories: Iterable[str]) -> set[str]:
     result = {"."}
     for path in directories:
         current = os.path.dirname(path)
@@ -1429,7 +1426,7 @@ def _all_parent_directories_of(directories: Iterable[str]) -> Set[str]:
 def _compute_multi_arch_for_arch_all_doc(
     binary_package: BinaryPackage,
     fs_root: FSPath,
-) -> Optional[str]:
+) -> str | None:
     if not binary_package.name.endswith(("-doc", "-docs")):
         # We limit by package name, since there are tricks involving a `Multi-Arch: no` depending on a
         # `Multi-Arch: same` to emulate `Multi-Arch: allowed`. Said `Multi-Arch: no` can have no contents.
@@ -1452,8 +1449,8 @@ def _compute_multi_arch_for_arch_all_doc(
 def _any_unacceptable_paths(
     fs_root: FSPath,
     *,
-    acceptable_no_descend_paths: Union[List[str], AbstractSet[str]] = frozenset(),
-    acceptable_files: Union[List[str], AbstractSet[str]] = frozenset(),
+    acceptable_no_descend_paths: list[str] | AbstractSet[str] = frozenset(),
+    acceptable_files: list[str] | AbstractSet[str] = frozenset(),
 ) -> bool:
     acceptable_intermediate_dirs = _all_parent_directories_of(
         itertools.chain(acceptable_no_descend_paths, acceptable_files)
@@ -1473,7 +1470,7 @@ def auto_compute_multi_arch(
     binary_package: BinaryPackage,
     control_output_dir: VirtualPath,
     fs_root: FSPath,
-) -> Optional[str]:
+) -> str | None:
     resolved_arch = binary_package.resolved_architecture
     if any(
         script
@@ -1520,7 +1517,7 @@ def auto_compute_multi_arch(
     return "same"
 
 
-@functools.lru_cache()
+@functools.lru_cache
 def _has_t64_enabled() -> bool:
     try:
         output = subprocess.check_output(
@@ -1572,7 +1569,7 @@ def _t64_migration_substvar(
         substvars[_T64_PROVIDES] = ""
 
 
-@functools.lru_cache()
+@functools.lru_cache
 def dpkg_field_list_pkg_dep() -> Sequence[str]:
     try:
         output = subprocess.check_output(
@@ -1593,7 +1590,7 @@ _SUBSTVARS_FIELDS_NOT_SUPPORTED_BY_DPKG = {
 }
 
 
-@functools.lru_cache()
+@functools.lru_cache
 def all_auto_substvars() -> Sequence[str]:
     result = [x for x in dpkg_field_list_pkg_dep()]
     result.extend(_SUBSTVARS_FIELDS_NOT_SUPPORTED_BY_DPKG)
@@ -1605,7 +1602,7 @@ def _handle_auto_substvars(
     dctrl_file: BinaryPackage,
     substvars: FlushableSubstvars,
     has_dbgsym: bool,
-) -> Optional[str]:
+) -> str | None:
     auto_substvars_fields = all_auto_substvars()
     auto_substvars_fields_lc = {x.lower(): x for x in auto_substvars_fields}
     substvar_fields = collections.defaultdict(set)
@@ -1640,7 +1637,7 @@ def _handle_auto_substvars(
         if addendum is None:
             # No merging required
             continue
-        substvars_part = ", ".join(addendum)
+        substvars_part = ", ".join(sorted(addendum))
         existing_value = replacement_stanza.get(field_name)
 
         if existing_value is None or existing_value.isspace():
@@ -1697,8 +1694,8 @@ def _generate_control_files(
     control_output_dir: FSControlRootDir,
     fs_root: FSPath,
     substvars: FlushableSubstvars,
-    dbgsym_root_fs: Optional[VirtualPath],
-    dbgsym_build_ids: Optional[List[str]],
+    dbgsym_root_fs: VirtualPath | None,
+    dbgsym_build_ids: list[str] | None,
 ) -> None:
     binary_package = binary_package_data.binary_package
     source_package = binary_package_data.source_package
