@@ -29,6 +29,7 @@ from debputy.dh_migration.models import (
     AcceptableMigrationIssues,
     UnsupportedFeature,
     ConflictingChange,
+    MigrationRequest,
 )
 from debputy.highlevel_manifest import HighLevelManifest
 from debputy.integration_detection import determine_debputy_integration_mode
@@ -375,16 +376,17 @@ def perform_migration(
     if permit_destructive_changes is None and uses_git:
         permit_destructive_changes = True
 
+    migration_request = MigrationRequest(
+        debian_dir,
+        manifest,
+        acceptable_migration_issues,
+        migration_target,
+    )
+
     try:
         for migrator in migrators:
             feature_migration = FeatureMigration(migrator.__name__, fo)
-            migrator(
-                debian_dir,
-                manifest,
-                acceptable_migration_issues,
-                feature_migration,
-                migration_target,
-            )
+            migrator(migration_request, feature_migration)
             migrations.append(feature_migration)
     except CannotEmulateExecutableDHConfigFile as e:
         _error(
