@@ -13,22 +13,17 @@ import time
 from abc import ABC
 from contextlib import suppress
 from typing import (
-    List,
-    Dict,
     Optional,
-    Tuple,
-    Union,
     cast,
     Any,
     ContextManager,
     TextIO,
     BinaryIO,
-    NoReturn,
-    Type,
     Generic,
     TypeVar,
     overload,
     Literal,
+    Never,
 )
 from collections.abc import Iterable, Iterator, Mapping, Callable
 from weakref import ref, ReferenceType
@@ -90,7 +85,7 @@ TextOpenMode = Literal[
     "a",
     "at",
 ]
-OpenMode = Union[BinaryOpenMode, TextOpenMode]
+OpenMode = Literal[BinaryOpenMode, TextOpenMode]
 
 
 class AlwaysEmptyReadOnlyMetadataReference(PathMetadataReference[PMT]):
@@ -442,6 +437,14 @@ class VirtualPathBase(VirtualPath, ABC):
         buffering: int = -1,
     ) -> BinaryIO: ...
 
+    @overload
+    def open_child(
+        self,
+        name: str,
+        mode: OpenMode = ...,
+        buffering: int = -1,
+    ) -> TextIO | BinaryIO: ...
+
     @contextlib.contextmanager
     def open_child(
         self,
@@ -1708,7 +1711,7 @@ class VirtualTestPath(FSPath):
         self._fs_path = new_fs_path
 
 
-class FSOverlayBase(VirtualPathBase, Generic[FSP]):
+class OSFSOverlayBase(VirtualPathBase, Generic[FSP]):
     __slots__ = (
         "_path",
         "_fs_path",
@@ -1761,8 +1764,6 @@ class FSOverlayBase(VirtualPathBase, Generic[FSP]):
             return stat.S_ISDIR(self.stat().st_mode)
         except FileNotFoundError:
             return False
-        except NotImplementedError:
-            print(self.__class__)
 
     @property
     def is_file(self) -> bool:
@@ -1816,7 +1817,7 @@ class FSOverlayBase(VirtualPathBase, Generic[FSP]):
             metadata_type,
         )
 
-    def all_paths(self) -> Iterable["FSControlPath"]:
+    def all_paths(self) -> Iterable["OSFSControlPath"]:
         yield self
         if not self.is_dir:
             return
@@ -1849,7 +1850,7 @@ class FSOverlayBase(VirtualPathBase, Generic[FSP]):
         return children
 
 
-class FSROOverlay(FSOverlayBase["FSROOverlay"]):
+class OSFSROOverlay(OSFSOverlayBase["FSROOverlay"]):
     __slots__ = (
         "_stat_cache",
         "_readlink_cache",
@@ -1861,27 +1862,27 @@ class FSROOverlay(FSOverlayBase["FSROOverlay"]):
         self,
         path: str,
         fs_path: str,
-        parent: Optional["FSROOverlay"],
+        parent: Optional["OSFSROOverlay"],
     ) -> None:
         super().__init__(path, fs_path, parent=parent)
         self._stat_cache: os.stat_result | None = None
         self._readlink_cache: str | None = None
         self._stat_failed_cache = False
-        self._children: Mapping[str, FSROOverlay] | None = None
+        self._children: Mapping[str, OSFSROOverlay] | None = None
 
     @classmethod
-    def create_root_dir(cls, path: str, fs_path: str) -> "FSROOverlay":
-        return FSROOverlay(path, fs_path, None)
+    def create_root_dir(cls, path: str, fs_path: str) -> "OSFSROOverlay":
+        return OSFSROOverlay(path, fs_path, None)
 
     @property
-    def iterdir(self) -> Iterable["FSROOverlay"]:
+    def iterdir(self) -> Iterable["OSFSROOverlay"]:
         if not self.is_dir:
             return
         if self._children is None:
             self._ensure_children_are_resolved()
         yield from assume_not_none(self._children).values()
 
-    def lookup(self, path: str) -> Optional["FSROOverlay"]:
+    def lookup(self, path: str) -> Optional["OSFSROOverlay"]:
         if not self.is_dir:
             return None
         if self._children is None:
@@ -1908,7 +1909,7 @@ class FSROOverlay(FSOverlayBase["FSROOverlay"]):
         if not self.is_dir or self._children:
             return
         self._children = self._resolve_children(
-            lambda n, fsp, p: FSROOverlay(n, fsp, p)
+            lambda n, fsp, p: OSFSROOverlay(n, fsp, p)
         )
 
     @property
@@ -1924,17 +1925,17 @@ class FSROOverlay(FSOverlayBase["FSROOverlay"]):
             key = key.name
         return self._children[key]
 
-    def __delitem__(self, key) -> None:
+    def __delitem__(self, key) -> Never:
         self._error_ro_fs()
 
     @property
     def is_read_write(self) -> bool:
         return False
 
-    def _rw_check(self) -> None:
+    def _rw_check(self) -> Never:
         self._error_ro_fs()
 
-    def _error_ro_fs(self) -> NoReturn:
+    def _error_ro_fs(self) -> Never:
         raise DebputyFSIsROError(
             f'Attempt to write to "{self.path}" failed:'
             " Debputy Virtual File system is R/O."
@@ -1959,7 +1960,7 @@ class FSROOverlay(FSOverlayBase["FSROOverlay"]):
         return stat.S_IMODE(self.stat().st_mode)
 
     @mode.setter
-    def mode(self, _unused: int) -> None:
+    def mode(self, _unused: int) -> Never:
         self._error_ro_fs()
 
     @property
@@ -1967,7 +1968,7 @@ class FSROOverlay(FSOverlayBase["FSROOverlay"]):
         return self.stat().st_mtime
 
     @mtime.setter
-    def mtime(self, new_mtime: float) -> None:
+    def mtime(self, new_mtime: float) -> Never:
         self._error_ro_fs()
 
     def readlink(self) -> str:
@@ -1981,10 +1982,10 @@ class FSROOverlay(FSOverlayBase["FSROOverlay"]):
         self,
         owner: StaticFileSystemOwner | None,
         group: StaticFileSystemGroup | None,
-    ) -> None:
+    ) -> Never:
         self._error_ro_fs()
 
-    def mkdir(self, name: str) -> "VirtualPath":
+    def mkdir(self, name: str) -> Never:
         self._error_ro_fs()
 
     def add_file(
@@ -1995,17 +1996,17 @@ class FSROOverlay(FSOverlayBase["FSROOverlay"]):
         use_fs_path_mode: bool = False,
         mode: int = 0o0644,
         mtime: float | None = None,
-    ) -> ContextManager["VirtualPath"]:
+    ) -> Never:
         self._error_ro_fs()
 
-    def add_symlink(self, link_name: str, link_target: str) -> "VirtualPath":
+    def add_symlink(self, link_name: str, link_target: str) -> Never:
         self._error_ro_fs()
 
-    def unlink(self, *, recursive: bool = False) -> None:
+    def unlink(self, *, recursive: bool = False) -> Never:
         self._error_ro_fs()
 
 
-class FSROOverlayRootDir(FSROOverlay):
+class OSFSROOverlayRootDir(OSFSROOverlay):
     __slots__ = ("_plugin_context",)
 
     def __init__(self, path: str, fs_path: str) -> None:
@@ -2021,17 +2022,17 @@ class FSROOverlayRootDir(FSROOverlay):
             yield r
 
 
-class FSControlPath(FSOverlayBase["FSControlPath"]):
+class OSFSControlPath(OSFSOverlayBase["FSControlPath"]):
 
     @property
-    def iterdir(self) -> Iterable["FSControlPath"]:
+    def iterdir(self) -> Iterable["OSFSControlPath"]:
         if not self.is_dir:
             return
         yield from self._resolve_children(
-            lambda n, fsp, p: FSControlPath(n, fsp, p)
+            lambda n, fsp, p: OSFSControlPath(n, fsp, p)
         ).values()
 
-    def lookup(self, path: str) -> Optional["FSControlPath"]:
+    def lookup(self, path: str) -> Optional["OSFSControlPath"]:
         if not self.is_dir:
             return None
 
@@ -2064,7 +2065,7 @@ class FSControlPath(FSOverlayBase["FSControlPath"]):
     def __getitem__(self, key) -> "VirtualPath":
         if not self.is_dir:
             raise KeyError(key)
-        children = self._resolve_children(lambda n, fsp, p: FSControlPath(n, fsp, p))
+        children = self._resolve_children(lambda n, fsp, p: OSFSControlPath(n, fsp, p))
         if isinstance(key, FSPath):
             key = key.name
         return children[key]
@@ -2092,7 +2093,7 @@ class FSControlPath(FSOverlayBase["FSControlPath"]):
     def mtime(self, new_mtime: float) -> None:
         os.utime(self.fs_path, (new_mtime, new_mtime))
 
-    def readlink(self) -> str:
+    def readlink(self) -> Never:
         if not self.is_symlink:
             raise TypeError(f"readlink is only valid for symlinks ({self.path!r})")
         assert False
@@ -2106,7 +2107,7 @@ class FSControlPath(FSOverlayBase["FSControlPath"]):
             "No need to chown paths in the control.tar: They are always root:root"
         )
 
-    def mkdir(self, name: str) -> "VirtualPath":
+    def mkdir(self, name: str) -> Never:
         raise TypeError("The control.tar never contains subdirectories.")
 
     @contextlib.contextmanager
@@ -2141,7 +2142,7 @@ class FSControlPath(FSOverlayBase["FSControlPath"]):
         with open(fs_path, "wb") as fd:
             # Ensure that the fs_path exists and default mode is reasonable
             os.chmod(fd.fileno(), mode)
-        child = FSControlPath(
+        child = OSFSControlPath(
             name,
             fs_path,
             self,
@@ -2166,7 +2167,7 @@ class FSControlPath(FSOverlayBase["FSControlPath"]):
         if restore_mode is not None:
             self.mode = restore_mode
 
-    def add_symlink(self, link_name: str, link_target: str) -> "VirtualPath":
+    def add_symlink(self, link_name: str, link_target: str) -> Never:
         raise TypeError("The control.tar never contains symlinks.")
 
     def unlink(self, *, recursive: bool = False) -> None:
@@ -2178,7 +2179,7 @@ class FSControlPath(FSOverlayBase["FSControlPath"]):
         os.unlink(self.fs_path)
 
 
-class FSControlRootDir(FSControlPath):
+class FSControlRootDir(OSFSControlPath):
 
     @classmethod
     def create_root_dir(cls, fs_path: str) -> "FSControlRootDir":
@@ -2195,7 +2196,7 @@ class FSControlRootDir(FSControlPath):
         # Ignored, but accepted for compat with FSPath's variant of this function.
         # - This is used by install_or_generate_conffiles.
         reference_path: VirtualPath | None = None,  # noqa
-    ) -> "FSControlPath":
+    ) -> "OSFSControlPath":
         if "/" in name or name in {".", ".."}:
             raise ValueError(f'Invalid file name: "{name}"')
         if not self.is_dir:
