@@ -2,25 +2,16 @@ import dataclasses
 import functools
 import os
 import textwrap
+import typing
 from contextlib import suppress
 from dataclasses import dataclass, field
 from typing import (
-    List,
-    Dict,
-    Iterable,
-    Mapping,
     Any,
-    Union,
-    Optional,
     TypeVar,
     Generic,
     cast,
-    Set,
-    Tuple,
-    Sequence,
-    FrozenSet,
-    Callable,
 )
+from collections.abc import Iterable, Mapping, Sequence, Callable
 
 from debian.debian_support import DpkgArchTable
 
@@ -37,7 +28,7 @@ from .exceptions import (
     DebputySubstitutionError,
     DebputyRuntimeErrorWithPreamble,
 )
-from .filesystem_scan import FSPath, FSRootDir, FSROOverlay, FSControlRootDir
+from .filesystem_scan import FSPath, FSRootDir, OSFSROOverlay, FSControlRootDir
 from .installations import (
     InstallRule,
     SourcePathMatcher,
@@ -120,8 +111,8 @@ class PathNotCoveredByInstallRulesError(DebputyRuntimeErrorWithPreamble):
 class DbgsymInfo:
     binary_package: BinaryPackage
     dbgsym_fs_root: FSPath
-    _dbgsym_root_fs: Optional[str]
-    dbgsym_ids: List[str]
+    _dbgsym_root_fs: str | None
+    dbgsym_ids: list[str]
     run_dwz: bool
 
     @property
@@ -168,20 +159,20 @@ class PackageTransformationDefinition:
     binary_package: BinaryPackage
     substitution: Substitution
     is_auto_generated_package: bool
-    binary_version: Optional[str] = None
-    search_dirs: Optional[List[FileSystemExactMatchRule]] = None
-    dpkg_maintscript_helper_snippets: List[DpkgMaintscriptHelperCommand] = field(
+    binary_version: str | None = None
+    search_dirs: list[FileSystemExactMatchRule] | None = None
+    dpkg_maintscript_helper_snippets: list[DpkgMaintscriptHelperCommand] = field(
         default_factory=list
     )
-    maintscript_snippets: Dict[str, MaintscriptSnippetContainer] = field(
+    maintscript_snippets: dict[str, MaintscriptSnippetContainer] = field(
         default_factory=dict
     )
-    transformations: List[TransformationRule] = field(default_factory=list)
-    reserved_packager_provided_files: Dict[str, List[PackagerProvidedFile]] = field(
+    transformations: list[TransformationRule] = field(default_factory=list)
+    reserved_packager_provided_files: dict[str, list[PackagerProvidedFile]] = field(
         default_factory=dict
     )
-    install_rules: List[InstallRule] = field(default_factory=list)
-    requested_service_rules: List[ServiceRule] = field(default_factory=list)
+    install_rules: list[InstallRule] = field(default_factory=list)
+    requested_service_rules: list[ServiceRule] = field(default_factory=list)
 
 
 def _path_to_tar_member(
@@ -275,8 +266,8 @@ class AbstractYAMLSubStore(Generic[ST]):
     def __init__(
         self,
         parent_store: Any,
-        parent_key: Optional[Union[int, str]],
-        store: Optional[ST] = None,
+        parent_key: int | str | None,
+        store: ST | None = None,
     ) -> None:
         if parent_store is not None and parent_key is not None:
             try:
@@ -338,13 +329,13 @@ class AbstractYAMLSubStore(Generic[ST]):
             raise RuntimeError("The definition has been removed!")
 
 
-class AbstractYAMLListSubStore(Generic[T], AbstractYAMLSubStore[List[T]]):
-    def _create_new_instance(self) -> List[T]:
+class AbstractYAMLListSubStore(Generic[T], AbstractYAMLSubStore[list[T]]):
+    def _create_new_instance(self) -> list[T]:
         return CommentedSeq()
 
 
-class AbstractYAMLDictSubStore(Generic[T], AbstractYAMLSubStore[Dict[str, T]]):
-    def _create_new_instance(self) -> Dict[str, T]:
+class AbstractYAMLDictSubStore(Generic[T], AbstractYAMLSubStore[dict[str, T]]):
+    def _create_new_instance(self) -> dict[str, T]:
         return CommentedMap()
 
 
@@ -363,7 +354,7 @@ class MutableCondition:
 class MutableYAMLSymlink(AbstractYAMLDictSubStore[Any]):
     @classmethod
     def new_symlink(
-        cls, link_path: str, link_target: str, condition: Optional[Any]
+        cls, link_path: str, link_target: str, condition: Any | None
     ) -> "MutableYAMLSymlink":
         inner = {
             MK_TRANSFORMATIONS_CREATE_SYMLINK_LINK_PATH: link_path,
@@ -387,7 +378,7 @@ class MutableYAMLSymlink(AbstractYAMLDictSubStore[Any]):
         ] = path
 
     @property
-    def symlink_target(self) -> Optional[str]:
+    def symlink_target(self) -> str | None:
         return self._store[MK_TRANSFORMATIONS_CREATE_SYMLINK][
             MK_TRANSFORMATIONS_CREATE_SYMLINK_LINK_TARGET
         ]
@@ -404,8 +395,8 @@ class MutableYAMLConffileManagementItem(AbstractYAMLDictSubStore[Any]):
     def rm_conffile(
         cls,
         conffile: str,
-        prior_to_version: Optional[str],
-        owning_package: Optional[str],
+        prior_to_version: str | None,
+        owning_package: str | None,
     ) -> "MutableYAMLConffileManagementItem":
         r = cls(
             None,
@@ -427,8 +418,8 @@ class MutableYAMLConffileManagementItem(AbstractYAMLDictSubStore[Any]):
         cls,
         old_conffile: str,
         new_conffile: str,
-        prior_to_version: Optional[str],
-        owning_package: Optional[str],
+        prior_to_version: str | None,
+        owning_package: str | None,
     ) -> "MutableYAMLConffileManagementItem":
         r = cls(
             None,
@@ -449,7 +440,7 @@ class MutableYAMLConffileManagementItem(AbstractYAMLDictSubStore[Any]):
         return r
 
     @property
-    def _container(self) -> Dict[str, Any]:
+    def _container(self) -> dict[str, Any]:
         assert len(self._store) == 1
         return next(iter(self._store.values()))
 
@@ -492,11 +483,11 @@ class MutableYAMLConffileManagementItem(AbstractYAMLDictSubStore[Any]):
         self._container[MK_CONFFILE_MANAGEMENT_RENAME_TARGET] = value
 
     @property
-    def prior_to_version(self) -> Optional[str]:
+    def prior_to_version(self) -> str | None:
         return self._container.get(MK_CONFFILE_MANAGEMENT_X_PRIOR_TO_VERSION)
 
     @prior_to_version.setter
-    def prior_to_version(self, value: Optional[str]) -> None:
+    def prior_to_version(self, value: str | None) -> None:
         if value is None:
             try:
                 del self._container[MK_CONFFILE_MANAGEMENT_X_PRIOR_TO_VERSION]
@@ -506,11 +497,11 @@ class MutableYAMLConffileManagementItem(AbstractYAMLDictSubStore[Any]):
             self._container[MK_CONFFILE_MANAGEMENT_X_PRIOR_TO_VERSION] = value
 
     @property
-    def owning_package(self) -> Optional[str]:
+    def owning_package(self) -> str | None:
         return self._container[MK_CONFFILE_MANAGEMENT_X_PRIOR_TO_VERSION]
 
     @owning_package.setter
-    def owning_package(self, value: Optional[str]) -> None:
+    def owning_package(self, value: str | None) -> None:
         if value is None:
             try:
                 del self._container[MK_CONFFILE_MANAGEMENT_X_OWNING_PACKAGE]
@@ -523,7 +514,7 @@ class MutableYAMLConffileManagementItem(AbstractYAMLDictSubStore[Any]):
 class MutableYAMLPackageDefinition(AbstractYAMLDictSubStore):
     def _list_store(
         self, key, *, create_if_absent: bool = False
-    ) -> Optional[List[Dict[str, Any]]]:
+    ) -> list[dict[str, Any]] | None:
         if self._is_detached or key not in self._store:
             if create_if_absent:
                 return None
@@ -571,12 +562,12 @@ class MutableYAMLPackageDefinition(AbstractYAMLDictSubStore):
 
 class AbstractMutableYAMLInstallRule(AbstractYAMLDictSubStore):
     @property
-    def _container(self) -> Dict[str, Any]:
+    def _container(self) -> dict[str, Any]:
         assert len(self._store) == 1
         return next(iter(self._store.values()))
 
     @property
-    def into(self) -> Optional[List[str]]:
+    def into(self) -> list[str] | None:
         v = self._container[MK_INSTALLATIONS_INSTALL_INTO]
         if v is None:
             return None
@@ -585,7 +576,7 @@ class AbstractMutableYAMLInstallRule(AbstractYAMLDictSubStore):
         return v
 
     @into.setter
-    def into(self, new_value: Optional[Union[str, List[str]]]) -> None:
+    def into(self, new_value: str | list[str] | None) -> None:
         if new_value is None:
             with suppress(KeyError):
                 del self._container[MK_INSTALLATIONS_INSTALL_INTO]
@@ -597,11 +588,11 @@ class AbstractMutableYAMLInstallRule(AbstractYAMLDictSubStore):
         self._container[MK_INSTALLATIONS_INSTALL_INTO] = new_list
 
     @property
-    def when(self) -> Optional[Union[str, Mapping[str, Any]]]:
+    def when(self) -> str | Mapping[str, Any] | None:
         return self._container[MK_CONDITION_WHEN]
 
     @when.setter
-    def when(self, new_value: Optional[Union[str, Mapping[str, Any]]]) -> None:
+    def when(self, new_value: str | Mapping[str, Any] | None) -> None:
         if new_value is None:
             with suppress(KeyError):
                 del self._container[MK_CONDITION_WHEN]
@@ -615,11 +606,11 @@ class AbstractMutableYAMLInstallRule(AbstractYAMLDictSubStore):
     @classmethod
     def install_dest(
         cls,
-        sources: Union[str, List[str]],
-        into: Optional[Union[str, List[str]]],
+        sources: str | list[str],
+        into: str | list[str] | None,
         *,
-        dest_dir: Optional[str] = None,
-        when: Optional[Union[str, Mapping[str, Any]]] = None,
+        dest_dir: str | None = None,
+        when: str | Mapping[str, Any] | None = None,
     ) -> "MutableYAMLInstallRuleInstall":
         k = MK_INSTALLATIONS_INSTALL_SOURCES
         if isinstance(sources, str):
@@ -646,11 +637,11 @@ class AbstractMutableYAMLInstallRule(AbstractYAMLDictSubStore):
     @classmethod
     def multi_dest_install(
         cls,
-        sources: Union[str, List[str]],
+        sources: str | list[str],
         dest_dirs: Sequence[str],
-        into: Optional[Union[str, List[str]]],
+        into: str | list[str] | None,
         *,
-        when: Optional[Union[str, Mapping[str, Any]]] = None,
+        when: str | Mapping[str, Any] | None = None,
     ) -> "MutableYAMLInstallRuleInstall":
         k = MK_INSTALLATIONS_INSTALL_SOURCES
         if isinstance(sources, str):
@@ -679,8 +670,8 @@ class AbstractMutableYAMLInstallRule(AbstractYAMLDictSubStore):
         cls,
         source: str,
         install_as: str,
-        into: Optional[Union[str, List[str]]],
-        when: Optional[Union[str, Mapping[str, Any]]] = None,
+        into: str | list[str] | None,
+        when: str | Mapping[str, Any] | None = None,
     ) -> "MutableYAMLInstallRuleInstall":
         r = MutableYAMLInstallRuleInstall(
             None,
@@ -706,8 +697,8 @@ class AbstractMutableYAMLInstallRule(AbstractYAMLDictSubStore):
         cls,
         source: str,
         install_as: str,
-        into: Optional[Union[str, List[str]]],
-        when: Optional[Union[str, Mapping[str, Any]]] = None,
+        into: str | list[str] | None,
+        when: str | Mapping[str, Any] | None = None,
     ) -> "MutableYAMLInstallRuleInstall":
         r = MutableYAMLInstallRuleInstall(
             None,
@@ -731,11 +722,11 @@ class AbstractMutableYAMLInstallRule(AbstractYAMLDictSubStore):
     @classmethod
     def install_docs(
         cls,
-        sources: Union[str, List[str]],
-        into: Optional[Union[str, List[str]]],
+        sources: str | list[str],
+        into: str | list[str] | None,
         *,
-        dest_dir: Optional[str] = None,
-        when: Optional[Union[str, Mapping[str, Any]]] = None,
+        dest_dir: str | None = None,
+        when: str | Mapping[str, Any] | None = None,
     ) -> "MutableYAMLInstallRuleInstall":
         k = MK_INSTALLATIONS_INSTALL_SOURCES
         if isinstance(sources, str):
@@ -762,9 +753,9 @@ class AbstractMutableYAMLInstallRule(AbstractYAMLDictSubStore):
     @classmethod
     def install_examples(
         cls,
-        sources: Union[str, List[str]],
-        into: Optional[Union[str, List[str]]],
-        when: Optional[Union[str, Mapping[str, Any]]] = None,
+        sources: str | list[str],
+        into: str | list[str] | None,
+        when: str | Mapping[str, Any] | None = None,
     ) -> "MutableYAMLInstallRuleInstallExamples":
         k = MK_INSTALLATIONS_INSTALL_SOURCES
         if isinstance(sources, str):
@@ -790,10 +781,10 @@ class AbstractMutableYAMLInstallRule(AbstractYAMLDictSubStore):
     @classmethod
     def install_man(
         cls,
-        sources: Union[str, List[str]],
-        into: Optional[Union[str, List[str]]],
-        language: Optional[str],
-        when: Optional[Union[str, Mapping[str, Any]]] = None,
+        sources: str | list[str],
+        into: str | list[str] | None,
+        language: str | None,
+        when: str | Mapping[str, Any] | None = None,
     ) -> "MutableYAMLInstallRuleMan":
         k = MK_INSTALLATIONS_INSTALL_SOURCES
         if isinstance(sources, str):
@@ -820,7 +811,7 @@ class AbstractMutableYAMLInstallRule(AbstractYAMLDictSubStore):
     @classmethod
     def discard(
         cls,
-        sources: Union[str, List[str]],
+        sources: str | list[str],
     ) -> "MutableYAMLInstallRuleDiscard":
         return MutableYAMLInstallRuleDiscard(
             None,
@@ -835,11 +826,11 @@ class MutableYAMLInstallRuleInstallExamples(AbstractMutableYAMLInstallRule):
 
 class MutableYAMLInstallRuleMan(AbstractMutableYAMLInstallRule):
     @property
-    def language(self) -> Optional[str]:
+    def language(self) -> str | None:
         return self._container[MK_INSTALLATIONS_INSTALL_MAN_LANGUAGE]
 
     @language.setter
-    def language(self, new_value: Optional[str]) -> None:
+    def language(self, new_value: str | None) -> None:
         if new_value is not None:
             self._container[MK_INSTALLATIONS_INSTALL_MAN_LANGUAGE] = new_value
             return
@@ -853,14 +844,14 @@ class MutableYAMLInstallRuleDiscard(AbstractMutableYAMLInstallRule):
 
 class MutableYAMLInstallRuleInstall(AbstractMutableYAMLInstallRule):
     @property
-    def sources(self) -> List[str]:
+    def sources(self) -> list[str]:
         v = self._container[MK_INSTALLATIONS_INSTALL_SOURCES]
         if isinstance(v, str):
             return [v]
         return v
 
     @sources.setter
-    def sources(self, new_value: Union[str, List[str]]) -> None:
+    def sources(self, new_value: str | list[str]) -> None:
         if isinstance(new_value, str):
             self._container[MK_INSTALLATIONS_INSTALL_SOURCES] = new_value
             return
@@ -868,11 +859,11 @@ class MutableYAMLInstallRuleInstall(AbstractMutableYAMLInstallRule):
         self._container[MK_INSTALLATIONS_INSTALL_SOURCES] = new_list
 
     @property
-    def dest_dir(self) -> Optional[str]:
+    def dest_dir(self) -> str | None:
         return self._container.get(MK_INSTALLATIONS_INSTALL_DEST_DIR)
 
     @dest_dir.setter
-    def dest_dir(self, new_value: Optional[str]) -> None:
+    def dest_dir(self, new_value: str | None) -> None:
         if new_value is not None and self.dest_as is not None:
             raise ValueError(
                 f'Cannot both have a "{MK_INSTALLATIONS_INSTALL_DEST_DIR}" and'
@@ -885,11 +876,11 @@ class MutableYAMLInstallRuleInstall(AbstractMutableYAMLInstallRule):
                 del self._container[MK_INSTALLATIONS_INSTALL_DEST_DIR]
 
     @property
-    def dest_as(self) -> Optional[str]:
+    def dest_as(self) -> str | None:
         return self._container.get(MK_INSTALLATIONS_INSTALL_AS)
 
     @dest_as.setter
-    def dest_as(self, new_value: Optional[str]) -> None:
+    def dest_as(self, new_value: str | None) -> None:
         if new_value is not None:
             if self.dest_dir is not None:
                 raise ValueError(
@@ -942,7 +933,7 @@ class MutableYAMLInstallationsDefinition(AbstractYAMLListSubStore[Any]):
 
 class MutableYAMLManifestVariables(AbstractYAMLDictSubStore):
     @property
-    def variables(self) -> Dict[str, Any]:
+    def variables(self) -> dict[str, Any]:
         return self._store
 
     def __setitem__(self, key: str, value: Any) -> None:
@@ -960,6 +951,25 @@ class MutableYAMLManifestDefinitions(AbstractYAMLDictSubStore):
         return d
 
 
+class MutableYAMLRemoveDuringCleanDefinitions(AbstractYAMLListSubStore[str]):
+    def append(self, rule: str) -> None:
+        self.create_definition_if_missing()
+        self._store.append(rule)
+
+    def __len__(self) -> int:
+        return len(self._store)
+
+    def extend(self, rules: Iterable[str]) -> None:
+        it = iter(rules)
+        try:
+            first_rule = next(it)
+        except StopIteration:
+            return
+        self.create_definition_if_missing()
+        self._store.append(first_rule)
+        self._store.extend(it)
+
+
 class MutableYAMLManifest:
     def __init__(self, store: Any) -> None:
         self._store = store
@@ -978,6 +988,18 @@ class MutableYAMLManifest:
             raise ValueError("Unsupported version")
         self._store[MK_MANIFEST_VERSION] = version
 
+    def remove_during_clean(
+        self,
+        *,
+        create_if_absent: bool = True,
+    ) -> MutableYAMLRemoveDuringCleanDefinitions:
+        d = MutableYAMLRemoveDuringCleanDefinitions(
+            self._store, MK_MANIFEST_REMOVE_DURING_CLEAN
+        )
+        if create_if_absent:
+            d.create_definition_if_missing()
+        return d
+
     def installations(
         self,
         *,
@@ -1062,7 +1084,7 @@ def _list_automatic_discard_rules(path_matcher: SourcePathMatcher) -> None:
     if not sum((len(v) for v in used_discard_rules.values()), 0):
         return
     _info("The following automatic discard rules were triggered:")
-    example_path: Optional[str] = None
+    example_path: str | None = None
     for rule in sorted(used_discard_rules):
         for fs_path in sorted(used_discard_rules[rule]):
             if example_path is None:
@@ -1091,7 +1113,7 @@ def _install_everything_from_source_dir_if_present(
     source_condition_context: ConditionContext,
     source_dir: VirtualPath,
     *,
-    into_dir: Optional[VirtualPath] = None,
+    into_dir: VirtualPath | None = None,
 ) -> None:
     attribute_path = AttributePath.builtin_path()[f"installing {source_dir.fs_path}"]
     pkg_set = frozenset([dctrl_bin])
@@ -1104,7 +1126,7 @@ def _install_everything_from_source_dir_if_present(
         f"Built-in; install everything from {source_dir.fs_path} into {dctrl_bin.name}",
         None,
     )
-    pkg_search_dir: Tuple[SearchDir] = (
+    pkg_search_dir: tuple[SearchDir] = (
         SearchDir(
             source_dir,
             pkg_set,
@@ -1135,11 +1157,11 @@ def _install_everything_from_source_dir_if_present(
 
 
 def _add_build_install_dirs_to_per_package_search_dirs(
-    build_system_install_dirs: Sequence[Tuple[str, FrozenSet[BinaryPackage]]],
-    per_package_search_dirs: Dict[BinaryPackage, List[VirtualPath]],
+    build_system_install_dirs: Sequence[tuple[str, frozenset[BinaryPackage]]],
+    per_package_search_dirs: dict[BinaryPackage, list[VirtualPath]],
     as_path: Callable[[str], VirtualPath],
 ) -> None:
-    seen_pp_search_dirs: Set[Tuple[BinaryPackage, str]] = set()
+    seen_pp_search_dirs: set[tuple[BinaryPackage, str]] = set()
     for dest_dir, for_packages in build_system_install_dirs:
         dest_path = as_path(dest_dir)
         for pkg in for_packages:
@@ -1157,8 +1179,9 @@ class HighLevelManifest:
     def __init__(
         self,
         manifest_path: str,
-        mutable_manifest: Optional[MutableYAMLManifest],
-        install_rules: Optional[List[InstallRule]],
+        mutable_manifest: MutableYAMLManifest | None,
+        remove_during_clean_rules: list[FileSystemMatchRule],
+        install_rules: list[InstallRule] | None,
         source_package: SourcePackage,
         binary_packages: Mapping[str, BinaryPackage],
         substitution: Substitution,
@@ -1167,23 +1190,31 @@ class HighLevelManifest:
         dpkg_arch_query_table: DpkgArchTable,
         build_env: DebBuildOptionsAndProfiles,
         build_environments: BuildEnvironments,
-        build_rules: Optional[List[BuildRule]],
+        build_rules: list[BuildRule] | None,
+        value_table: Mapping[
+            tuple[SourcePackage | BinaryPackage, type[Any]],
+            Any,
+        ],
         plugin_provided_feature_set: PluginProvidedFeatureSet,
         debian_dir: VirtualPath,
     ) -> None:
         self.manifest_path = manifest_path
         self.mutable_manifest = mutable_manifest
+        self._remove_during_clean_rules: list[FileSystemMatchRule] = (
+            remove_during_clean_rules
+        )
         self._install_rules = install_rules
-        self._source_package = source_package
+        self.source_package = source_package
         self._binary_packages = binary_packages
         self.substitution = substitution
         self.package_transformations = package_transformations
         self._dpkg_architecture_variables = dpkg_architecture_variables
-        self._dpkg_arch_query_table = dpkg_arch_query_table
+        self.dpkg_arch_query_table = dpkg_arch_query_table
         self._build_env = build_env
-        self._used_for: Set[str] = set()
+        self._used_for: set[str] = set()
         self.build_environments = build_environments
         self.build_rules = build_rules
+        self._value_table = value_table
         self._plugin_provided_feature_set = plugin_provided_feature_set
         self._debian_dir = debian_dir
         self._source_condition_context = ConditionContext(
@@ -1191,7 +1222,7 @@ class HighLevelManifest:
             substitution=self.substitution,
             deb_options_and_profiles=self._build_env,
             dpkg_architecture_variables=self._dpkg_architecture_variables,
-            dpkg_arch_query_table=self._dpkg_arch_query_table,
+            dpkg_arch_query_table=self.dpkg_arch_query_table,
         )
 
     def source_version(self, include_binnmu_version: bool = True) -> str:
@@ -1226,6 +1257,10 @@ class HighLevelManifest:
     def plugin_provided_feature_set(self) -> PluginProvidedFeatureSet:
         return self._plugin_provided_feature_set
 
+    @property
+    def remove_during_clean_rules(self) -> list[FileSystemMatchRule]:
+        return self._remove_during_clean_rules
+
     @property
     def active_packages(self) -> Iterable[BinaryPackage]:
         yield from (p for p in self._binary_packages.values() if p.should_be_acted_on)
@@ -1234,11 +1269,28 @@ class HighLevelManifest:
     def all_packages(self) -> Iterable[BinaryPackage]:
         yield from self._binary_packages.values()
 
+    def manifest_configuration[T](
+        self,
+        context_package: SourcePackage | BinaryPackage,
+        value_type: type[T],
+    ) -> T | None:
+        res = self._value_table.get((context_package, value_type))
+        return typing.cast("T | None", res)
+
     def package_state_for(self, package: str) -> PackageTransformationDefinition:
         return self.package_transformations[package]
 
     def _detect_doc_main_package_for(self, package: BinaryPackage) -> BinaryPackage:
         name = package.name
+        for doc_main_field in ("Doc-Main-Package", "X-Doc-Main-Package"):
+            doc_main_package_name = package.fields.get(doc_main_field)
+            if doc_main_package_name:
+                main_package = self._binary_packages.get(doc_main_package_name)
+                if main_package is None:
+                    _error(
+                        f"Invalid Doc-Main-Package for {name}: The package {doc_main_package_name!r} is not listed in d/control"
+                    )
+                return main_package
         # If it is not a -doc package, then docs should be installed
         # under its own package name.
         if not name.endswith("-doc"):
@@ -1258,9 +1310,9 @@ class HighLevelManifest:
     def perform_installations(
         self,
         integration_mode: DebputyIntegrationMode,
-        build_system_install_dirs: Sequence[Tuple[str, FrozenSet[BinaryPackage]]],
+        build_system_install_dirs: Sequence[tuple[str, frozenset[BinaryPackage]]],
         *,
-        install_request_context: Optional[InstallSearchDirContext] = None,
+        install_request_context: InstallSearchDirContext | None = None,
     ) -> PackageDataTable:
         package_data_dict = {}
         package_data_table = PackageDataTable(package_data_dict)
@@ -1283,7 +1335,7 @@ class HighLevelManifest:
 
             @functools.lru_cache(None)
             def _as_path(fs_path: str) -> VirtualPath:
-                return FSROOverlay.create_root_dir(".", fs_path)
+                return OSFSROOverlay.create_root_dir(".", fs_path)
 
             dtmp_dir = _as_path("debian/tmp")
             source_root_dir = _as_path(".")
@@ -1335,7 +1387,7 @@ class HighLevelManifest:
             dtmp_dir = None
             search_dirs = install_request_context.search_dirs
             into = frozenset(self._binary_packages.values())
-            seen: Set[BinaryPackage] = set()
+            seen: set[BinaryPackage] = set()
             for search_dir in search_dirs:
                 seen.update(search_dir.applies_to)
 
@@ -1415,7 +1467,7 @@ class HighLevelManifest:
             else:
                 build_system_staging_dir_fs_path = os.path.join("debian", package)
                 if os.path.isdir(build_system_staging_dir_fs_path):
-                    build_system_staging_dir = FSROOverlay.create_root_dir(
+                    build_system_staging_dir = OSFSROOverlay.create_root_dir(
                         ".",
                         build_system_staging_dir_fs_path,
                     )
@@ -1483,7 +1535,6 @@ class HighLevelManifest:
                 dctrl_bin,
                 udeb_package,
                 package_data_table,
-                # FIXME: source_package
             )
 
             ctrl_creator = BinaryCtrlAccessorProviderCreator(
@@ -1496,7 +1547,7 @@ class HighLevelManifest:
             if not enable_manifest_installation_feature:
                 assert_no_dbgsym_migration(dctrl_bin)
                 dh_dbgsym_root_fs = dhe_dbgsym_root_dir(dctrl_bin)
-                dh_dbgsym_root_path = FSROOverlay.create_root_dir(
+                dh_dbgsym_root_path = OSFSROOverlay.create_root_dir(
                     "",
                     dh_dbgsym_root_fs,
                 )
@@ -1529,7 +1580,7 @@ class HighLevelManifest:
                 )
 
             package_data_dict[package] = BinaryPackageData(
-                self._source_package,
+                self.source_package,
                 dctrl_bin,
                 build_system_pkg_staging_dir,
                 fs_root,
@@ -1545,7 +1596,7 @@ class HighLevelManifest:
         return package_data_table
 
     def condition_context(
-        self, binary_package: Optional[Union[BinaryPackage, str]]
+        self, binary_package: BinaryPackage | str | None
     ) -> ConditionContext:
         if binary_package is None:
             return self._source_condition_context
@@ -1577,7 +1628,7 @@ class HighLevelManifest:
             substitution=package_transformation.substitution,
             deb_options_and_profiles=self._build_env,
             dpkg_architecture_variables=self._dpkg_architecture_variables,
-            dpkg_arch_query_table=self._dpkg_arch_query_table,
+            dpkg_arch_query_table=self.dpkg_arch_query_table,
         )
         norm_rules = list(
             builtin_mode_normalization_rules(
@@ -1634,16 +1685,16 @@ class HighLevelManifest:
 @dataclasses.dataclass(slots=True)
 class SearchDirOrderState:
     search_dir: VirtualPath
-    applies_to: Union[Set[BinaryPackage], FrozenSet[BinaryPackage]] = dataclasses.field(
+    applies_to: set[BinaryPackage] | frozenset[BinaryPackage] = dataclasses.field(
         default_factory=set
     )
-    after: Set[str] = dataclasses.field(default_factory=set)
+    after: set[str] = dataclasses.field(default_factory=set)
 
 
 def _present_installation_dirs(
     search_dirs: Sequence[SearchDir],
     checked_missing_dirs: Sequence[VirtualPath],
-    all_pkgs: FrozenSet[BinaryPackage],
+    all_pkgs: frozenset[BinaryPackage],
 ) -> None:
     _info("The following directories are considered search dirs (in order):")
     max_len = max((len(s.search_dir.fs_path) for s in search_dirs), default=1)
@@ -1667,16 +1718,16 @@ def _present_installation_dirs(
 
 
 def _determine_search_dir_order(
-    requested: Mapping[BinaryPackage, List[VirtualPath]],
-    all_pkgs: FrozenSet[BinaryPackage],
-    default_search_dirs: List[VirtualPath],
+    requested: Mapping[BinaryPackage, list[VirtualPath]],
+    all_pkgs: frozenset[BinaryPackage],
+    default_search_dirs: list[VirtualPath],
     source_root: VirtualPath,
 ) -> Sequence[SearchDir]:
     search_dir_table = {}
     assert requested.keys() <= all_pkgs
     for pkg in all_pkgs:
         paths = requested.get(pkg, default_search_dirs)
-        previous_search_dir: Optional[SearchDirOrderState] = None
+        previous_search_dir: SearchDirOrderState | None = None
         for path in paths:
             try:
                 search_dir_state = search_dir_table[path.fs_path]
