@@ -1,17 +1,11 @@
 import collections
 import contextlib
+from collections.abc import Callable, Mapping, Iterator
 from typing import (
-    Optional,
-    Dict,
-    Callable,
-    List,
     Any,
-    Union,
-    Mapping,
     IO,
-    Iterator,
     cast,
-    Tuple,
+    TYPE_CHECKING,
 )
 
 from debian.debian_support import DpkgArchTable
@@ -32,6 +26,8 @@ from debputy.path_matcher import (
     ExactFileSystemPath,
     MatchRule,
 )
+from debputy.plugin.api.parser_tables import OPARSER_MANIFEST_ROOT
+from debputy.plugins.debputy.build_system_rules import BuildRule
 from debputy.substitution import Substitution
 from debputy.util import (
     _normalize_path,
@@ -51,7 +47,7 @@ from ._manifest_constants import (
     MK_MANIFEST_REMOVE_DURING_CLEAN,
 )
 from .architecture_support import DpkgArchitectureBuildProcessValuesTable
-from .filesystem_scan import FSROOverlay
+from .filesystem_scan import OSFSROOverlay
 from .installations import InstallRule, PPFInstallRule
 from .manifest_parser.base_types import (
     BuildEnvironments,
@@ -70,11 +66,15 @@ from .plugin.api.impl_types import (
     DispatchingTableParser,
     PackageContextData,
 )
-from debputy.plugin.api.parser_tables import OPARSER_MANIFEST_ROOT
 from .plugin.api.spec import DebputyIntegrationMode
-from debputy.plugins.debputy.build_system_rules import BuildRule
+from .plugin.plugin_state import with_binary_pkg_parsing_context, begin_parsing_context
 from .yaml import YAMLError, MANIFEST_YAML
 
+
+if TYPE_CHECKING:
+    from .plugins.debputy.binary_package_rules import BinaryVersion
+
+
 try:
     from Levenshtein import distance
 except ImportError:
@@ -115,8 +115,8 @@ else:
 def _per_package_subst_variables(
     p: BinaryPackage,
     *,
-    name: Optional[str] = None,
-) -> Dict[str, str]:
+    name: str | None = None,
+) -> dict[str, str]:
     return {
         "PACKAGE": name if name is not None else p.name,
     }
@@ -136,41 +136,45 @@ class HighLevelManifestParser(ParserContextData):
         debputy_integration_mode: DebputyIntegrationMode,
         *,
         # Available for testing purposes only
-        debian_dir: Union[str, VirtualPath] = "./debian",
+        debian_dir: str | VirtualPath = "./debian",
     ):
         self.manifest_path = manifest_path
         self._source_package = source_package
         self._binary_packages = binary_packages
-        self._mutable_yaml_manifest: Optional[MutableYAMLManifest] = None
+        self._mutable_yaml_manifest: MutableYAMLManifest | None = None
         # In source context, some variables are known to be unresolvable. Record this, so
         # we can give better error messages.
         self._substitution = substitution
         self._dpkg_architecture_variables = dpkg_architecture_variables
         self._dpkg_arch_query_table = dpkg_arch_query_table
         self._deb_options_and_profiles = build_env
-        self._package_state_stack: List[PackageTransformationDefinition] = []
+        self._package_state_stack: list[PackageTransformationDefinition] = []
         self._plugin_provided_feature_set = plugin_provided_feature_set
         self._debputy_integration_mode = debputy_integration_mode
         self._declared_variables = {}
         self._used_named_envs = set()
-        self._build_environments: Optional[BuildEnvironments] = BuildEnvironments(
+        self._build_environments: BuildEnvironments | None = BuildEnvironments(
             {},
             None,
         )
         self._has_set_default_build_environment = False
         self._read_build_environment = False
-        self._build_rules: Optional[List[BuildRule]] = None
+        self._build_rules: list[BuildRule] | None = None
+        self._value_table: dict[
+            tuple[SourcePackage | BinaryPackage, type[Any]],
+            Any,
+        ] = {}
 
         if isinstance(debian_dir, str):
-            debian_dir = FSROOverlay.create_root_dir("debian", debian_dir)
+            debian_dir = OSFSROOverlay.create_root_dir("debian", debian_dir)
 
         self._debian_dir = debian_dir
 
         # Delayed initialized; we rely on this delay to parse the variables.
         self._all_package_states = None
 
-        self._install_rules: Optional[List[InstallRule]] = None
-        self._remove_during_clean_rules: List[FileSystemMatchRule] = []
+        self._install_rules: list[InstallRule] | None = None
+        self._remove_during_clean_rules: list[FileSystemMatchRule] = []
         self._ownership_caches_loaded = False
         self._used = False
 
@@ -209,6 +213,10 @@ class HighLevelManifestParser(ParserContextData):
                 ),
             )
 
+    @property
+    def source_package(self) -> SourcePackage:
+        return self._source_package
+
     @property
     def binary_packages(self) -> Mapping[str, BinaryPackage]:
         return self._binary_packages
@@ -245,6 +253,13 @@ class HighLevelManifestParser(ParserContextData):
         if self._used:
             raise TypeError("build_manifest can only be called once!")
         self._used = True
+        return begin_parsing_context(
+            self._value_table,
+            self._source_package,
+            self._build_manifest,
+        )
+
+    def _build_manifest(self) -> HighLevelManifest:
         self._ensure_package_states_is_initialized()
         for var, attribute_path in self._declared_variables.items():
             if not self.substitution.is_used(var):
@@ -293,13 +308,15 @@ class HighLevelManifestParser(ParserContextData):
             self._deb_options_and_profiles,
             build_environments,
             self._build_rules,
+            self._value_table,
             self._plugin_provided_feature_set,
             self._debian_dir,
         )
 
     @contextlib.contextmanager
     def binary_package_context(
-        self, package_name: str
+        self,
+        package_name: str,
     ) -> Iterator[PackageTransformationDefinition]:
         if package_name not in self._package_states:
             self._error(
@@ -310,7 +327,8 @@ class HighLevelManifestParser(ParserContextData):
         package_state = self._package_states[package_name]
         self._package_state_stack.append(package_state)
         ps_len = len(self._package_state_stack)
-        yield package_state
+        with with_binary_pkg_parsing_context(package_state.binary_package):
+            yield package_state
         if ps_len != len(self._package_state_stack):
             raise RuntimeError("Internal error: Unbalanced stack manipulation detected")
         self._package_state_stack.pop()
@@ -333,7 +351,7 @@ class HighLevelManifestParser(ParserContextData):
 
     def add_extra_substitution_variables(
         self,
-        **extra_substitutions: Tuple[str, AttributePath],
+        **extra_substitutions: tuple[str, AttributePath],
     ) -> Substitution:
         if self._package_state_stack or self._all_package_states is not None:
             # For one, it would not "bubble up" correctly when added to the lowest stack.
@@ -368,7 +386,7 @@ class HighLevelManifestParser(ParserContextData):
 
     def _register_build_environment(
         self,
-        name: Optional[str],
+        name: str | None,
         build_environment: BuildEnvironmentDefinition,
         attribute_path: AttributePath,
         is_default: bool = False,
@@ -402,7 +420,7 @@ class HighLevelManifestParser(ParserContextData):
 
     def resolve_build_environment(
         self,
-        name: Optional[str],
+        name: str | None,
         attribute_path: AttributePath,
     ) -> BuildEnvironmentDefinition:
         if name is None:
@@ -509,7 +527,7 @@ class YAMLManifestParser(HighLevelManifestParser):
         key: str,
         attribute_parent_path: AttributePath,
         expected_type=None,
-        extra: Optional[Union[str, Callable[[], str]]] = None,
+        extra: str | Callable[[], str] | None = None,
     ):
         v = d.get(key)
         if v is None:
@@ -534,8 +552,8 @@ class YAMLManifestParser(HighLevelManifestParser):
         self,
         v,
         t,
-        key: Union[str, int, AttributePath],
-        attribute_parent_path: Optional[AttributePath],
+        key: str | int | AttributePath,
+        attribute_parent_path: AttributePath | None,
     ):
         if v is None:
             return None
@@ -554,7 +572,7 @@ class YAMLManifestParser(HighLevelManifestParser):
             )
         return v
 
-    def from_yaml_dict(self, yaml_data: object) -> "HighLevelManifest":
+    def _from_yaml_dict(self, yaml_data: object) -> "HighLevelManifest":
         attribute_path = AttributePath.root_path(yaml_data)
         parser_generator = self._plugin_provided_feature_set.manifest_parser_generator
         dispatchable_object_parsers = parser_generator.dispatchable_object_parsers
@@ -589,7 +607,7 @@ class YAMLManifestParser(HighLevelManifestParser):
                         f'Cannot define rules for package "{package_name}" (at {definition_source.path}). It is an'
                         " auto-generated package."
                     )
-                binary_version = parsed.get(MK_BINARY_VERSION)
+                binary_version: str | None = parsed.get(MK_BINARY_VERSION)
                 if binary_version is not None:
                     package_state.binary_version = (
                         package_state.substitution.substitute(
@@ -615,7 +633,7 @@ class YAMLManifestParser(HighLevelManifestParser):
 
         return self.build_manifest()
 
-    def _parse_manifest(self, fd: Union[IO[bytes], str]) -> HighLevelManifest:
+    def _parse_manifest(self, fd: IO[bytes] | str) -> HighLevelManifest:
         try:
             data = MANIFEST_YAML.load(fd)
         except YAMLError as e:
@@ -640,12 +658,18 @@ class YAMLManifestParser(HighLevelManifestParser):
                 f"Could not parse {self.manifest_path} as a YAML document: {msg}"
             ) from e
         self._mutable_yaml_manifest = MutableYAMLManifest(data)
-        return self.from_yaml_dict(data)
+
+        return begin_parsing_context(
+            self._value_table,
+            self._source_package,
+            self._from_yaml_dict,
+            data,
+        )
 
     def parse_manifest(
         self,
         *,
-        fd: Optional[Union[IO[bytes], str]] = None,
+        fd: IO[bytes] | str | None = None,
     ) -> HighLevelManifest:
         if fd is None:
             with open(self.manifest_path, "rb") as fd:
