@@ -13,7 +13,7 @@ from collections.abc import Mapping, Sequence, Callable
 
 from debputy.commands.debputy_cmd.context import CommandContext
 from debputy.commands.debputy_cmd.output import _output_styling, IOBasedOutputStyling
-from debputy.filesystem_scan import FSROOverlay
+from debputy.filesystem_scan import OSFSROOverlay
 from debputy.linting.lint_util import (
     LintReport,
     LintStateImpl,
@@ -98,7 +98,7 @@ class LintContext:
 
 
 def gather_lint_info(context: CommandContext) -> LintContext:
-    source_root = FSROOverlay.create_root_dir(".", ".")
+    source_root = OSFSROOverlay.create_root_dir(".", ".")
     debian_dir = source_root.get("debian")
     if debian_dir is not None and not debian_dir.is_dir:
         debian_dir = None
