@@ -9,18 +9,14 @@ from typing import (
     Optional,
     List,
     Any,
-    Mapping,
-    Container,
     TYPE_CHECKING,
     Tuple,
     Literal,
     Set,
     Dict,
-    Iterable,
-    Sequence,
-    Callable,
     Type,
 )
+from collections.abc import Mapping, Container, Iterable, Sequence, Callable
 
 import debputy.l10n as l10n
 from debputy.commands.debputy_cmd.output import (
@@ -32,7 +28,7 @@ from debputy.dh.dh_assistant import (
     DhSequencerData,
     extract_dh_addons_from_control,
 )
-from debputy.filesystem_scan import FSROOverlay, VirtualPathBase
+from debputy.filesystem_scan import OSFSROOverlay, VirtualPathBase
 from debputy.l10n import Translations
 from debputy.linting.lint_util import (
     LintState,
@@ -87,9 +83,9 @@ else:
 class FileCache:
     doc_uri: str
     path: str
-    is_open_in_editor: Optional[bool] = None
-    last_doc_version: Optional[int] = None
-    last_mtime: Optional[float] = None
+    is_open_in_editor: bool | None = None
+    last_doc_version: int | None = None
+    last_mtime: float | None = None
     is_valid: bool = False
 
     def _update_cache(self, doc: "TextDocument", source: str) -> None:
@@ -105,7 +101,7 @@ class FileCache:
             is_open = False
         else:
             is_open = True
-        new_content: Optional[str] = None
+        new_content: str | None = None
         if is_open:
             last_doc_version = self.last_doc_version
             current_doc_version = doc.version
@@ -141,7 +137,7 @@ class FileCache:
 
 @dataclasses.dataclass(slots=True)
 class Deb822FileCache(FileCache):
-    deb822_file: Optional[Deb822FileElement] = None
+    deb822_file: Deb822FileElement | None = None
 
     def _update_cache(self, doc: "TextDocument", source: str) -> None:
         deb822_file = parse_deb822_file(
@@ -157,9 +153,9 @@ class Deb822FileCache(FileCache):
 
 @dataclasses.dataclass(slots=True)
 class DctrlFileCache(Deb822FileCache):
-    dctrl_parser: Optional[DctrlParser] = None
-    source_package: Optional[SourcePackage] = None
-    binary_packages: Optional[Mapping[str, BinaryPackage]] = None
+    dctrl_parser: DctrlParser | None = None
+    source_package: SourcePackage | None = None
+    binary_packages: Mapping[str, BinaryPackage] | None = None
 
     def _update_cache(self, doc: "TextDocument", source: str) -> None:
         deb822_file, source_package, binary_packages = (
@@ -180,7 +176,7 @@ class DctrlFileCache(Deb822FileCache):
 
 @dataclasses.dataclass(slots=True)
 class SalsaCICache(FileCache):
-    parsed_content: Optional[CommentedMap] = None
+    parsed_content: CommentedMap | None = None
 
     def _update_cache(self, doc: "TextDocument", source: str) -> None:
         try:
@@ -196,7 +192,7 @@ class SalsaCICache(FileCache):
 
 @dataclasses.dataclass(slots=True)
 class DebianRulesCache(FileCache):
-    sequences: Optional[Set[str]] = None
+    sequences: set[str] | None = None
     saw_dh: bool = False
 
     def _update_cache(self, doc: "TextDocument", source: str) -> None:
@@ -225,12 +221,12 @@ class LSProvidedLintState(LintState):
         self._doc = doc
         # Cache lines (doc.lines re-splits everytime)
         self._lines = doc.lines
-        self._source_root = FSROOverlay.create_root_dir(".", source_root)
+        self._source_root = OSFSROOverlay.create_root_dir(".", source_root)
         debian_dir = self._source_root.get("debian")
         if debian_dir is not None and not debian_dir.is_dir:
             debian_dir = None
         self._debian_dir = debian_dir
-        self._diagnostics: Optional[List[types.Diagnostic]] = None
+        self._diagnostics: list[types.Diagnostic] | None = None
         self._last_emitted_diagnostic_count = 0
         dctrl_file = os.path.join(debian_dir_path, "control")
 
@@ -283,15 +279,15 @@ class LSProvidedLintState(LintState):
         return self._doc.uri
 
     @property
-    def doc_version(self) -> Optional[int]:
+    def doc_version(self) -> int | None:
         return self._doc.version
 
     @property
-    def source_root(self) -> Optional[VirtualPathBase]:
+    def source_root(self) -> VirtualPathBase | None:
         return self._source_root
 
     @property
-    def debian_dir(self) -> Optional[VirtualPathBase]:
+    def debian_dir(self) -> VirtualPathBase | None:
         return self._debian_dir
 
     @property
@@ -303,40 +299,40 @@ class LSProvidedLintState(LintState):
         return self._doc.source
 
     @property
-    def lines(self) -> List[str]:
+    def lines(self) -> list[str]:
         return self._lines
 
     @property
     def position_codec(self) -> LintCapablePositionCodec:
         return self._doc.position_codec
 
-    def _resolve_dctrl(self) -> Optional[DctrlFileCache]:
+    def _resolve_dctrl(self) -> DctrlFileCache | None:
         dctrl_cache = self._dctrl_cache
         dctrl_cache.resolve_cache(self._ls)
         return dctrl_cache
 
     @property
-    def parsed_deb822_file_content(self) -> Optional[Deb822FileElement]:
+    def parsed_deb822_file_content(self) -> Deb822FileElement | None:
         cache = self._deb822_file
         cache.resolve_cache(self._ls)
         return cache.deb822_file
 
     @property
-    def source_package(self) -> Optional[SourcePackage]:
+    def source_package(self) -> SourcePackage | None:
         return self._resolve_dctrl().source_package
 
     @property
-    def binary_packages(self) -> Optional[Mapping[str, BinaryPackage]]:
+    def binary_packages(self) -> Mapping[str, BinaryPackage] | None:
         return self._resolve_dctrl().binary_packages
 
-    def _resolve_salsa_ci(self) -> Optional[CommentedMap]:
+    def _resolve_salsa_ci(self) -> CommentedMap | None:
         for salsa_ci_cache in self._salsa_ci_caches:
             if salsa_ci_cache.resolve_cache(self._ls):
                 return salsa_ci_cache.parsed_content
         return None
 
     @property
-    def effective_preference(self) -> Optional[EffectiveFormattingPreference]:
+    def effective_preference(self) -> EffectiveFormattingPreference | None:
         source_package = self.source_package
         salsa_ci = self._resolve_salsa_ci()
         if source_package is None and salsa_ci is None:
@@ -353,12 +349,12 @@ class LSProvidedLintState(LintState):
         return self._ls.maint_preferences
 
     @property
-    def salsa_ci(self) -> Optional[CommentedMap]:
+    def salsa_ci(self) -> CommentedMap | None:
         return None
 
     @property
     def dh_sequencer_data(self) -> DhSequencerData:
-        dh_sequences: Set[str] = set()
+        dh_sequences: set[str] = set()
         saw_dh = False
         src_pkg = self.source_package
         drules_cache = self._drules_cache
@@ -406,7 +402,7 @@ class LSProvidedLintState(LintState):
     async def run_diagnostics(
         self,
         linter: AsyncLinterImpl,
-    ) -> List[types.Diagnostic]:
+    ) -> list[types.Diagnostic]:
         if self._diagnostics is not None:
             raise RuntimeError(
                 "run_diagnostics cannot be run while it is already running"
@@ -445,7 +441,7 @@ class LSProvidedLintState(LintState):
 
 
 def _preference(
-    client_preference: Optional[List[types.MarkupKind]],
+    client_preference: list[types.MarkupKind] | None,
     options: Container[types.MarkupKind],
     fallback_kind: types.MarkupKind,
 ) -> types.MarkupKind:
@@ -465,20 +461,20 @@ class DebputyLanguageServer(LanguageServer):
         **kwargs: Any,
     ) -> None:
         super().__init__(*args, **kwargs)
-        self._dctrl_parser: Optional[DctrlParser] = None
-        self._plugin_feature_set: Optional[PluginProvidedFeatureSet] = None
-        self._trust_language_ids: Optional[bool] = None
+        self._dctrl_parser: DctrlParser | None = None
+        self._plugin_feature_set: PluginProvidedFeatureSet | None = None
+        self._trust_language_ids: bool | None = None
         self._finished_initialization = False
         self.maint_preferences = MaintainerPreferenceTable({}, {})
         self.apt_cache = AptCache()
         self.background_tasks = set()
-        self.client_locale: Optional[str] = None
-        self.forced_locale: Optional[str] = None
-        self._active_locale: Optional[List[str]] = None
-        self._diagnostic_reports: Dict[str, DiagnosticReport] = {}
+        self.client_locale: str | None = None
+        self.forced_locale: str | None = None
+        self._active_locale: list[str] | None = None
+        self._diagnostic_reports: dict[str, DiagnosticReport] = {}
         self.workspace_text_edit_support = WorkspaceTextEditSupport()
         self.debputy_config = load_debputy_config()
-        self._file_state_caches: Dict[str, Dict[Type[FileCache], FileCache]] = (
+        self._file_state_caches: dict[str, dict[type[FileCache], FileCache]] = (
             collections.defaultdict(dict)
         )
         self.hover_output_style = OutputStyle()
@@ -486,6 +482,7 @@ class DebputyLanguageServer(LanguageServer):
     def finish_startup_initialization(self) -> None:
         if self._finished_initialization:
             return
+
         assert self._dctrl_parser is not None
         assert self._plugin_feature_set is not None
         assert self._trust_language_ids is not None
@@ -521,6 +518,30 @@ class DebputyLanguageServer(LanguageServer):
             supported_resource_operation_edit_kinds=self._supported_resource_operation_edit_kinds,
             supports_document_changes=self._supports_ws_document_changes,
         )
+        self._mask_ghost_requests()
+
+    def _mask_ghost_requests(self) -> None:
+        try:
+            if types.TEXT_DOCUMENT_DOCUMENT_HIGHLIGHT not in self.lsp.fm.features:
+
+                # Work around `kate` bug (https://bugs.kde.org/show_bug.cgi?id=506664) for selected requests
+                # that are really annoying (that is, triggers all the time).
+                def _ghost_request(*_args: Any) -> None:
+                    _info(
+                        "Ignoring unsupported request from client that it should not have sent."
+                    )
+
+                self.lsp.fm.features[types.TEXT_DOCUMENT_DOCUMENT_HIGHLIGHT] = (
+                    _ghost_request
+                )
+        except (AttributeError, TypeError, KeyError, ValueError) as e:
+            _info(
+                f"Could install ghost handler, continuing without. Error was: {str(e)}"
+            )
+        else:
+            _info(
+                f"Injecting fake {types.TEXT_DOCUMENT_DOCUMENT_HIGHLIGHT} handler to work around bugs like KDE#506664"
+            )
 
     def _update_locale(self) -> None:
         if self.forced_locale is not None:
@@ -534,7 +555,7 @@ class DebputyLanguageServer(LanguageServer):
         self,
         uri: str,
         path: str,
-        cache_type: Type[T],
+        cache_type: type[T],
         initializer: Callable[[str, str], T],
     ) -> T:
         inner_cache = self._file_state_caches.get(path)
@@ -630,7 +651,7 @@ class DebputyLanguageServer(LanguageServer):
         return LSProvidedLintState(self, doc, source_root, dir_path, self.dctrl_parser)
 
     @property
-    def _client_hover_markup_formats(self) -> Optional[List[types.MarkupKind]]:
+    def _client_hover_markup_formats(self) -> list[types.MarkupKind] | None:
         try:
             return (
                 self.client_capabilities.text_document.hover.content_format
@@ -657,7 +678,7 @@ class DebputyLanguageServer(LanguageServer):
     @property
     def _client_completion_item_document_markup_formats(
         self,
-    ) -> Optional[List[types.MarkupKind]]:
+    ) -> list[types.MarkupKind] | None:
         try:
             return (
                 self.client_capabilities.text_document.completion.completion_item.documentation_format  # type : ignore
@@ -716,7 +737,7 @@ class DebputyLanguageServer(LanguageServer):
     def determine_language_id(
         self,
         doc: "TextDocument",
-    ) -> Tuple[Literal["editor-provided", "filename"], str, str]:
+    ) -> tuple[Literal["editor-provided", "filename"], str, str]:
         lang_id = doc.language_id
         path = doc.path
         try:
@@ -762,7 +783,7 @@ class DebputyLanguageServer(LanguageServer):
         self,
         doc_uri: str,
         version: int,
-        diagnostics: List[types.Diagnostic],
+        diagnostics: list[types.Diagnostic],
         is_partial: bool,
     ) -> None:
         self._diagnostic_reports[doc_uri] = DiagnosticReport(
@@ -778,7 +799,7 @@ class DebputyLanguageServer(LanguageServer):
         self,
         uri: str,
         text_range: types.Range,
-    ) -> Optional[List[types.Diagnostic]]:
+    ) -> list[types.Diagnostic] | None:
         report = self._diagnostic_reports.get(uri)
         if report is None or report.is_in_progress:
             return None
