@@ -1,30 +1,30 @@
 import inspect
 import os.path
 from typing import (
-    Sequence,
     Union,
     Optional,
     TypeVar,
     List,
     TYPE_CHECKING,
-    Callable,
     Literal,
 )
+from collections.abc import Sequence, Callable
 
 from debputy import __version__
 from debputy.linting.lint_util import AbortTaskError
 from debputy.lsp.lsp_features import (
-    DIAGNOSTIC_HANDLERS,
+    LSP_DIAGNOSTIC_HANDLERS,
     COMPLETER_HANDLERS,
     HOVER_HANDLERS,
     SEMANTIC_TOKENS_FULL_HANDLERS,
     CODE_ACTION_HANDLERS,
     SEMANTIC_TOKENS_LEGEND,
     WILL_SAVE_WAIT_UNTIL_HANDLERS,
-    FORMAT_FILE_HANDLERS,
+    LSP_FORMAT_FILE_HANDLERS,
     C,
     TEXT_DOC_INLAY_HANDLERS,
     HandlerDispatchTable,
+    DOCUMENT_LINK_HANDLERS,
 )
 from debputy.util import _info, _trace_log
 
@@ -98,7 +98,7 @@ def _close_document(
 
 async def _open_or_changed_document(
     ls: "DebputyLanguageServer",
-    params: Union[types.DidOpenTextDocumentParams, types.DidChangeTextDocumentParams],
+    params: types.DidOpenTextDocumentParams | types.DidChangeTextDocumentParams,
     event_name: Literal["Opened", "Changed"],
 ) -> None:
     doc_uri = params.text_document.uri
@@ -117,18 +117,18 @@ async def _diagnostics_for(
     ls: "DebputyLanguageServer",
     doc: "TextDocument",
     expected_version: int,
-    params: Union[
-        types.DidOpenTextDocumentParams,
-        types.DidChangeTextDocumentParams,
-        types.DocumentDiagnosticParams,
-    ],
+    params: (
+        types.DidOpenTextDocumentParams
+        | types.DidChangeTextDocumentParams
+        | types.DocumentDiagnosticParams
+    ),
     event_name: Literal["Opened", "Changed"],
 ) -> None:
     doc_uri = doc.uri
     id_source, language_id, normalized_filename = ls.determine_language_id(doc)
     log_func = _info if event_name == "Opened" else _trace_log
     handler = _resolve_handler(
-        DIAGNOSTIC_HANDLERS,
+        LSP_DIAGNOSTIC_HANDLERS,
         language_id,
         normalized_filename,
         log_func,
@@ -157,7 +157,7 @@ async def _diagnostics_for(
 async def _completions(
     ls: "DebputyLanguageServer",
     params: types.CompletionParams,
-) -> Optional[Union[types.CompletionList, Sequence[types.CompletionItem]]]:
+) -> types.CompletionList | Sequence[types.CompletionItem] | None:
     return await _dispatch_standard_handler(
         ls,
         params.text_document.uri,
@@ -171,7 +171,7 @@ async def _completions(
 async def _hover(
     ls: "DebputyLanguageServer",
     params: types.CompletionParams,
-) -> Optional[types.Hover]:
+) -> types.Hover | None:
     return await _dispatch_standard_handler(
         ls,
         params.text_document.uri,
@@ -185,7 +185,7 @@ async def _hover(
 async def _doc_inlay_hint(
     ls: "DebputyLanguageServer",
     params: types.InlayHintParams,
-) -> Optional[List[types.InlayHint]]:
+) -> list[types.InlayHint] | None:
     return await _dispatch_standard_handler(
         ls,
         params.text_document.uri,
@@ -199,7 +199,7 @@ async def _doc_inlay_hint(
 async def _code_actions(
     ls: "DebputyLanguageServer",
     params: types.CodeActionParams,
-) -> Optional[List[Union[types.Command, types.CodeAction]]]:
+) -> list[types.Command | types.CodeAction] | None:
     return await _dispatch_standard_handler(
         ls,
         params.text_document.uri,
@@ -213,7 +213,7 @@ async def _code_actions(
 async def _folding_ranges(
     ls: "DebputyLanguageServer",
     params: types.FoldingRangeParams,
-) -> Optional[Sequence[types.FoldingRange]]:
+) -> Sequence[types.FoldingRange] | None:
     return await _dispatch_standard_handler(
         ls,
         params.text_document.uri,
@@ -233,7 +233,7 @@ async def _folding_ranges(
 async def semantic_tokens_full(
     ls: "DebputyLanguageServer",
     params: types.SemanticTokensParams,
-) -> Optional[types.SemanticTokens]:
+) -> types.SemanticTokens | None:
     return await _dispatch_standard_handler(
         ls,
         params.text_document.uri,
@@ -247,7 +247,7 @@ async def semantic_tokens_full(
 async def _will_save_wait_until(
     ls: "DebputyLanguageServer",
     params: types.WillSaveTextDocumentParams,
-) -> Optional[Sequence[types.TextEdit]]:
+) -> Sequence[types.TextEdit] | None:
     return await _dispatch_standard_handler(
         ls,
         params.text_document.uri,
@@ -261,23 +261,37 @@ async def _will_save_wait_until(
 async def _format_document(
     ls: "DebputyLanguageServer",
     params: types.WillSaveTextDocumentParams,
-) -> Optional[Sequence[types.TextEdit]]:
+) -> Sequence[types.TextEdit] | None:
     return await _dispatch_standard_handler(
         ls,
         params.text_document.uri,
         params,
-        FORMAT_FILE_HANDLERS,
+        LSP_FORMAT_FILE_HANDLERS,
         "Full document formatting",
     )
 
 
+@DEBPUTY_LANGUAGE_SERVER.feature(types.TEXT_DOCUMENT_DOCUMENT_LINK)
+async def _document_link(
+    ls: "DebputyLanguageServer",
+    params: types.DocumentLinkParams,
+) -> Optional[Sequence[types.DocumentLink]]:
+    return await _dispatch_standard_handler(
+        ls,
+        params.text_document.uri,
+        params,
+        DOCUMENT_LINK_HANDLERS,
+        "Document links request",
+    )
+
+
 async def _dispatch_standard_handler(
     ls: "DebputyLanguageServer",
     doc_uri: str,
     params: P,
     handler_table: HandlerDispatchTable[C],
     request_type: str,
-) -> Optional[R]:
+) -> R | None:
     doc = ls.workspace.get_text_document(doc_uri)
 
     id_source, language_id, normalized_filename = ls.determine_language_id(doc)
@@ -303,7 +317,7 @@ def _resolve_handler(
     language_id: str,
     normalized_filename: str,
     log_func: Callable[[str], None],
-) -> Optional[C]:
+) -> C | None:
     dispatch_table = handler_table.get(language_id) if language_id != "" else None
     log_func(f"resolve_handler({language_id=}, {normalized_filename=})")
     if dispatch_table is None:
