@@ -3,15 +3,13 @@ import textwrap
 from typing import (
     Optional,
     Union,
-    Mapping,
-    Sequence,
-    Callable,
-    Iterable,
     Any,
     Self,
     TYPE_CHECKING,
 )
+from collections.abc import Mapping, Sequence, Callable, Iterable
 
+from debputy.lsp.named_styles import ALL_PUBLIC_NAMED_STYLES
 from debputy.lsp.ref_models.deb822_reference_parse_models import UsageHint
 from debputy.lsp.vendoring._deb822_repro import Deb822ParagraphElement
 
@@ -26,8 +24,8 @@ LSP_DATA_DOMAIN = "debputy-lsp-data"
 
 
 def format_comp_item_synopsis_doc(
-    usage_hint: Optional[UsageHint],
-    synopsis_doc: Optional[str],
+    usage_hint: UsageHint | None,
+    synopsis_doc: str | None,
     is_deprecated: bool,
 ) -> str:
     if is_deprecated:
@@ -48,30 +46,26 @@ def format_comp_item_synopsis_doc(
 @dataclasses.dataclass(slots=True, frozen=True)
 class Keyword:
     value: str
-    synopsis: Optional[str] = None
-    long_description: Optional[str] = None
+    synopsis: str | None = None
+    long_description: str | None = None
     translation_context: str = ""
     is_obsolete: bool = False
-    replaced_by: Optional[str] = None
+    replaced_by: str | None = None
     is_exclusive: bool = False
     """For keywords in fields that allow multiple keywords, the `is_exclusive` can be
     used for keywords that cannot be used with other keywords. As an example, the `all`
     value in `Architecture` of `debian/control` cannot be used with any other architecture.
     """
-    is_alias_of: Optional[str] = None
+    is_alias_of: str | None = None
     is_completion_suggestion: bool = True
-    sort_text: Optional[
-        Union[
-            str,
-            Callable[
-                ["Keyword", "LintState", Sequence[Deb822ParagraphElement], str], str
-            ],
-        ]
-    ] = None
-    usage_hint: Optional[UsageHint] = None
-    can_complete_keyword_in_stanza: Optional[
+    sort_text: None | (
+        str
+        | Callable[["Keyword", "LintState", Sequence[Deb822ParagraphElement], str], str]
+    ) = None
+    usage_hint: UsageHint | None = None
+    can_complete_keyword_in_stanza: None | (
         Callable[[Iterable[Deb822ParagraphElement]], bool]
-    ] = None
+    ) = None
 
     @property
     def is_deprecated(self) -> bool:
@@ -82,7 +76,7 @@ class Keyword:
         lint_state: "LintState",
         stanza_parts: Sequence[Deb822ParagraphElement],
         value_being_completed: str,
-    ) -> Optional[str]:
+    ) -> str | None:
         sort_text = self.sort_text
         if sort_text is None:
             return None
@@ -153,7 +147,7 @@ class Keyword:
         )
 
 
-def allowed_values(*values: Union[str, Keyword]) -> Mapping[str, Keyword]:
+def allowed_values(*values: str | Keyword) -> Mapping[str, Keyword]:
     as_keywords = [k if isinstance(k, Keyword) else Keyword(k) for k in values]
     as_mapping = {k.value: k for k in as_keywords if k.value}
     # Simple bug check
@@ -163,20 +157,7 @@ def allowed_values(*values: Union[str, Keyword]) -> Mapping[str, Keyword]:
 
 # This is the set of styles that `debputy` explicitly supports, which is more narrow than
 # the ones in the config file.
-ALL_PUBLIC_NAMED_STYLES = allowed_values(
-    Keyword(
-        "black",
-        long_description=textwrap.dedent(
-            """\
-            Uncompromising file formatting of Debian packaging files
-
-            By using it, you  agree to cede control over minutiae of hand-formatting. In
-            return, the formatter gives you speed, determinism, and freedom from style
-            discussions about formatting.
-
-            The `black` style is inspired by the `black` Python code formatter. Like with
-            `black`, the style will evolve over time.
-    """
-        ),
-    ),
+ALL_PUBLIC_NAMED_STYLES_AS_KEYWORDS = allowed_values(
+    Keyword(s.name, long_description=s.long_description)
+    for s in ALL_PUBLIC_NAMED_STYLES
 )
