@@ -1,11 +1,9 @@
 import contextlib
 from typing import (
-    Iterator,
-    Optional,
-    Mapping,
     NoReturn,
     TYPE_CHECKING,
 )
+from collections.abc import Iterator, Mapping
 
 from debian.debian_support import DpkgArchTable
 
@@ -18,7 +16,7 @@ from debputy.manifest_parser.util import (
     _ALL_PACKAGE_TYPES,
     resolve_package_type_selectors,
 )
-from debputy.packages import BinaryPackage
+from debputy.packages import BinaryPackage, SourcePackage
 from debputy.plugin.api.impl_types import (
     TP,
     DispatchingTableParser,
@@ -32,6 +30,11 @@ if TYPE_CHECKING:
 
 
 class ParserContextData:
+
+    @property
+    def source_package(self) -> SourcePackage:
+        raise NotImplementedError
+
     @property
     def binary_packages(self) -> Mapping[str, BinaryPackage]:
         raise NotImplementedError
@@ -49,8 +52,8 @@ class ParserContextData:
         attribute_path: AttributePath,
         *,
         package_type: PackageTypeSelector = _ALL_PACKAGE_TYPES,
-        package_attribute: Optional[str] = None,
-    ) -> Optional[BinaryPackage]:
+        package_attribute: str | None = None,
+    ) -> BinaryPackage | None:
         resolved_package_types = resolve_package_type_selectors(package_type)
         possible_matches = [
             p
@@ -134,6 +137,6 @@ class ParserContextData:
         raise NotImplementedError
 
     def resolve_build_environment(
-        self, name: Optional[str], attribute_path: AttributePath
+        self, name: str | None, attribute_path: AttributePath
     ) -> BuildEnvironmentDefinition:
         raise NotImplementedError
