@@ -4,6 +4,7 @@ import functools
 import importlib
 import importlib.resources
 import importlib.util
+import inspect
 import itertools
 import json
 import os
@@ -11,33 +12,25 @@ import re
 import subprocess
 import sys
 from abc import ABC
+from collections.abc import Callable, Iterable, Sequence, Iterator, Mapping, Container
 from importlib.resources.abc import Traversable
+from io import IOBase
 from json import JSONDecodeError
 from pathlib import Path
+from types import NoneType
 from typing import (
-    Optional,
-    Callable,
-    Dict,
-    Tuple,
-    Iterable,
-    Sequence,
-    Type,
-    List,
-    Union,
-    Set,
-    Iterator,
     IO,
-    Mapping,
     AbstractSet,
     cast,
-    FrozenSet,
     Any,
     Literal,
-    Container,
     TYPE_CHECKING,
     is_typeddict,
+    AnyStr,
+    overload,
 )
 
+import debputy
 from debputy import DEBPUTY_DOC_ROOT_DIR
 from debputy.exceptions import (
     DebputySubstitutionError,
@@ -83,10 +76,11 @@ from debputy.plugin.api.impl_types import (
     ServiceManagerDetails,
     KnownPackagingFileInfo,
     PluginProvidedKnownPackagingFile,
-    InstallPatternDHCompatRule,
+    DHCompatibilityBasedRule,
     PluginProvidedTypeMapping,
     PluginProvidedBuildSystemAutoDetection,
     BSR,
+    TP,
 )
 from debputy.plugin.api.plugin_parser import (
     PLUGIN_METADATA_PARSER,
@@ -124,17 +118,19 @@ from debputy.plugin.api.spec import (
     BuildSystemManifestRuleMetadata,
     INTEGRATION_MODE_FULL,
     only_integrations,
+    DebputyPluginDefinition,
 )
 from debputy.plugin.api.std_docs import _STD_ATTR_DOCS
-from debputy.plugin.debputy.to_be_api_types import (
-    BuildRuleParsedFormat,
-    BSPF,
-    debputy_build_system,
-)
 from debputy.plugin.plugin_state import (
     run_in_context_of_plugin,
     run_in_context_of_plugin_wrap_errors,
     wrap_plugin_code,
+    register_manifest_type_value_in_context,
+)
+from debputy.plugins.debputy.to_be_api_types import (
+    BuildRuleParsedFormat,
+    BSPF,
+    debputy_build_system,
 )
 from debputy.substitution import (
     Substitution,
@@ -156,15 +152,16 @@ if TYPE_CHECKING:
     from debputy.highlevel_manifest import HighLevelManifest
 
 PLUGIN_TEST_SUFFIX = re.compile(r"_(?:t|test|check)(?:_([a-z0-9_]+))?[.]py$")
+PLUGIN_PYTHON_RES_PATH = importlib.resources.files(debputy.plugins.__name__)
 
 
 def _validate_known_packaging_file_dh_compat_rules(
-    dh_compat_rules: Optional[List[InstallPatternDHCompatRule]],
+    dh_compat_rules: list[DHCompatibilityBasedRule] | None,
 ) -> None:
     max_compat = None
     if not dh_compat_rules:
         return
-    dh_compat_rule: InstallPatternDHCompatRule
+    dh_compat_rule: DHCompatibilityBasedRule
     for idx, dh_compat_rule in enumerate(dh_compat_rules):
         dh_version = dh_compat_rule.get("starting_with_debhelper_version")
         compat = dh_compat_rule.get("starting_with_compat_level")
@@ -175,7 +172,7 @@ def _validate_known_packaging_file_dh_compat_rules(
         }
         if not remaining:
             raise ValueError(
-                f"The dh compat-rule at index {idx} does not affect anything not have any rules!? So why have it?"
+                f"The dh compat-rule at index {idx} does not affect anything / not have any rules!? So why have it?"
             )
         if dh_version is None and compat is None and idx < len(dh_compat_rules) - 1:
             raise ValueError(
@@ -217,6 +214,7 @@ class DebputyPluginInitializerProvider(DebputyPluginInitializer):
         "_unloaders",
         "_is_doc_cache_resolved",
         "_doc_cache",
+        "_registered_manifest_types",
         "_load_started",
     )
 
@@ -228,13 +226,18 @@ class DebputyPluginInitializerProvider(DebputyPluginInitializer):
     ) -> None:
         self._plugin_metadata: DebputyPluginMetadata = plugin_metadata
         self._feature_set = feature_set
-        self._plugin_detector_ids: Set[str] = set()
+        self._plugin_detector_ids: set[str] = set()
         self._substitution = substitution
-        self._unloaders: List[Callable[[], None]] = []
+        self._unloaders: list[Callable[[], None]] = []
         self._is_doc_cache_resolved: bool = False
-        self._doc_cache: Optional[DebputyParsedDoc] = None
+        self._doc_cache: DebputyParsedDoc | None = None
+        self._registered_manifest_types: dict[type[Any], DebputyPluginMetadata] = {}
         self._load_started = False
 
+    @property
+    def plugin_metadata(self) -> DebputyPluginMetadata:
+        return self._plugin_metadata
+
     def unload_plugin(self) -> None:
         if self._load_started:
             for unloader in self._unloaders:
@@ -273,7 +276,7 @@ class DebputyPluginInitializerProvider(DebputyPluginInitializer):
                 f"Exception while attempting to load plugin {metadata.plugin_name}"
             ) from e
 
-    def _resolve_docs(self) -> Optional[DebputyParsedDoc]:
+    def _resolve_docs(self) -> DebputyParsedDoc | None:
         doc_cache = self._doc_cache
         if doc_cache is not None:
             return doc_cache
@@ -311,11 +314,11 @@ class DebputyPluginInitializerProvider(DebputyPluginInitializer):
 
     def _pluggable_manifest_docs_for(
         self,
-        rule_type: Union[TTP, str],
-        rule_name: Union[str, List[str]],
+        rule_type: TTP | str,
+        rule_name: str | list[str],
         *,
-        inline_reference_documentation: Optional[ParserDocumentation] = None,
-    ) -> Optional[ParserDocumentation]:
+        inline_reference_documentation: ParserDocumentation | None = None,
+    ) -> ParserDocumentation | None:
         ref_data = self._resolve_docs()
         if ref_data is not None:
             primary_rule_name = (
@@ -339,18 +342,18 @@ class DebputyPluginInitializerProvider(DebputyPluginInitializer):
         installed_path: str,
         *,
         default_mode: int = 0o0644,
-        default_priority: Optional[int] = None,
+        default_priority: int | None = None,
         allow_name_segment: bool = True,
         allow_architecture_segment: bool = False,
-        post_formatting_rewrite: Optional[Callable[[str], str]] = None,
+        post_formatting_rewrite: Callable[[str], str] | None = None,
         packageless_is_fallback_for_all_packages: bool = False,
         reservation_only: bool = False,
-        format_callback: Optional[
+        format_callback: None | (
             Callable[[str, PPFFormatParam, VirtualPath], str]
-        ] = None,
-        reference_documentation: Optional[
+        ) = None,
+        reference_documentation: None | (
             PackagerProvidedFileReferenceDocumentation
-        ] = None,
+        ) = None,
     ) -> None:
         packager_provided_files = self._feature_set.packager_provided_files
         existing = packager_provided_files.get(stem)
@@ -508,12 +511,12 @@ class DebputyPluginInitializerProvider(DebputyPluginInitializer):
     def manifest_variable_provider(
         self,
         provider: Callable[[VariableContext], Mapping[str, str]],
-        variables: Union[Sequence[str], Mapping[str, Optional[str]]],
+        variables: Sequence[str] | Mapping[str, str | None],
     ) -> None:
         self._restricted_api()
         cached_provider = functools.lru_cache(None)(provider)
         permitted_variables = frozenset(variables)
-        variables_iter: Iterable[Tuple[str, Optional[str]]]
+        variables_iter: Iterable[tuple[str, str | None]]
         if not isinstance(variables, Mapping):
             variables_iter = zip(variables, itertools.repeat(None))
         else:
@@ -686,10 +689,10 @@ class DebputyPluginInitializerProvider(DebputyPluginInitializer):
         name: str,
         should_discard: Callable[[VirtualPath], bool],
         *,
-        rule_reference_documentation: Optional[str] = None,
-        examples: Union[
-            AutomaticDiscardRuleExample, Sequence[AutomaticDiscardRuleExample]
-        ] = tuple(),
+        rule_reference_documentation: str | None = None,
+        examples: (
+            AutomaticDiscardRuleExample | Sequence[AutomaticDiscardRuleExample]
+        ) = tuple(),
     ) -> None:
         """Register an automatic discard rule
 
@@ -814,7 +817,8 @@ class DebputyPluginInitializerProvider(DebputyPluginInitializer):
         self,
         variable_name: str,
         value: str,
-        variable_reference_documentation: Optional[str] = None,
+        *,
+        variable_reference_documentation: str | None = None,
     ) -> None:
         self._check_variable_name(variable_name)
         manifest_variables = self._feature_set.manifest_variables
@@ -855,10 +859,10 @@ class DebputyPluginInitializerProvider(DebputyPluginInitializer):
     def provide_manifest_keyword(
         self,
         rule_type: TTP,
-        rule_name: Union[str, List[str]],
+        rule_name: str | list[str],
         handler: DIPKWHandler,
         *,
-        inline_reference_documentation: Optional[ParserDocumentation] = None,
+        inline_reference_documentation: ParserDocumentation | None = None,
     ) -> None:
         self._restricted_api()
         parser_generator = self._feature_set.manifest_parser_generator
@@ -896,13 +900,13 @@ class DebputyPluginInitializerProvider(DebputyPluginInitializer):
         rule_type: str,
         rule_name: str,
         *,
-        object_parser_key: Optional[str] = None,
-        on_end_parse_step: Optional[
+        object_parser_key: str | None = None,
+        on_end_parse_step: None | (
             Callable[
-                [str, Optional[Mapping[str, Any]], AttributePath, ParserContextData],
+                [str, Mapping[str, Any] | None, AttributePath, ParserContextData],
                 None,
             ]
-        ] = None,
+        ) = None,
         nested_in_package_context: bool = False,
     ) -> None:
         self._restricted_api()
@@ -944,17 +948,18 @@ class DebputyPluginInitializerProvider(DebputyPluginInitializer):
 
     def pluggable_manifest_rule(
         self,
-        rule_type: Union[TTP, str],
-        rule_name: Union[str, Sequence[str]],
-        parsed_format: Type[PF],
+        rule_type: TTP | str,
+        rule_name: str | Sequence[str],
+        parsed_format: type[PF],
         handler: DIPHandler,
         *,
-        source_format: Optional[SF] = None,
-        inline_reference_documentation: Optional[ParserDocumentation] = None,
-        expected_debputy_integration_mode: Optional[
+        source_format: SF | None = None,
+        inline_reference_documentation: ParserDocumentation | None = None,
+        expected_debputy_integration_mode: None | (
             Container[DebputyIntegrationMode]
-        ] = None,
+        ) = None,
         apply_standard_attribute_documentation: bool = False,
+        register_value: bool = True,
     ) -> None:
         # When changing this, consider which types will be unrestricted
         self._restricted_api()
@@ -972,7 +977,18 @@ class DebputyPluginInitializerProvider(DebputyPluginInitializer):
                     f"The rule_type was not a supported type. It must be one of {types}"
                 )
             dispatching_parser = parser_generator.dispatchable_object_parsers[rule_type]
+            signature = inspect.signature(handler)
+            if (
+                signature.return_annotation is signature.empty
+                or signature.return_annotation == NoneType
+            ):
+                raise ValueError(
+                    "The handler must have a return type (that is not None)"
+                )
+            register_as_type = signature.return_annotation
         else:
+            # Dispatchable types cannot be resolved
+            register_as_type = None
             if rule_type not in parser_generator.dispatchable_table_parsers:
                 types = ", ".join(
                     sorted(
@@ -984,6 +1000,19 @@ class DebputyPluginInitializerProvider(DebputyPluginInitializer):
                 )
             dispatching_parser = parser_generator.dispatchable_table_parsers[rule_type]
 
+        if register_as_type is not None and not register_value:
+            register_as_type = None
+
+        if register_as_type is not None:
+            existing_registration = self._registered_manifest_types.get(
+                register_as_type
+            )
+            if existing_registration is not None:
+                raise ValueError(
+                    f"Cannot register rule {rule_name!r} for plugin {self._plugin_name}. The plugin {existing_registration.plugin_name} already registered a manifest rule with type {register_as_type!r}"
+                )
+            self._registered_manifest_types[register_as_type] = self._plugin_metadata
+
         inline_reference_documentation = self._pluggable_manifest_docs_for(
             rule_type,
             rule_name,
@@ -1002,10 +1031,22 @@ class DebputyPluginInitializerProvider(DebputyPluginInitializer):
             expected_debputy_integration_mode=expected_debputy_integration_mode,
             automatic_docs=docs,
         )
+
+        def _registering_handler(
+            name: str,
+            parsed_data: PF,
+            attribute_path: AttributePath,
+            parser_context: ParserContextData,
+        ) -> TP:
+            value = handler(name, parsed_data, attribute_path, parser_context)
+            if register_as_type is not None:
+                register_manifest_type_value_in_context(register_as_type, value)
+            return value
+
         dispatching_parser.register_parser(
             rule_name,
             parser,
-            wrap_plugin_code(self._plugin_name, handler),
+            wrap_plugin_code(self._plugin_name, _registering_handler),
             self._plugin_metadata,
         )
 
@@ -1065,10 +1106,10 @@ class DebputyPluginInitializerProvider(DebputyPluginInitializer):
         manifest_keyword: str,
         rule_type: type[BSR],
         *,
-        shadowing_build_systems_when_active: FrozenSet[str] = frozenset(),
-        constructor: Optional[
+        shadowing_build_systems_when_active: frozenset[str] = frozenset(),
+        constructor: None | (
             Callable[[BuildRuleParsedFormat, AttributePath, "HighLevelManifest"], BSR]
-        ] = None,
+        ) = None,
     ) -> None:
         self._restricted_api()
         feature_set = self._feature_set
@@ -1131,13 +1172,17 @@ class DebputyPluginInitializerProvider(DebputyPluginInitializer):
         path = packaging_file_details.get("path")
         dhpkgfile = packaging_file_details.get("pkgfile")
 
-        packaging_file_details: KnownPackagingFileInfo = packaging_file_details.copy()
+        packaging_file_details = packaging_file_details.copy()
 
         if detection_method == "path":
             if dhpkgfile is not None:
                 raise ValueError(
                     'The "pkgfile" attribute cannot be used when detection-method is "path" (or omitted)'
                 )
+            if path is None:
+                raise ValueError(
+                    'The "path" attribute  must be present when detection-method is "path" (or omitted)'
+                )
             if path != _normalize_path(path, with_prefix=False):
                 raise ValueError(
                     f"The path for known packaging files must be normalized. Please replace"
@@ -1150,6 +1195,10 @@ class DebputyPluginInitializerProvider(DebputyPluginInitializer):
                 raise ValueError(
                     'The "path" attribute cannot be used when detection-method is "dh.pkgfile"'
                 )
+            if dhpkgfile is None:
+                raise ValueError(
+                    'The "pkgfile" attribute must be present when detection-method is "dh.pkgfile"'
+                )
             if "/" in dhpkgfile:
                 raise ValueError(
                     'The "pkgfile" attribute ḿust be a name stem such as "install" (no "/" are allowed)'
@@ -1190,7 +1239,7 @@ class DebputyPluginInitializerProvider(DebputyPluginInitializer):
         self,
         type_mapping: TypeMapping,
         *,
-        reference_documentation: Optional[TypeMappingDocumentation] = None,
+        reference_documentation: TypeMappingDocumentation | None = None,
     ) -> None:
         self._restricted_api()
         target_type = type_mapping.target_type
@@ -1220,7 +1269,7 @@ class DebputyPluginInitializerProvider(DebputyPluginInitializer):
     def _restricted_api(
         self,
         *,
-        allowed_plugins: Union[Set[str], FrozenSet[str]] = frozenset(),
+        allowed_plugins: set[str] | frozenset[str] = frozenset(),
     ) -> None:
         if self._plugin_name != "debputy" and self._plugin_name not in allowed_plugins:
             raise PluginAPIViolationError(
@@ -1250,7 +1299,7 @@ class MaintscriptAccessorProviderBase(MaintscriptAccessor, ABC):
         condition: str,
         run_snippet: str,
         /,
-        indent: Optional[bool] = None,
+        indent: bool | None = None,
     ) -> str:
         if indent is None:
             # We auto-determine this based on heredocs currently
@@ -1268,7 +1317,7 @@ class MaintscriptAccessorProviderBase(MaintscriptAccessor, ABC):
         self,
         run_snippet: str,
         /,
-        indent: Optional[bool] = None,
+        indent: bool | None = None,
         perform_substitution: bool = True,
         skip_on_rollback: bool = False,
     ) -> None:
@@ -1286,7 +1335,7 @@ class MaintscriptAccessorProviderBase(MaintscriptAccessor, ABC):
         self,
         run_snippet: str,
         /,
-        indent: Optional[bool] = None,
+        indent: bool | None = None,
         perform_substitution: bool = True,
     ) -> None:
         condition = '[ "$1" = "configure" -a -z "$2" ]'
@@ -1301,7 +1350,7 @@ class MaintscriptAccessorProviderBase(MaintscriptAccessor, ABC):
         self,
         run_snippet: str,
         /,
-        indent: Optional[bool] = None,
+        indent: bool | None = None,
         perform_substitution: bool = True,
     ) -> None:
         condition = '[ "$1" = "configure" -a -n "$2" ]'
@@ -1317,7 +1366,7 @@ class MaintscriptAccessorProviderBase(MaintscriptAccessor, ABC):
         version: str,
         run_snippet: str,
         /,
-        indent: Optional[bool] = None,
+        indent: bool | None = None,
         perform_substitution: bool = True,
     ) -> None:
         condition = '[ "$1" = "configure" ] && dpkg --compare-versions le-nl "$2"'
@@ -1332,7 +1381,7 @@ class MaintscriptAccessorProviderBase(MaintscriptAccessor, ABC):
         self,
         run_snippet: str,
         /,
-        indent: Optional[bool] = None,
+        indent: bool | None = None,
         perform_substitution: bool = True,
     ) -> None:
         condition = '[ "$1" = "remove" ]'
@@ -1347,7 +1396,7 @@ class MaintscriptAccessorProviderBase(MaintscriptAccessor, ABC):
         self,
         run_snippet: str,
         /,
-        indent: Optional[bool] = None,
+        indent: bool | None = None,
         perform_substitution: bool = True,
     ) -> None:
         condition = '[ "$1" = "remove" ]'
@@ -1362,7 +1411,7 @@ class MaintscriptAccessorProviderBase(MaintscriptAccessor, ABC):
         self,
         run_snippet: str,
         /,
-        indent: Optional[bool] = None,
+        indent: bool | None = None,
         perform_substitution: bool = True,
     ) -> None:
         condition = '[ "$1" = "purge" ]'
@@ -1406,10 +1455,10 @@ class MaintscriptAccessorProvider(MaintscriptAccessorProviderBase):
         self,
         plugin_metadata: DebputyPluginMetadata,
         plugin_source_id: str,
-        maintscript_snippets: Dict[str, MaintscriptSnippetContainer],
+        maintscript_snippets: dict[str, MaintscriptSnippetContainer],
         package_substitution: Substitution,
         *,
-        default_snippet_order: Optional[Literal["service"]] = None,
+        default_snippet_order: Literal["service"] | None = None,
     ):
         self._plugin_metadata = plugin_metadata
         self._plugin_source_id = plugin_source_id
@@ -1453,16 +1502,16 @@ class BinaryCtrlAccessorProviderBase(BinaryCtrlAccessor):
         plugin_metadata: DebputyPluginMetadata,
         plugin_source_id: str,
         package_metadata_context: PackageProcessingContext,
-        triggers: Dict[Tuple[DpkgTriggerType, str], PluginProvidedTrigger],
+        triggers: dict[tuple[DpkgTriggerType, str], PluginProvidedTrigger],
         substvars: FlushableSubstvars,
-        shlibs_details: Tuple[Optional[str], Optional[List[str]]],
+        shlibs_details: tuple[str | None, list[str] | None],
     ) -> None:
         self._plugin_metadata = plugin_metadata
         self._plugin_source_id = plugin_source_id
         self._package_metadata_context = package_metadata_context
         self._triggers = triggers
         self._substvars = substvars
-        self._maintscript: Optional[MaintscriptAccessor] = None
+        self._maintscript: MaintscriptAccessor | None = None
         self._shlibs_details = shlibs_details
 
     def _create_maintscript_accessor(self) -> MaintscriptAccessor:
@@ -1533,13 +1582,13 @@ class BinaryCtrlAccessorProvider(BinaryCtrlAccessorProviderBase):
         plugin_metadata: DebputyPluginMetadata,
         plugin_source_id: str,
         package_metadata_context: PackageProcessingContext,
-        triggers: Dict[Tuple[DpkgTriggerType, str], PluginProvidedTrigger],
+        triggers: dict[tuple[DpkgTriggerType, str], PluginProvidedTrigger],
         substvars: FlushableSubstvars,
-        maintscript_snippets: Dict[str, MaintscriptSnippetContainer],
+        maintscript_snippets: dict[str, MaintscriptSnippetContainer],
         package_substitution: Substitution,
-        shlibs_details: Tuple[Optional[str], Optional[List[str]]],
+        shlibs_details: tuple[str | None, list[str] | None],
         *,
-        default_snippet_order: Optional[Literal["service"]] = None,
+        default_snippet_order: Literal["service"] | None = None,
     ) -> None:
         super().__init__(
             plugin_metadata,
@@ -1573,22 +1622,22 @@ class BinaryCtrlAccessorProviderCreator:
         self,
         package_metadata_context: PackageProcessingContext,
         substvars: FlushableSubstvars,
-        maintscript_snippets: Dict[str, MaintscriptSnippetContainer],
+        maintscript_snippets: dict[str, MaintscriptSnippetContainer],
         substitution: Substitution,
     ) -> None:
         self._package_metadata_context = package_metadata_context
         self._substvars = substvars
         self._maintscript_snippets = maintscript_snippets
         self._substitution = substitution
-        self._triggers: Dict[Tuple[DpkgTriggerType, str], PluginProvidedTrigger] = {}
-        self.shlibs_details: Tuple[Optional[str], Optional[List[str]]] = None, None
+        self._triggers: dict[tuple[DpkgTriggerType, str], PluginProvidedTrigger] = {}
+        self.shlibs_details: tuple[str | None, list[str] | None] = None, None
 
     def for_plugin(
         self,
         plugin_metadata: DebputyPluginMetadata,
         plugin_source_id: str,
         *,
-        default_snippet_order: Optional[Literal["service"]] = None,
+        default_snippet_order: Literal["service"] | None = None,
     ) -> BinaryCtrlAccessor:
         return BinaryCtrlAccessorProvider(
             plugin_metadata,
@@ -1608,8 +1657,8 @@ class BinaryCtrlAccessorProviderCreator:
 
 def _resolve_bundled_plugin_docs_path(
     plugin_name: str,
-    loader: Optional[PluginInitializationEntryPoint],
-) -> Optional[Union[Traversable, Path]]:
+    loader: PluginInitializationEntryPoint | None,
+) -> Traversable | Path | None:
     plugin_module = getattr(loader, "__module__")
     assert plugin_module is not None
     plugin_package_name = sys.modules[plugin_module].__package__
@@ -1619,10 +1668,12 @@ def _resolve_bundled_plugin_docs_path(
 
 
 def plugin_metadata_for_debputys_own_plugin(
-    loader: Optional[PluginInitializationEntryPoint] = None,
+    loader: PluginInitializationEntryPoint | None = None,
 ) -> DebputyPluginMetadata:
     if loader is None:
-        from debputy.plugin.debputy.debputy_plugin import initialize_debputy_features
+        from debputy.plugins.debputy.debputy_plugin import (
+            initialize_debputy_features,
+        )
 
         loader = initialize_debputy_features
     plugin_name = "debputy"
@@ -1642,9 +1693,9 @@ def plugin_metadata_for_debputys_own_plugin(
 def load_plugin_features(
     plugin_search_dirs: Sequence[str],
     substitution: Substitution,
-    requested_plugins_only: Optional[Sequence[str]] = None,
-    required_plugins: Optional[Set[str]] = None,
-    plugin_feature_set: Optional[PluginProvidedFeatureSet] = None,
+    requested_plugins_only: Sequence[str] | None = None,
+    required_plugins: set[str] | None = None,
+    plugin_feature_set: PluginProvidedFeatureSet | None = None,
     debug_mode: bool = False,
 ) -> PluginProvidedFeatureSet:
     if plugin_feature_set is None:
@@ -1720,13 +1771,21 @@ def find_json_plugin(
 
 def find_related_implementation_files_for_plugin(
     plugin_metadata: DebputyPluginMetadata,
-) -> List[str]:
+) -> list[str]:
     if plugin_metadata.is_bundled:
         plugin_name = plugin_metadata.plugin_name
         _error(
             f"Cannot run find related files for {plugin_name}: The plugin seems to be bundled"
             " or loaded via a mechanism that does not support detecting its tests."
         )
+
+    if plugin_metadata.is_from_python_path:
+        plugin_name = plugin_metadata.plugin_name
+        # Maybe they could be, but that is for another day.
+        _error(
+            f"Cannot run find related files for {plugin_name}: The plugin is installed into python path"
+            " and these are not supported."
+        )
     files = []
     module_name, module_file = _find_plugin_implementation_file(
         plugin_metadata.plugin_name,
@@ -1750,7 +1809,7 @@ def find_related_implementation_files_for_plugin(
 
 def find_tests_for_plugin(
     plugin_metadata: DebputyPluginMetadata,
-) -> List[str]:
+) -> list[str]:
     plugin_name = plugin_metadata.plugin_name
     plugin_path = plugin_metadata.plugin_path
 
@@ -1760,6 +1819,14 @@ def find_tests_for_plugin(
             " mechanism that does not support detecting its tests."
         )
 
+    if plugin_metadata.is_from_python_path:
+        plugin_name = plugin_metadata.plugin_name
+        # Maybe they could be, but that is for another day.
+        _error(
+            f"Cannot run find related files for {plugin_name}: The plugin is installed into python path"
+            " and these are not supported."
+        )
+
     plugin_dir = os.path.dirname(plugin_path)
     test_basename_prefix = plugin_metadata.plugin_name.replace("-", "_")
     tests = []
@@ -1779,7 +1846,6 @@ def find_json_plugins(
     requested_plugins: Iterable[str],
 ) -> Iterable[DebputyPluginMetadata]:
     for plugin_name_or_path in requested_plugins:
-        found = False
         if "/" in plugin_name_or_path:
             if not os.path.isfile(plugin_name_or_path):
                 raise PluginNotFoundError(
@@ -1795,14 +1861,25 @@ def find_json_plugins(
             )
             if not os.path.isfile(path):
                 continue
-            found = True
             yield parse_json_plugin_desc(path)
-        if not found:
-            search_dir_str = ":".join(search_dirs)
-            raise PluginNotFoundError(
-                f"Unable to load the plugin {plugin_name_or_path}: Could not find {plugin_name_or_path}.json in the"
-                f" debputy/plugins subdir of any of the search dirs ({search_dir_str})"
-            )
+            return
+
+        path_root = PLUGIN_PYTHON_RES_PATH
+        pp_path = path_root.joinpath(f"{plugin_name_or_path}.json")
+        if pp_path or pp_path.is_file():
+            with pp_path.open() as fd:
+                yield parse_json_plugin_desc(
+                    f"PYTHONPATH:debputy/plugins/{pp_path.name}",
+                    fd=fd,
+                    is_from_python_path=True,
+                )
+            return
+
+        search_dir_str = ":".join(search_dirs)
+        raise PluginNotFoundError(
+            f"Unable to load the plugin {plugin_name_or_path}: Could not find {plugin_name_or_path}.json in the"
+            f" debputy/plugins subdir of any of the search dirs ({search_dir_str})"
+        )
 
 
 def _find_all_json_plugins(
@@ -1825,6 +1902,7 @@ def _find_all_json_plugins(
                     or entry.name in seen
                 ):
                     continue
+                seen.add(entry.name)
                 try:
                     plugin_metadata = parse_json_plugin_desc(entry.path)
                 except PluginBaseError as e:
@@ -1842,13 +1920,28 @@ def _find_all_json_plugins(
                 else:
                     yield plugin_metadata
 
+    for pp_entry in PLUGIN_PYTHON_RES_PATH.iterdir():
+        if (
+            not pp_entry.name.endswith(".json")
+            or not pp_entry.is_file()
+            or pp_entry.name in seen
+        ):
+            continue
+        seen.add(pp_entry.name)
+        with pp_entry.open() as fd:
+            yield parse_json_plugin_desc(
+                f"PYTHONPATH:debputy/plugins/{pp_entry.name}",
+                fd=fd,
+                is_from_python_path=True,
+            )
+
 
 def _find_plugin_implementation_file(
     plugin_name: str,
     json_file_path: str,
-) -> Tuple[str, str]:
+) -> tuple[str, str]:
     guessed_module_basename = plugin_name.replace("-", "_")
-    module_name = f"debputy.plugin.{guessed_module_basename}"
+    module_name = f"debputy.plugins.{guessed_module_basename}"
     module_fs_path = os.path.join(
         os.path.dirname(json_file_path), f"{guessed_module_basename}.py"
     )
@@ -1858,7 +1951,7 @@ def _find_plugin_implementation_file(
 def _resolve_module_initializer(
     plugin_name: str,
     plugin_initializer_name: str,
-    module_name: Optional[str],
+    module_name: str | None,
     json_file_path: str,
 ) -> PluginInitializationEntryPoint:
     module = None
@@ -1915,13 +2008,25 @@ def _resolve_module_initializer(
         getattr,
         module,
         plugin_initializer_name,
+        None,
     )
 
     if plugin_initializer is None:
         raise PluginMetadataError(
             f'The plugin defined in {json_file_path} claimed that module "{module_name}" would have an'
-            f" attribute called {plugin_initializer}. However, it does not. Please correct the plugin"
-            f" metadata or initializer name in the Python module."
+            f' attribute called "{plugin_initializer_name}" to initialize the plugin. However, that attribute'
+            " does not exist or cannot be resolved. Please correct the plugin metadata or initializer name"
+            " in the Python module."
+        )
+    if isinstance(plugin_initializer, DebputyPluginDefinition):
+        return plugin_initializer.initialize
+    if not callable(plugin_initializer):
+        raise PluginMetadataError(
+            f'The plugin defined in {json_file_path} claimed that module "{module_name}" would have an'
+            f' attribute called "{plugin_initializer_name}" for initializing the plugin. While that'
+            " attribute exists, it is neither a `DebputyPluginDefinition`"
+            " (`plugin_definition = define_debputy_plugin()`) nor is it `callable`"
+            " (`def initialize(api: DebputyPluginInitializer) -> None:`)."
         )
     return cast("PluginInitializationEntryPoint", plugin_initializer)
 
@@ -2076,7 +2181,7 @@ def _json_plugin_loader(
 
     if not plugin_initializers:
         raise PluginMetadataError(
-            f"The plugin defined in {json_file_path} does not seem to provide features, "
+            f"The plugin defined in {json_file_path} does not seem to provide features,"
             f" such as module + plugin-initializer or packager-provided-files."
         )
 
@@ -2090,8 +2195,23 @@ def _json_plugin_loader(
     return _chain_loader
 
 
+@overload
 @contextlib.contextmanager
-def _open(path: str, fd: Optional[IO[bytes]] = None) -> Iterator[IO[bytes]]:
+def _open(
+    path: str,
+    fd: IO[AnyStr] | IOBase = ...,
+) -> Iterator[IO[AnyStr] | IOBase]: ...
+
+
+@overload
+@contextlib.contextmanager
+def _open(path: str, fd: None = None) -> Iterator[IO[bytes]]: ...
+
+
+@contextlib.contextmanager
+def _open(
+    path: str, fd: IO[AnyStr] | IOBase | None = None
+) -> Iterator[IO[AnyStr] | IOBase]:
     if fd is not None:
         yield fd
     else:
@@ -2102,7 +2222,7 @@ def _open(path: str, fd: Optional[IO[bytes]] = None) -> Iterator[IO[bytes]]:
 def _resolve_json_plugin_docs_path(
     plugin_name: str,
     plugin_path: str,
-) -> Optional[Union[Traversable, Path]]:
+) -> Traversable | Path | None:
     plugin_dir = os.path.dirname(plugin_path)
     return Path(os.path.join(plugin_dir, plugin_name + "_docs.yaml"))
 
@@ -2110,7 +2230,8 @@ def _resolve_json_plugin_docs_path(
 def parse_json_plugin_desc(
     path: str,
     *,
-    fd: Optional[IO[bytes]] = None,
+    fd: IO[AnyStr] | IOBase | None = None,
+    is_from_python_path: bool = False,
 ) -> DebputyPluginMetadata:
     with _open(path, fd=fd) as rfd:
         try:
@@ -2159,6 +2280,7 @@ def parse_json_plugin_desc(
         ),
         plugin_initializer=None,
         plugin_path=path,
+        is_from_python_path=is_from_python_path,
     )
 
 
@@ -2174,7 +2296,7 @@ class ServiceDefinitionImpl(ServiceDefinition[DSD]):
     on_upgrade: ServiceUpgradeRule
     definition_source: str
     is_plugin_provided_definition: bool
-    service_context: Optional[DSD]
+    service_context: DSD | None
 
     def replace(self, **changes: Any) -> "ServiceDefinitionImpl[DSD]":
         return dataclasses.replace(self, **changes)
@@ -2185,8 +2307,8 @@ class ServiceRegistryImpl(ServiceRegistry[DSD]):
 
     def __init__(self, service_manager_details: ServiceManagerDetails) -> None:
         self._service_manager_details = service_manager_details
-        self._service_definitions: List[ServiceDefinition[DSD]] = []
-        self._seen_services = set()
+        self._service_definitions: list[ServiceDefinition[DSD]] = []
+        self._seen_services: set[tuple[str, str, str]] = set()
 
     @property
     def detected_services(self) -> Sequence[ServiceDefinition[DSD]]:
@@ -2195,14 +2317,14 @@ class ServiceRegistryImpl(ServiceRegistry[DSD]):
     def register_service(
         self,
         path: VirtualPath,
-        name: Union[str, List[str]],
+        name: str | list[str],
         *,
         type_of_service: str = "service",  # "timer", etc.
         service_scope: str = "system",
         enable_by_default: bool = True,
         start_by_default: bool = True,
         default_upgrade_rule: ServiceUpgradeRule = "restart",
-        service_context: Optional[DSD] = None,
+        service_context: DSD | None = None,
     ) -> None:
         names = name if isinstance(name, list) else [name]
         if len(names) < 1:
