@@ -1,34 +1,25 @@
 import dataclasses
 import os.path
+from collections.abc import Callable, Sequence, Iterable, Mapping, Iterator, Container
 from importlib.resources.abc import Traversable
 from pathlib import Path
 from typing import (
     Optional,
-    Callable,
-    FrozenSet,
-    Dict,
-    List,
-    Tuple,
     Generic,
     TYPE_CHECKING,
     TypeVar,
     cast,
     Any,
-    Sequence,
-    Union,
-    Type,
     TypedDict,
-    Iterable,
-    Mapping,
     NotRequired,
     Literal,
-    Set,
-    Iterator,
-    Container,
     Protocol,
 )
 from weakref import ref
 
+from debian.debian_support import DpkgArchTable
+
+from debputy._deb_options_profiles import DebBuildOptionsAndProfiles
 from debputy.exceptions import (
     DebputyFSIsROError,
     PluginAPIViolationError,
@@ -38,11 +29,11 @@ from debputy.exceptions import (
     PluginInitializationError,
 )
 from debputy.filesystem_scan import as_path_def
-from debputy.lsp.diagnostics import LintSeverity
+from debputy.manifest_conditions import ConditionContext
 from debputy.manifest_parser.exceptions import ManifestParseException
 from debputy.manifest_parser.tagging_types import DebputyParsedContent, TypeMapping
 from debputy.manifest_parser.util import AttributePath, check_integration_mode
-from debputy.packages import BinaryPackage
+from debputy.packages import BinaryPackage, SourcePackage
 from debputy.plugin.api import (
     VirtualPath,
     BinaryCtrlAccessor,
@@ -70,6 +61,7 @@ from debputy.substitution import VariableContext
 from debputy.util import _normalize_path, package_cross_check_precheck
 
 if TYPE_CHECKING:
+    from debputy.lsp.diagnostics import LintSeverity
     from debputy.plugin.api.spec import (
         ServiceDetector,
         ServiceIntegrator,
@@ -89,7 +81,7 @@ TD = TypeVar("TD", bound="Union[DebputyParsedContent, List[DebputyParsedContent]
 PF = TypeVar("PF")
 SF = TypeVar("SF")
 TP = TypeVar("TP")
-TTP = Type[TP]
+TTP = type[TP]
 BSR = TypeVar("BSR", bound="BuildSystemRule")
 
 DIPKWHandler = Callable[[str, AttributePath, "ParserContextData"], TP]
@@ -100,16 +92,14 @@ DIPHandler = Callable[[str, PF, AttributePath, "ParserContextData"], TP]
 class DebputyPluginMetadata:
     plugin_name: str
     api_compat_version: int
-    plugin_loader: Optional[Callable[[], Callable[["DebputyPluginInitializer"], None]]]
-    plugin_initializer: Optional[Callable[["DebputyPluginInitializer"], None]]
+    plugin_loader: Callable[[], Callable[["DebputyPluginInitializer"], None]] | None
+    plugin_initializer: Callable[["DebputyPluginInitializer"], None] | None
     plugin_path: str
-    plugin_doc_path_resolver: Callable[[], Optional[Union[Traversable, Path]]] = (
-        lambda: None
-    )
+    plugin_doc_path_resolver: Callable[[], Traversable | Path | None] = lambda: None
     is_from_python_path: bool = False
     _is_initialized: bool = False
     _is_doc_path_resolved: bool = False
-    _plugin_doc_path: Optional[Union[Traversable, Path]] = None
+    _plugin_doc_path: Traversable | Path | None = None
 
     @property
     def is_bundled(self) -> bool:
@@ -124,7 +114,7 @@ class DebputyPluginMetadata:
         return self._is_initialized
 
     @property
-    def plugin_doc_path(self) -> Optional[Union[Traversable, Path]]:
+    def plugin_doc_path(self) -> Traversable | Path | None:
         if not self._is_doc_path_resolved:
             self._plugin_doc_path = self.plugin_doc_path_resolver()
             self._is_doc_path_resolved = True
@@ -180,7 +170,7 @@ class PluginProvidedParser(Generic[PF, TP]):
 
 
 class PPFFormatParam(TypedDict):
-    priority: Optional[int]
+    priority: int | None
     name: str
     owning_package: str
 
@@ -191,16 +181,14 @@ class PackagerProvidedFileClassSpec:
     stem: str
     installed_as_format: str
     default_mode: int
-    default_priority: Optional[int]
+    default_priority: int | None
     allow_name_segment: bool
     allow_architecture_segment: bool
-    post_formatting_rewrite: Optional[Callable[[str], str]]
+    post_formatting_rewrite: Callable[[str], str] | None
     packageless_is_fallback_for_all_packages: bool
     reservation_only: bool
-    formatting_callback: Optional[Callable[[str, PPFFormatParam, VirtualPath], str]] = (
-        None
-    )
-    reference_documentation: Optional[PackagerProvidedFileReferenceDocumentation] = None
+    formatting_callback: Callable[[str, PPFFormatParam, VirtualPath], str] | None = None
+    reference_documentation: PackagerProvidedFileReferenceDocumentation | None = None
     bug_950723: bool = False
     has_active_command: bool = True
 
@@ -213,10 +201,10 @@ class PackagerProvidedFileClassSpec:
         assigned_name: str,
         # Note this method is currently used 1:1 inside plugin tests.
         *,
-        owning_package: Optional[str] = None,
-        assigned_priority: Optional[int] = None,
-        path: Optional[VirtualPath] = None,
-    ) -> Tuple[str, str]:
+        owning_package: str | None = None,
+        assigned_priority: int | None = None,
+        path: VirtualPath | None = None,
+    ) -> tuple[str, str]:
         if assigned_priority is not None and not self.supports_priority:
             raise ValueError(
                 f"Cannot assign priority to packager provided files with stem"
@@ -259,7 +247,7 @@ class MetadataOrMaintscriptDetector:
     plugin_metadata: DebputyPluginMetadata
     detector_id: str
     detector: MetadataAutoDetector
-    applies_to_package_types: FrozenSet[str]
+    applies_to_package_types: frozenset[str]
     enabled: bool = True
 
     def applies_to(self, binary_package: BinaryPackage) -> bool:
@@ -290,17 +278,17 @@ class MetadataOrMaintscriptDetector:
 
 class DeclarativeInputParser(Generic[TD]):
     @property
-    def inline_reference_documentation(self) -> Optional[ParserDocumentation]:
+    def inline_reference_documentation(self) -> ParserDocumentation | None:
         return None
 
     @property
     def expected_debputy_integration_mode(
         self,
-    ) -> Optional[Container[DebputyIntegrationMode]]:
+    ) -> Container[DebputyIntegrationMode] | None:
         return None
 
     @property
-    def reference_documentation_url(self) -> Optional[str]:
+    def reference_documentation_url(self) -> str | None:
         doc = self.inline_reference_documentation
         return doc.documentation_reference_url if doc is not None else None
 
@@ -325,10 +313,10 @@ class DelegatingDeclarativeInputParser(DeclarativeInputParser[TD]):
         self,
         delegate: DeclarativeInputParser[TD],
         *,
-        inline_reference_documentation: Optional[ParserDocumentation] = None,
-        expected_debputy_integration_mode: Optional[
+        inline_reference_documentation: ParserDocumentation | None = None,
+        expected_debputy_integration_mode: None | (
             Container[DebputyIntegrationMode]
-        ] = None,
+        ) = None,
     ) -> None:
         self.delegate = delegate
         self._reference_documentation = inline_reference_documentation
@@ -337,11 +325,11 @@ class DelegatingDeclarativeInputParser(DeclarativeInputParser[TD]):
     @property
     def expected_debputy_integration_mode(
         self,
-    ) -> Optional[Container[DebputyIntegrationMode]]:
+    ) -> Container[DebputyIntegrationMode] | None:
         return self._expected_debputy_integration_mode
 
     @property
-    def inline_reference_documentation(self) -> Optional[ParserDocumentation]:
+    def inline_reference_documentation(self) -> ParserDocumentation | None:
         doc = self._reference_documentation
         if doc is None:
             return self.delegate.inline_reference_documentation
@@ -391,10 +379,10 @@ class ListWrappedDeclarativeInputParser(DelegatingDeclarativeInputParser[TD]):
 class DispatchingParserBase(Generic[TP]):
     def __init__(self, manifest_attribute_path_template: str) -> None:
         self.manifest_attribute_path_template = manifest_attribute_path_template
-        self._parsers: Dict[str, PluginProvidedParser[Any, TP]] = {}
+        self._parsers: dict[str, PluginProvidedParser[Any, TP]] = {}
 
     @property
-    def unknown_keys_diagnostic_severity(self) -> Optional[LintSeverity]:
+    def unknown_keys_diagnostic_severity(self) -> Optional["LintSeverity"]:
         return "error"
 
     def is_known_keyword(self, keyword: str) -> bool:
@@ -408,11 +396,11 @@ class DispatchingParserBase(Generic[TP]):
 
     def register_keyword(
         self,
-        keyword: Union[str, Sequence[str]],
+        keyword: str | Sequence[str],
         handler: DIPKWHandler,
         plugin_metadata: DebputyPluginMetadata,
         *,
-        inline_reference_documentation: Optional[ParserDocumentation] = None,
+        inline_reference_documentation: ParserDocumentation | None = None,
     ) -> None:
         reference_documentation_url = None
         if inline_reference_documentation:
@@ -450,7 +438,7 @@ class DispatchingParserBase(Generic[TP]):
 
     def register_parser(
         self,
-        keyword: Union[str, List[str]],
+        keyword: str | list[str],
         parser: "DeclarativeInputParser[PF]",
         handler: Callable[[str, PF, "AttributePath", "ParserContextData"], TP],
         plugin_metadata: DebputyPluginMetadata,
@@ -464,7 +452,7 @@ class DispatchingParserBase(Generic[TP]):
 
     def _add_parser(
         self,
-        keyword: Union[str, Iterable[str]],
+        keyword: str | Iterable[str],
         ppp: "PluginProvidedParser[PF, TP]",
     ) -> None:
         ks = [keyword] if isinstance(keyword, str) else keyword
@@ -504,15 +492,15 @@ class DispatchingObjectParser(
         self,
         manifest_attribute_path_template: str,
         *,
-        parser_documentation: Optional[ParserDocumentation] = None,
-        expected_debputy_integration_mode: Optional[
+        parser_documentation: ParserDocumentation | None = None,
+        expected_debputy_integration_mode: None | (
             Container[DebputyIntegrationMode]
-        ] = None,
-        unknown_keys_diagnostic_severity: Optional[LintSeverity] = "error",
+        ) = None,
+        unknown_keys_diagnostic_severity: Optional["LintSeverity"] = "error",
         allow_unknown_keys: bool = False,
     ) -> None:
         super().__init__(manifest_attribute_path_template)
-        self._attribute_documentation: List[ParserAttributeDocumentation] = []
+        self._attribute_documentation: list[ParserAttributeDocumentation] = []
         if parser_documentation is None:
             parser_documentation = reference_documentation()
         self._parser_documentation = parser_documentation
@@ -521,21 +509,21 @@ class DispatchingObjectParser(
         self._allow_unknown_keys = allow_unknown_keys
 
     @property
-    def unknown_keys_diagnostic_severity(self) -> Optional[LintSeverity]:
+    def unknown_keys_diagnostic_severity(self) -> Optional["LintSeverity"]:
         return self._unknown_keys_diagnostic_severity
 
     @property
     def expected_debputy_integration_mode(
         self,
-    ) -> Optional[Container[DebputyIntegrationMode]]:
+    ) -> Container[DebputyIntegrationMode] | None:
         return self._expected_debputy_integration_mode
 
     @property
-    def reference_documentation_url(self) -> Optional[str]:
+    def reference_documentation_url(self) -> str | None:
         return self._parser_documentation.documentation_reference_url
 
     @property
-    def inline_reference_documentation(self) -> Optional[ParserDocumentation]:
+    def inline_reference_documentation(self) -> ParserDocumentation | None:
         ref_doc = self._parser_documentation
         return reference_documentation(
             title=ref_doc.title,
@@ -560,12 +548,12 @@ class DispatchingObjectParser(
         parser: "DispatchingObjectParser",
         plugin_metadata: DebputyPluginMetadata,
         *,
-        on_end_parse_step: Optional[
+        on_end_parse_step: None | (
             Callable[
-                [str, Optional[Mapping[str, Any]], AttributePath, "ParserContextData"],
+                [str, Mapping[str, Any] | None, AttributePath, "ParserContextData"],
                 None,
             ]
-        ] = None,
+        ) = None,
         nested_in_package_context: bool = False,
     ) -> None:
         def _handler(
@@ -674,10 +662,10 @@ class InPackageContextParser(
         manifest_attribute_path_template: str,
         delegate: DeclarativeInputParser[TP],
         *,
-        parser_documentation: Optional[ParserDocumentation] = None,
+        parser_documentation: ParserDocumentation | None = None,
     ) -> None:
         self.manifest_attribute_path_template = manifest_attribute_path_template
-        self._attribute_documentation: List[ParserAttributeDocumentation] = []
+        self._attribute_documentation: list[ParserAttributeDocumentation] = []
         super().__init__(delegate, inline_reference_documentation=parser_documentation)
 
     def parse_input(
@@ -778,8 +766,8 @@ class DispatchingTableParser(
 
 @dataclasses.dataclass(slots=True)
 class DeclarativeValuelessKeywordInputParser(DeclarativeInputParser[None]):
-    inline_reference_documentation: Optional[ParserDocumentation] = None
-    documentation_reference: Optional[str] = None
+    inline_reference_documentation: ParserDocumentation | None = None
+    documentation_reference: str | None = None
 
     def parse_input(
         self,
@@ -803,9 +791,9 @@ class DeclarativeValuelessKeywordInputParser(DeclarativeInputParser[None]):
 class PluginProvidedManifestVariable:
     plugin_metadata: DebputyPluginMetadata
     variable_name: str
-    variable_value: Optional[Union[str, Callable[[VariableContext], str]]]
+    variable_value: str | Callable[[VariableContext], str] | None
     is_context_specific_variable: bool
-    variable_reference_documentation: Optional[str] = None
+    variable_reference_documentation: str | None = None
     is_documentation_placeholder: bool = False
     is_for_special_case: bool = False
 
@@ -828,13 +816,13 @@ class PluginProvidedManifestVariable:
 
 @dataclasses.dataclass(slots=True, frozen=True)
 class AutomaticDiscardRuleExample:
-    content: Sequence[Tuple[PathDef, bool]]
-    description: Optional[str] = None
+    content: Sequence[tuple[PathDef, bool]]
+    description: str | None = None
 
 
 def automatic_discard_rule_example(
-    *content: Union[str, PathDef, Tuple[Union[str, PathDef], bool]],
-    example_description: Optional[str] = None,
+    *content: str | PathDef | tuple[str | PathDef, bool],
+    example_description: str | None = None,
 ) -> AutomaticDiscardRuleExample:
     """Provide an example for an automatic discard rule
 
@@ -909,16 +897,16 @@ def automatic_discard_rule_example(
 @dataclasses.dataclass(slots=True, frozen=True)
 class PluginProvidedPackageProcessor:
     processor_id: str
-    applies_to_package_types: FrozenSet[str]
+    applies_to_package_types: frozenset[str]
     package_processor: PackageProcessor
-    dependencies: FrozenSet[Tuple[str, str]]
+    dependencies: frozenset[tuple[str, str]]
     plugin_metadata: DebputyPluginMetadata
 
     def applies_to(self, binary_package: BinaryPackage) -> bool:
         return binary_package.package_type in self.applies_to_package_types
 
     @property
-    def dependency_id(self) -> Tuple[str, str]:
+    def dependency_id(self) -> tuple[str, str]:
         return self.plugin_metadata.plugin_name, self.processor_id
 
     def run_package_processor(
@@ -935,7 +923,7 @@ class PluginProvidedDiscardRule:
     name: str
     plugin_metadata: DebputyPluginMetadata
     discard_check: Callable[[VirtualPath], bool]
-    reference_documentation: Optional[str]
+    reference_documentation: str | None
     examples: Sequence[AutomaticDiscardRuleExample] = tuple()
 
     def should_discard(self, path: VirtualPath) -> bool:
@@ -950,12 +938,8 @@ class ServiceManagerDetails:
     plugin_metadata: DebputyPluginMetadata
 
 
-ReferenceValue = TypedDict(
-    "ReferenceValue",
-    {
-        "description": str,
-    },
-)
+class ReferenceValue(TypedDict):
+    description: str
 
 
 def _reference_data_value(
@@ -1092,8 +1076,8 @@ KNOWN_PACKAGING_FILE_CONFIG_FEATURE_DESCRIPTION: Mapping[
     ),
 }
 
-CONFIG_FEATURE_ALIASES: Dict[
-    KnownPackagingConfigFeature, List[Tuple[KnownPackagingConfigFeature, int]]
+CONFIG_FEATURE_ALIASES: dict[
+    KnownPackagingConfigFeature, list[tuple[KnownPackagingConfigFeature, int]]
 ] = {
     "dh-filearray": [
         ("dh-filearray", 0),
@@ -1109,8 +1093,8 @@ CONFIG_FEATURE_ALIASES: Dict[
 
 
 def _implies(
-    features: List[KnownPackagingConfigFeature],
-    seen: Set[KnownPackagingConfigFeature],
+    features: list[KnownPackagingConfigFeature],
+    seen: set[KnownPackagingConfigFeature],
     implying: Sequence[KnownPackagingConfigFeature],
     implied: KnownPackagingConfigFeature,
 ) -> None:
@@ -1123,9 +1107,9 @@ def _implies(
 
 def expand_known_packaging_config_features(
     compat_level: int,
-    features: List[KnownPackagingConfigFeature],
-) -> List[KnownPackagingConfigFeature]:
-    final_features: List[KnownPackagingConfigFeature] = []
+    features: list[KnownPackagingConfigFeature],
+) -> list[KnownPackagingConfigFeature]:
+    final_features: list[KnownPackagingConfigFeature] = []
     seen = set()
     for feature in features:
         expanded = CONFIG_FEATURE_ALIASES.get(feature)
@@ -1151,7 +1135,7 @@ def expand_known_packaging_config_features(
 
 class DHCompatibilityBasedRule(DebputyParsedContent):
     install_pattern: NotRequired[str]
-    add_config_features: NotRequired[List[KnownPackagingConfigFeature]]
+    add_config_features: NotRequired[list[KnownPackagingConfigFeature]]
     starting_with_compat_level: NotRequired[int]
 
 
@@ -1160,13 +1144,13 @@ class KnownPackagingFileInfo(DebputyParsedContent):
     path: NotRequired[str]
     pkgfile: NotRequired[str]
     detection_method: NotRequired[Literal["path", "dh.pkgfile"]]
-    file_categories: NotRequired[List[KnownPackagingFileCategories]]
-    documentation_uris: NotRequired[List[str]]
-    debputy_cmd_templates: NotRequired[List[List[str]]]
-    debhelper_commands: NotRequired[List[str]]
-    config_features: NotRequired[List[KnownPackagingConfigFeature]]
+    file_categories: NotRequired[list[KnownPackagingFileCategories]]
+    documentation_uris: NotRequired[list[str]]
+    debputy_cmd_templates: NotRequired[list[list[str]]]
+    debhelper_commands: NotRequired[list[str]]
+    config_features: NotRequired[list[KnownPackagingConfigFeature]]
     install_pattern: NotRequired[str]
-    dh_compat_rules: NotRequired[List[DHCompatibilityBasedRule]]
+    dh_compat_rules: NotRequired[list[DHCompatibilityBasedRule]]
     default_priority: NotRequired[int]
     post_formatting_rewrite: NotRequired[Literal["period-to-underscore"]]
     packageless_is_fallback_for_all_packages: NotRequired[bool]
@@ -1189,20 +1173,20 @@ class BuildSystemAutoDetector(Protocol):
 @dataclasses.dataclass(slots=True, frozen=True)
 class PluginProvidedTypeMapping:
     mapped_type: TypeMapping[Any, Any]
-    reference_documentation: Optional[TypeMappingDocumentation]
+    reference_documentation: TypeMappingDocumentation | None
     plugin_metadata: DebputyPluginMetadata
 
 
 @dataclasses.dataclass(slots=True, frozen=True)
 class PluginProvidedBuildSystemAutoDetection(Generic[BSR]):
     manifest_keyword: str
-    build_system_rule_type: Type[BSR]
+    build_system_rule_type: type[BSR]
     detector: BuildSystemAutoDetector
     constructor: Callable[
         ["BuildRuleParsedFormat", AttributePath, "HighLevelManifest"],
         BSR,
     ]
-    auto_detection_shadow_build_systems: FrozenSet[str]
+    auto_detection_shadow_build_systems: frozenset[str]
     plugin_metadata: DebputyPluginMetadata
 
 
@@ -1237,16 +1221,16 @@ class PackageProcessingContextProvider(PackageProcessingContext):
         self,
         manifest: "HighLevelManifest",
         binary_package: BinaryPackage,
-        related_udeb_package: Optional[BinaryPackage],
+        related_udeb_package: BinaryPackage | None,
         package_data_table: PackageDataTable,
     ) -> None:
         self._manifest = manifest
         self._binary_package = binary_package
         self._related_udeb_package = related_udeb_package
         self._package_data_table = ref(package_data_table)
-        self._cross_check_cache: Optional[
-            Sequence[Tuple[BinaryPackage, "VirtualPath"]]
-        ] = None
+        self._cross_check_cache: None | (
+            Sequence[tuple[BinaryPackage, "VirtualPath"]]
+        ) = None
 
     def _package_state_for(
         self,
@@ -1266,12 +1250,16 @@ class PackageProcessingContextProvider(PackageProcessingContext):
             include_binnmu_version=not package.is_arch_all
         )
 
+    @property
+    def source_package(self) -> SourcePackage:
+        return self._manifest.source_package
+
     @property
     def binary_package(self) -> BinaryPackage:
         return self._binary_package
 
     @property
-    def related_udeb_package(self) -> Optional[BinaryPackage]:
+    def related_udeb_package(self) -> BinaryPackage | None:
         return self._related_udeb_package
 
     @property
@@ -1279,13 +1267,13 @@ class PackageProcessingContextProvider(PackageProcessingContext):
         return self._package_version_for(self._binary_package)
 
     @property
-    def related_udeb_package_version(self) -> Optional[str]:
+    def related_udeb_package_version(self) -> str | None:
         udeb = self._related_udeb_package
         if udeb is None:
             return None
         return self._package_version_for(udeb)
 
-    def accessible_package_roots(self) -> Iterable[Tuple[BinaryPackage, "VirtualPath"]]:
+    def accessible_package_roots(self) -> Iterable[tuple[BinaryPackage, "VirtualPath"]]:
         package_table = self._package_data_table()
         if package_table is None:
             raise ReferenceError(
@@ -1310,6 +1298,30 @@ class PackageProcessingContextProvider(PackageProcessingContext):
             self._cross_check_cache = cache
         return cache
 
+    def manifest_configuration[T](
+        self,
+        context_package: SourcePackage | BinaryPackage,
+        value_type: type[T],
+    ) -> T | None:
+        return self._manifest.manifest_configuration(context_package, value_type)
+
+    @property
+    def dpkg_arch_query_table(self) -> DpkgArchTable:
+        return self._manifest.dpkg_arch_query_table
+
+    @property
+    def deb_options_and_profiles(self) -> DebBuildOptionsAndProfiles:
+        return self._manifest.deb_options_and_profiles
+
+    @property
+    def source_condition_context(self) -> ConditionContext:
+        return self._manifest.source_condition_context
+
+    def condition_context(
+        self, binary_package: BinaryPackage | None
+    ) -> ConditionContext:
+        return self._manifest.condition_context(binary_package)
+
 
 @dataclasses.dataclass(slots=True, frozen=True)
 class PluginProvidedTrigger:
