@@ -4,15 +4,10 @@ import os
 import tempfile
 import textwrap
 from typing import (
-    Iterable,
     Optional,
-    Callable,
     Literal,
     Union,
-    Iterator,
     overload,
-    FrozenSet,
-    Sequence,
     TypeVar,
     Any,
     TYPE_CHECKING,
@@ -20,22 +15,27 @@ from typing import (
     BinaryIO,
     Generic,
     ContextManager,
-    List,
-    Type,
-    Tuple,
     get_args,
-    Container,
     final,
+    cast,
 )
+from collections.abc import Iterable, Callable, Iterator, Sequence, Container
 
+from debian.debian_support import DpkgArchTable
 from debian.substvars import Substvars
 
 from debputy import util
-from debputy.exceptions import TestPathWithNonExistentFSPathError, PureVirtualPathError
+from debputy._deb_options_profiles import DebBuildOptionsAndProfiles
+from debputy.exceptions import (
+    TestPathWithNonExistentFSPathError,
+    PureVirtualPathError,
+    PluginInitializationError,
+)
 from debputy.interpreter import Interpreter, extract_shebang_interpreter_from_file
+from debputy.manifest_conditions import ConditionContext
 from debputy.manifest_parser.tagging_types import DebputyDispatchableType
 from debputy.manifest_parser.util import parse_symbolic_mode
-from debputy.packages import BinaryPackage
+from debputy.packages import BinaryPackage, SourcePackage
 from debputy.types import S
 
 if TYPE_CHECKING:
@@ -45,7 +45,12 @@ if TYPE_CHECKING:
     )
     from debputy.plugin.api.doc_parsing import ParserRefDocumentation
     from debputy.plugin.api.impl_types import DIPHandler
-    from debputy.plugin.debputy.to_be_api_types import BuildSystemRule
+    from debputy.plugins.debputy.to_be_api_types import BuildSystemRule
+    from debputy.plugin.api.impl import DebputyPluginInitializerProvider
+
+    # Work around it being "unused" since `cast("...", ...)` does not register as a use
+    # for most static analysis tools.
+    assert DebputyPluginInitializerProvider
 
 
 DP = TypeVar("DP", bound=DebputyDispatchableType)
@@ -97,7 +102,7 @@ DebputyIntegrationMode = Literal[
 INTEGRATION_MODE_FULL: DebputyIntegrationMode = "full"
 INTEGRATION_MODE_DH_DEBPUTY_RRR: DebputyIntegrationMode = "dh-sequence-zz-debputy-rrr"
 INTEGRATION_MODE_DH_DEBPUTY: DebputyIntegrationMode = "dh-sequence-zz-debputy"
-ALL_DEBPUTY_INTEGRATION_MODES: FrozenSet[DebputyIntegrationMode] = frozenset(
+ALL_DEBPUTY_INTEGRATION_MODES: frozenset[DebputyIntegrationMode] = frozenset(
     get_args(DebputyIntegrationMode)
 )
 
@@ -118,7 +123,7 @@ def not_integrations(
 
 @dataclasses.dataclass(slots=True, frozen=True)
 class PackagerProvidedFileReferenceDocumentation:
-    description: Optional[str] = None
+    description: str | None = None
     format_documentation_uris: Sequence[str] = tuple()
 
     def replace(self, **changes: Any) -> "PackagerProvidedFileReferenceDocumentation":
@@ -127,8 +132,8 @@ class PackagerProvidedFileReferenceDocumentation:
 
 def packager_provided_file_reference_documentation(
     *,
-    description: Optional[str] = None,
-    format_documentation_uris: Optional[Sequence[str]] = tuple(),
+    description: str | None = None,
+    format_documentation_uris: Sequence[str] | None = tuple(),
 ) -> PackagerProvidedFileReferenceDocumentation:
     """Provide documentation for a given packager provided file.
 
@@ -184,7 +189,7 @@ class PathMetadataReference(Generic[PMT]):
         raise NotImplementedError
 
     @property
-    def value(self) -> Optional[PMT]:
+    def value(self) -> PMT | None:
         """Fetch the currently stored value if present.
 
         :return: The value previously stored if any. Returns `None` if the value was never
@@ -193,7 +198,7 @@ class PathMetadataReference(Generic[PMT]):
         raise NotImplementedError
 
     @value.setter
-    def value(self, value: Optional[PMT]) -> None:
+    def value(self, value: PMT | None) -> None:
         """Replace any current value with the provided value
 
         This operation is only possible if the path is writable *and* the caller is from
@@ -214,13 +219,13 @@ class PathMetadataReference(Generic[PMT]):
 @dataclasses.dataclass(slots=True)
 class PathDef:
     path_name: str
-    mode: Optional[int] = None
-    mtime: Optional[int] = None
-    has_fs_path: Optional[bool] = None
-    fs_path: Optional[str] = None
-    link_target: Optional[str] = None
-    content: Optional[str] = None
-    materialized_content: Optional[str] = None
+    mode: int | None = None
+    mtime: int | None = None
+    has_fs_path: bool | None = None
+    fs_path: str | None = None
+    link_target: str | None = None
+    content: str | None = None
+    materialized_content: str | None = None
 
 
 @dataclasses.dataclass(slots=True, frozen=True)
@@ -228,31 +233,29 @@ class DispatchablePluggableManifestRuleMetadata(Generic[DP]):
     """NOT PUBLIC API (used internally by part of the public API)"""
 
     manifest_keywords: Sequence[str]
-    dispatched_type: Type[DP]
+    dispatched_type: type[DP]
     unwrapped_constructor: "DIPHandler"
-    expected_debputy_integration_mode: Optional[Container[DebputyIntegrationMode]] = (
-        None
-    )
+    expected_debputy_integration_mode: Container[DebputyIntegrationMode] | None = None
     online_reference_documentation: Optional["ParserDocumentation"] = None
     apply_standard_attribute_documentation: bool = False
-    source_format: Optional[Any] = None
+    source_format: Any | None = None
 
 
 @dataclasses.dataclass(slots=True, frozen=True)
 class BuildSystemManifestRuleMetadata(DispatchablePluggableManifestRuleMetadata):
-    build_system_impl: Optional[Type["BuildSystemRule"]] = None
-    auto_detection_shadow_build_systems: FrozenSet[str] = frozenset()
+    build_system_impl: type["BuildSystemRule"] | None = None
+    auto_detection_shadow_build_systems: frozenset[str] = frozenset()
 
 
 def virtual_path_def(
     path_name: str,
     /,
-    mode: Optional[int] = None,
-    mtime: Optional[int] = None,
-    fs_path: Optional[str] = None,
-    link_target: Optional[str] = None,
-    content: Optional[str] = None,
-    materialized_content: Optional[str] = None,
+    mode: int | None = None,
+    mtime: int | None = None,
+    fs_path: str | None = None,
+    link_target: str | None = None,
+    content: str | None = None,
+    materialized_content: str | None = None,
 ) -> PathDef:
     """Define a virtual path for use with examples or, in tests, `build_virtual_file_system`
 
@@ -328,6 +331,11 @@ class PackageProcessingContext:
 
     __slots__ = ()
 
+    @property
+    def source_package(self) -> SourcePackage:
+        """The source package stanza from `debian/control`"""
+        raise NotImplementedError
+
     @property
     def binary_package(self) -> BinaryPackage:
         """The binary package stanza from `debian/control`"""
@@ -342,23 +350,321 @@ class PackageProcessingContext:
         raise NotImplementedError
 
     @property
-    def related_udeb_package(self) -> Optional[BinaryPackage]:
+    def related_udeb_package(self) -> BinaryPackage | None:
         """An udeb related to this binary package (if any)"""
         raise NotImplementedError
 
     @property
-    def related_udeb_package_version(self) -> Optional[str]:
+    def related_udeb_package_version(self) -> str | None:
         """The version of the related udeb package (if present)
 
         Note this never includes the binNMU version for arch:all packages, but it may for arch:any.
         """
         raise NotImplementedError
 
-    def accessible_package_roots(self) -> Iterable[Tuple[BinaryPackage, "VirtualPath"]]:
+    def accessible_package_roots(self) -> Iterable[tuple[BinaryPackage, "VirtualPath"]]:
         raise NotImplementedError
 
-    # """The source package stanza from `debian/control`"""
-    # source_package: SourcePackage
+    def manifest_configuration[T](
+        self,
+        context_package: SourcePackage | BinaryPackage,
+        value_type: type[T],
+    ) -> T | None:
+        """Request access to configuration from the manifest
+
+        This method will return the value associated with a pluggable manifest rule assuming
+        said configuration was provided.
+
+
+        :param context_package: The context in which the configuration will be. Generally, it will be
+          the binary package for anything under `packages:` and the source package otherwise.
+        :param value_type: The type used during registered (return type of the parser/unpack function)
+        :return: The value from the manifest if present or `None` otherwise
+        """
+        raise NotImplementedError
+
+    @property
+    def dpkg_arch_query_table(self) -> DpkgArchTable:
+        raise NotImplementedError
+
+    @property
+    def deb_options_and_profiles(self) -> DebBuildOptionsAndProfiles:
+        raise NotImplementedError
+
+    @property
+    def source_condition_context(self) -> ConditionContext:
+        raise NotImplementedError
+
+    def condition_context(
+        self, binary_package: BinaryPackage | None
+    ) -> ConditionContext:
+        raise NotImplementedError
+
+    @property
+    def binary_condition_context(self) -> ConditionContext:
+        return self.condition_context(self.binary_package)
+
+
+class DebputyPluginDefinition:
+    """Plugin definition entity
+
+    The plugin definition provides a way for the plugin code to register the features it provides via
+    accessors.
+    """
+
+    __slots__ = ("_generic_initializers",)
+
+    def __init__(self) -> None:
+        self._generic_initializers: list[
+            Callable[["DebputyPluginInitializerProvider"], None]
+        ] = []
+
+    @staticmethod
+    def _name2id(provided_id: str | None, name: str) -> str:
+        if provided_id is not None:
+            return provided_id
+        if name.endswith("_"):
+            name = name[:-1]
+        return name.replace("_", "-")
+
+    def metadata_or_maintscript_detector(
+        self,
+        func: MetadataAutoDetector | None = None,
+        *,
+        detector_id: str | None = None,
+        package_type: PackageTypeSelector = "deb",
+    ) -> Callable[[MetadataAutoDetector], MetadataAutoDetector] | MetadataAutoDetector:
+        """Provide a pre-assembly hook that can affect the metadata/maintscript of binary ("deb") packages
+
+        The provided hook will be run once per binary package to be assembled, and it can see all the content
+        ("data.tar") planned to be included in the deb. The hook may do any *read-only* analysis of this content
+        and provide metadata, alter substvars or inject maintscript snippets.  However, the hook must *not*
+        change the content ("data.tar") part of the deb.
+
+        The hook will be run unconditionally for all binary packages built. When the hook does not apply to all
+        packages, it must provide its own (internal) logic for detecting whether it is relevant and reduce itself
+        to a no-op if it should not apply to the current package.
+
+        Hooks are run in "some implementation defined order" and should not rely on being run before or after
+        any other hook.
+
+        The hooks are only applied to packages defined in `debian/control`. Notably, the metadata detector will
+        not apply to auto-generated `-dbgsym` packages (as those are not listed explicitly in `debian/control`).
+
+
+        >>> plugin_definition = define_debputy_plugin()
+        >>> @plugin_definition.metadata_or_maintscript_detector
+        ... def gsettings_dependencies(
+        ...     fs_root: "VirtualPath",
+        ...     ctrl: "BinaryCtrlAccessor",
+        ...     context: "PackageProcessingContext",
+        ... ) -> None:
+        ...     gsettings_schema_dir = fs_root.lookup("/usr/share/glib-2.0/schemas")
+        ...     if gsettings_schema_dir is None:
+        ...         return
+        ...     for path in gsettings_schema_dir.all_paths():
+        ...         if path.is_file and path.name.endswith((".xml", ".override")):
+        ...             ctrl.substvars.add_dependency(
+        ...                 "misc:Depends",
+        ...                 "dconf-gsettings-backend | gsettings-backend",
+        ...             )
+        ...             break
+
+        :param func: The function to be decorated/registered.
+        :param detector_id: A plugin-wide unique ID for this detector. Packagers may use this ID for disabling
+          the detector and accordingly the ID is part of the plugin's API toward the packager.
+        :param package_type: Which kind of packages this metadata detector applies to.  The package type is generally
+          defined by `Package-Type` field in the binary package. The default is to only run for regular `deb` packages
+          and ignore `udeb` packages.
+        """
+
+        def _decorate(f: MetadataAutoDetector) -> MetadataAutoDetector:
+
+            final_id = self._name2id(detector_id, f.__name__)
+
+            def _init(api: "DebputyPluginInitializer") -> None:
+                api.metadata_or_maintscript_detector(
+                    final_id,
+                    f,
+                    package_type=package_type,
+                )
+
+            self._generic_initializers.append(_init)
+
+            return f
+
+        if func:
+            return _decorate(func)
+        return _decorate
+
+    def manifest_variable(
+        self,
+        variable_name: str,
+        value: str,
+        *,
+        variable_reference_documentation: str | None = None,
+    ) -> None:
+        """Provide a variable that can be used in the package manifest
+
+            >>> plugin_definition = define_debputy_plugin()
+            >>> # Enable users to use "{{path:BASH_COMPLETION_DIR}}/foo" in their manifest.
+            >>> plugin_definition.manifest_variable(
+            ...     "path:BASH_COMPLETION_DIR",
+            ...     "/usr/share/bash-completion/completions",
+            ...     variable_reference_documentation="Directory to install bash completions into",
+            ... )
+
+        :param variable_name: The variable name. It should be provided without the substution prefix/suffix
+        :param value: The constant value that the variable should resolve to. It is not possible to provide
+          dynamic / context based values at this time.
+        :param variable_reference_documentation: A short snippet of reference documentation that explains
+          the purpose of the variable. This will be shown to users in various contexts such as the hover
+          docs.
+        """
+
+        def _init(api: "DebputyPluginInitializer") -> None:
+            api.manifest_variable(
+                variable_name,
+                value,
+                variable_reference_documentation=variable_reference_documentation,
+            )
+
+        self._generic_initializers.append(_init)
+
+    def packager_provided_file(
+        self,
+        stem: str,
+        installed_path: str,
+        *,
+        default_mode: int = 0o0644,
+        default_priority: int | None = None,
+        allow_name_segment: bool = True,
+        allow_architecture_segment: bool = False,
+        post_formatting_rewrite: Callable[[str], str] | None = None,
+        packageless_is_fallback_for_all_packages: bool = False,
+        reservation_only: bool = False,
+        reference_documentation: None | (
+            PackagerProvidedFileReferenceDocumentation
+        ) = None,
+    ) -> None:
+        """Register a packager provided file (debian/<pkg>.foo)
+
+        Register a packager provided file that debputy should automatically detect and install for the
+        packager (example `debian/foo.tmpfiles` -> `debian/foo/usr/lib/tmpfiles.d/foo.conf`).  A packager
+        provided file typically identified by a package prefix and a "stem" and by convention placed
+        in the `debian/` directory.
+
+        Where possible, please define these packager provided files via the JSON metadata rather than
+        via Python code.
+
+        Like debhelper, debputy also supports the `foo.bar.tmpfiles` variant where the file is to be
+        installed into the `foo` package but be named after the `bar` segment rather than the package name.
+        This feature can be controlled via the `allow_name_segment` parameter.
+
+        :param stem: The "stem" of the file. This would be the `tmpfiles` part of `debian/foo.tmpfiles`.
+          Note that this value must be unique across all registered packager provided files.
+        :param installed_path: A format string describing where the file should be installed. Would be
+          `/usr/lib/tmpfiles.d/{name}.conf` from the example above.
+
+          The caller should provide a string with one or more of the placeholders listed below (usually `{name}`
+          should be one of them). The format affect the entire path.
+
+          The following placeholders are supported:
+            * `{name}` - The name in the name segment (defaulting the package name if no name segment is given)
+            * `{priority}` / `{priority:02}` - The priority of the file. Only provided priorities are used (that
+               is, default_priority is not None).  The latter variant ensuring that the priority takes at least
+               two characters and the `0` character is left-padded for priorities that takes less than two
+               characters.
+            * `{owning_package}` - The name of the package.  Should only be used when `{name}` alone is insufficient.
+              If you do not want the "name" segment in the first place, use `allow_name_segment=False` instead.
+
+          The path is always interpreted as relative to the binary package root.
+
+        :param default_mode: The mode the installed file should have by default. Common options are 0o0644 (the default)
+          or 0o0755 (for files that must be executable).
+        :param allow_architecture_segment: If True, the file may have an optional "architecture" segment at the end
+           (`foo.tmpfiles.amd64`), which marks it architecture specific. When False, debputy will detect the
+           "architecture" segment and report the use as an error.  Note the architecture segment is only allowed for
+           arch:any packages. If a file targeting an arch:all package uses an architecture specific file it will
+           always result in an error.
+        :param allow_name_segment: If True, the file may have an optional "name" segment after the package name prefix.
+           (`foo.<name-here>.tmpfiles`). When False, debputy will detect the "name" segment and report the use as an
+           error.
+
+           Note that when this parameter is True, then a package can provide multiple instances with this stem. When
+           False, then at most one file can be provided per package.
+        :param default_priority: Special-case option for packager files that are installed into directories that have
+          "parse ordering" or "priority".  These files will generally be installed as something like `20-foo.conf`
+          where the `20-` denotes their "priority".  If the plugin is registering such a file type, then it should
+          provide a default priority.
+
+          The following placeholders are supported:
+            * `{name}` - The name in the name segment (defaulting the package name if no name segment is given)
+            * `{priority}` - The priority of the file. Only provided priorities are used (that is, default_priority
+               is not None)
+            * `{owning_package}` - The name of the package.  Should only be used when `{name}` alone is insufficient.
+              If you do not want the "name" segment in the first place, use `allow_name_segment=False` instead.
+        :param post_formatting_rewrite: An optional "name correcting" callback. It receives the formatted name and can
+          do any transformation required. The primary use-case for this is to replace "forbidden" characters. The most
+          common case for debputy itself is to replace "." with "_" for tools that refuse to work with files containing
+          "." (`lambda x: x.replace(".", "_")`).  The callback operates on basename of formatted version of the
+          `installed_path` and the callback should return the basename.
+        :param packageless_is_fallback_for_all_packages: If True, the packageless variant (such as, `debian/changelog`)
+          is a fallback for every package.
+        :param reference_documentation: Reference documentation for the packager provided file. Use the
+           packager_provided_file_reference_documentation function to provide the value for this parameter.
+        :param reservation_only: When True, tell debputy that the plugin reserves this packager provided file, but that
+          debputy should not actually install it automatically.  This is useful in the cases, where the plugin
+          needs to process the file before installing it.  The file will be marked as provided by this plugin. This
+          enables introspection and detects conflicts if other plugins attempts to claim the file.
+        """
+
+        def _init(api: "DebputyPluginInitializer") -> None:
+            api.packager_provided_file(
+                stem,
+                installed_path,
+                default_mode=default_mode,
+                default_priority=default_priority,
+                allow_name_segment=allow_name_segment,
+                allow_architecture_segment=allow_architecture_segment,
+                post_formatting_rewrite=post_formatting_rewrite,
+                packageless_is_fallback_for_all_packages=packageless_is_fallback_for_all_packages,
+                reservation_only=reservation_only,
+                reference_documentation=reference_documentation,
+            )
+
+        self._generic_initializers.append(_init)
+
+    def initialize(self, api: "DebputyPluginInitializer") -> None:
+        """Initialize the plugin from this definition
+
+        Most plugins will not need this function as the plugin loading will call this function
+        when relevant. However, a plugin can call this manually from a function-based initializer.
+        This is mostly useful if the function-based initializer need to set up a few features
+        that the `DebputyPluginDefinition` cannot do and the mix/match approach is not too
+        distracting for plugin maintenance.
+
+        :param api: The plugin initializer provided by the `debputy` plugin system
+        """
+
+        api_impl = cast(
+            "DebputyPluginInitializerProvider",
+            api,
+        )
+        initializers = self._generic_initializers
+        if not initializers:
+            plugin_name = api_impl.plugin_metadata.plugin_name
+            raise PluginInitializationError(
+                f"Initialization of {plugin_name}: The plugin definition was never used to register any features."
+                " If you want to conditionally register features, please use an initializer functon instead."
+            )
+
+        for initializer in initializers:
+            initializer(api_impl)
+
+
+def define_debputy_plugin() -> DebputyPluginDefinition:
+    return DebputyPluginDefinition()
 
 
 class DebputyPluginInitializer:
@@ -370,15 +676,15 @@ class DebputyPluginInitializer:
         installed_path: str,
         *,
         default_mode: int = 0o0644,
-        default_priority: Optional[int] = None,
+        default_priority: int | None = None,
         allow_name_segment: bool = True,
         allow_architecture_segment: bool = False,
-        post_formatting_rewrite: Optional[Callable[[str], str]] = None,
+        post_formatting_rewrite: Callable[[str], str] | None = None,
         packageless_is_fallback_for_all_packages: bool = False,
         reservation_only: bool = False,
-        reference_documentation: Optional[
+        reference_documentation: None | (
             PackagerProvidedFileReferenceDocumentation
-        ] = None,
+        ) = None,
     ) -> None:
         """Register a packager provided file (debian/<pkg>.foo)
 
@@ -462,7 +768,7 @@ class DebputyPluginInitializer:
         change the content ("data.tar") part of the deb.
 
         The hook will be run unconditionally for all binary packages built. When the hook does not apply to all
-        packages, it must provide its own (internal) logic for detecting whether it is relevant and reduced itself
+        packages, it must provide its own (internal) logic for detecting whether it is relevant and reduce itself
         to a no-op if it should not apply to the current package.
 
         Hooks are run in "some implementation defined order" and should not rely on being run before or after
@@ -485,7 +791,8 @@ class DebputyPluginInitializer:
         self,
         variable_name: str,
         value: str,
-        variable_reference_documentation: Optional[str] = None,
+        *,
+        variable_reference_documentation: str | None = None,
     ) -> None:
         """Provide a variable that can be used in the package manifest
 
@@ -511,7 +818,7 @@ class MaintscriptAccessor:
         self,
         run_snippet: str,
         /,
-        indent: Optional[bool] = None,
+        indent: bool | None = None,
         perform_substitution: bool = True,
         skip_on_rollback: bool = False,
     ) -> None:
@@ -547,7 +854,7 @@ class MaintscriptAccessor:
         self,
         run_snippet: str,
         /,
-        indent: Optional[bool] = None,
+        indent: bool | None = None,
         perform_substitution: bool = True,
     ) -> None:
         """Provide a snippet to be run when the package is about to be "configured" for the first time
@@ -583,7 +890,7 @@ class MaintscriptAccessor:
         self,
         run_snippet: str,
         /,
-        indent: Optional[bool] = None,
+        indent: bool | None = None,
         perform_substitution: bool = True,
     ) -> None:
         """Provide a snippet to be run when the package is about to be "configured" after an upgrade
@@ -610,7 +917,7 @@ class MaintscriptAccessor:
         version: str,
         run_snippet: str,
         /,
-        indent: Optional[bool] = None,
+        indent: bool | None = None,
         perform_substitution: bool = True,
     ) -> None:
         """Provide a snippet to be run when the package is about to be "configured" after an upgrade from a given version
@@ -637,7 +944,7 @@ class MaintscriptAccessor:
         self,
         run_snippet: str,
         /,
-        indent: Optional[bool] = None,
+        indent: bool | None = None,
         perform_substitution: bool = True,
     ) -> None:
         """Provide a snippet to be run when the package is about to be removed
@@ -663,7 +970,7 @@ class MaintscriptAccessor:
         self,
         run_snippet: str,
         /,
-        indent: Optional[bool] = None,
+        indent: bool | None = None,
         perform_substitution: bool = True,
     ) -> None:
         """Provide a snippet to be run when the package has been removed
@@ -691,7 +998,7 @@ class MaintscriptAccessor:
         self,
         run_snippet: str,
         /,
-        indent: Optional[bool] = None,
+        indent: bool | None = None,
         perform_substitution: bool = True,
     ) -> None:
         """Provide a snippet to be run when the package is being purged.
@@ -1020,7 +1327,7 @@ class VirtualPath:
         """
         return bool(self.mode & 0o0111)
 
-    def chmod(self, new_mode: Union[int, str]) -> None:
+    def chmod(self, new_mode: int | str) -> None:
         """Set the file mode of this path
 
         This is similar to setting the `mode` attribute. However, this method accepts
@@ -1120,14 +1427,14 @@ class VirtualPath:
         *,
         byte_io: bool,
         buffering: int = -1,
-    ) -> Union[TextIO, BinaryIO]: ...
+    ) -> TextIO | BinaryIO: ...
 
     def open(
         self,
         *,
         byte_io: bool = False,
         buffering: int = -1,
-    ) -> Union[TextIO, BinaryIO]:
+    ) -> TextIO | BinaryIO:
         """Open the file for reading.  Usually used with a context manager
 
         By default, the file is opened in text mode (utf-8). Binary mode can be requested
@@ -1151,7 +1458,7 @@ class VirtualPath:
 
         if byte_io:
             return open(self.fs_path, "rb", buffering=buffering)
-        return open(self.fs_path, "rt", encoding="utf-8", buffering=buffering)
+        return open(self.fs_path, encoding="utf-8", buffering=buffering)
 
     @property
     def fs_path(self) -> str:
@@ -1269,7 +1576,7 @@ class VirtualPath:
         unlink_if_exists: bool = True,
         use_fs_path_mode: bool = False,
         mode: int = 0o0644,
-        mtime: Optional[float] = None,
+        mtime: float | None = None,
     ) -> ContextManager["VirtualPath"]:
         """Add a new regular file as a child of this path
 
@@ -1389,7 +1696,7 @@ class VirtualPath:
         """
         raise NotImplementedError
 
-    def interpreter(self) -> Optional[Interpreter]:
+    def interpreter(self) -> Interpreter | None:
         """Determine the interpreter of the file (`#!`-line details)
 
         Note: this method is only applicable for files (`is_file` is True).
@@ -1406,7 +1713,7 @@ class VirtualPath:
 
     def metadata(
         self,
-        metadata_type: Type[PMT],
+        metadata_type: type[PMT],
     ) -> PathMetadataReference[PMT]:
         """Fetch the path metadata reference to access the underlying metadata
 
@@ -1462,7 +1769,7 @@ class FlushableSubstvars(Substvars):
             yield tmp.name
             # Re-open; seek did not work when I last tried (if I did it work, feel free to
             # convert back to seek - as long as it works!)
-            with open(tmp.name, "rt", encoding="utf-8") as fd:
+            with open(tmp.name, encoding="utf-8") as fd:
                 self.read_substvars(fd)
 
     def save(self) -> None:
@@ -1480,14 +1787,14 @@ class ServiceRegistry(Generic[DSD]):
     def register_service(
         self,
         path: VirtualPath,
-        name: Union[str, List[str]],
+        name: str | list[str],
         *,
         type_of_service: str = "service",  # "timer", etc.
         service_scope: str = "system",
         enable_by_default: bool = True,
         start_by_default: bool = True,
         default_upgrade_rule: ServiceUpgradeRule = "restart",
-        service_context: Optional[DSD] = None,
+        service_context: DSD | None = None,
     ) -> None:
         """Register a service detected in the package
 
@@ -1536,8 +1843,8 @@ class ServiceRegistry(Generic[DSD]):
 
 @dataclasses.dataclass(slots=True, frozen=True)
 class ParserAttributeDocumentation:
-    attributes: FrozenSet[str]
-    description: Optional[str]
+    attributes: frozenset[str]
+    description: str | None
 
     @property
     def is_hidden(self) -> bool:
@@ -1567,12 +1874,12 @@ def undocumented_attr(attr: str) -> ParserAttributeDocumentation:
 
 @dataclasses.dataclass(slots=True, frozen=True)
 class ParserDocumentation:
-    synopsis: Optional[str] = None
-    title: Optional[str] = None
-    description: Optional[str] = None
-    attribute_doc: Optional[Sequence[ParserAttributeDocumentation]] = None
-    alt_parser_description: Optional[str] = None
-    documentation_reference_url: Optional[str] = None
+    synopsis: str | None = None
+    title: str | None = None
+    description: str | None = None
+    attribute_doc: Sequence[ParserAttributeDocumentation] | None = None
+    alt_parser_description: str | None = None
+    documentation_reference_url: str | None = None
 
     def replace(self, **changes: Any) -> "ParserDocumentation":
         return dataclasses.replace(self, **changes)
@@ -1604,7 +1911,7 @@ class TypeMappingExample(Generic[S]):
 
 @dataclasses.dataclass(slots=True, frozen=True)
 class TypeMappingDocumentation(Generic[S]):
-    description: Optional[str] = None
+    description: str | None = None
     examples: Sequence[TypeMappingExample[S]] = tuple()
 
 
@@ -1614,8 +1921,8 @@ def type_mapping_example(source_input: S) -> TypeMappingExample[S]:
 
 def type_mapping_reference_documentation(
     *,
-    description: Optional[str] = None,
-    examples: Union[TypeMappingExample[S], Iterable[TypeMappingExample[S]]] = tuple(),
+    description: str | None = None,
+    examples: TypeMappingExample[S] | Iterable[TypeMappingExample[S]] = tuple(),
 ) -> TypeMappingDocumentation[S]:
     e = (
         tuple([examples])
@@ -1629,7 +1936,7 @@ def type_mapping_reference_documentation(
 
 
 def documented_attr(
-    attr: Union[str, Iterable[str]],
+    attr: str | Iterable[str],
     description: str,
 ) -> ParserAttributeDocumentation:
     """Describe an attribute or a group of attributes
@@ -1654,7 +1961,7 @@ def documented_attr(
 
 def reference_documentation(
     title: str = "Auto-generated reference documentation for ${RULE_NAME}",
-    description: Optional[str] = textwrap.dedent(
+    description: str | None = textwrap.dedent(
         """\
             This is an automatically generated reference documentation for ${RULE_NAME}. It is generated
             from input provided by ${PLUGIN_NAME} via the debputy API.
@@ -1664,10 +1971,10 @@ def reference_documentation(
              the manifest rule.)
             """
     ),
-    attributes: Optional[Sequence[ParserAttributeDocumentation]] = None,
-    non_mapping_description: Optional[str] = None,
-    reference_documentation_url: Optional[str] = None,
-    synopsis: Optional[str] = None,
+    attributes: Sequence[ParserAttributeDocumentation] | None = None,
+    non_mapping_description: str | None = None,
+    reference_documentation_url: str | None = None,
+    synopsis: str | None = None,
 ) -> ParserDocumentation:
     """Provide inline reference documentation for the manifest snippet
 
@@ -1825,7 +2132,7 @@ class ServiceDefinition(Generic[DSD]):
         raise NotImplementedError
 
     @property
-    def service_context(self) -> Optional[DSD]:
+    def service_context(self) -> DSD | None:
         """Custom service context (if any) provided by the detector code of the plugin
 
         :return: If the detection code provided a custom data when registering the
