@@ -1,25 +1,86 @@
+import collections.abc
+import contextlib
 import contextvars
 import functools
 import inspect
-from contextvars import ContextVar
-from typing import Optional, ParamSpec, TypeVar, NoReturn, Union
 from collections.abc import Callable
+from contextvars import ContextVar
+from typing import ParamSpec, TypeVar, NoReturn, Any
 
 from debputy.exceptions import (
     UnhandledOrUnexpectedErrorFromPluginError,
     DebputyRuntimeError,
 )
+from debputy.packages import SourcePackage, BinaryPackage
 from debputy.util import _trace_log, _is_trace_log_enabled
 
 _current_debputy_plugin_cxt_var: ContextVar[str | None] = ContextVar(
     "current_debputy_plugin",
     default=None,
 )
+_current_debputy_parsing_context: ContextVar[
+    tuple[
+        dict[tuple[SourcePackage | BinaryPackage, type[Any]], Any],
+        SourcePackage | BinaryPackage,
+    ]
+    | None
+] = ContextVar(
+    "current_debputy_parsing_context",
+    default=None,
+)
 
 P = ParamSpec("P")
 R = TypeVar("R")
 
 
+def register_manifest_type_value_in_context(
+    value_type: type[Any],
+    value: Any,
+) -> None:
+    context_vars = _current_debputy_parsing_context.get()
+    if context_vars is None:
+        raise AssertionError(
+            "register_manifest_type_value_in_context() was called, but no context was set."
+        )
+    value_table, context_pkg = context_vars
+    if (context_pkg, value_type) in value_table:
+        raise AssertionError(
+            f"The type {value_type!r} was already registered for {context_pkg}, which the plugin API should have prevented"
+        )
+    value_table[(context_pkg, value_type)] = value
+
+
+def begin_parsing_context(
+    value_table: dict[tuple[SourcePackage | BinaryPackage, type[Any]], Any],
+    context_pkg: SourcePackage,
+    func: Callable[P, R],
+    *args: P.args,
+    **kwargs: P.kwargs,
+) -> R:
+    context = contextvars.copy_context()
+    # Wish we could just do a regular set without wrapping it in `context.run`
+    context.run(_current_debputy_parsing_context.set, (value_table, context_pkg))
+    assert context.get(_current_debputy_parsing_context) == (value_table, context_pkg)
+    return context.run(func, *args, **kwargs)
+
+
+@contextlib.contextmanager
+def with_binary_pkg_parsing_context(
+    context_pkg: BinaryPackage,
+) -> collections.abc.Iterator[None]:
+    context_vars = _current_debputy_parsing_context.get()
+    if context_vars is None:
+        raise AssertionError(
+            "with_binary_pkg_parsing_context() was called, but no context was set."
+        )
+    value_table, _ = context_vars
+    token = _current_debputy_parsing_context.set((value_table, context_pkg))
+    try:
+        yield
+    finally:
+        _current_debputy_parsing_context.reset(token)
+
+
 def current_debputy_plugin_if_present() -> str | None:
     return _current_debputy_plugin_cxt_var.get()
 
