@@ -1,21 +1,30 @@
 import dataclasses
 import os
+import re
+import textwrap
 from typing import (
     Any,
-    List,
     NotRequired,
     Union,
     Literal,
     TypedDict,
     Annotated,
-    Optional,
-    FrozenSet,
     Self,
     cast,
 )
 
+from debian.deb822 import PkgRelation
+
+from debputy._manifest_constants import (
+    MK_INSTALLATION_SEARCH_DIRS,
+    MK_BINARY_VERSION,
+    MK_SERVICES,
+)
 from debputy.maintscript_snippet import DpkgMaintscriptHelperCommand, MaintscriptSnippet
-from debputy.manifest_parser.base_types import FileSystemExactMatchRule
+from debputy.manifest_parser.base_types import (
+    DebputyParsedContentStandardConditional,
+    FileSystemExactMatchRule,
+)
 from debputy.manifest_parser.declarative_parser import ParserGenerator
 from debputy.manifest_parser.exceptions import ManifestParseException
 from debputy.manifest_parser.parse_hints import DebputyParseHint
@@ -36,6 +45,12 @@ from debputy.plugin.api.spec import (
     INTEGRATION_MODE_DH_DEBPUTY_RRR,
     not_integrations,
 )
+from debputy.plugins.debputy.types import (
+    BuiltUsingBase,
+    BuiltUsing,
+    StaticBuiltUsing,
+    MatchedBuiltUsingRelation,
+)
 from debputy.transformation_rules import TransformationRule
 from debputy.util import _error, manifest_format_doc
 
@@ -69,38 +84,42 @@ ACCEPTABLE_CLEAN_ON_REMOVAL_IF_EXACT_MATCH_OR_SUBDIR_OF = frozenset(
 def register_binary_package_rules(api: DebputyPluginInitializerProvider) -> None:
     api.pluggable_manifest_rule(
         OPARSER_PACKAGES,
-        "binary-version",
+        MK_BINARY_VERSION,
         BinaryVersionParsedFormat,
         _parse_binary_version,
         source_format=str,
+        register_value=False,
     )
 
     api.pluggable_manifest_rule(
         OPARSER_PACKAGES,
         "transformations",
-        List[TransformationRule],
+        list[TransformationRule],
         _unpack_list,
+        register_value=False,
     )
 
     api.pluggable_manifest_rule(
         OPARSER_PACKAGES,
         "conffile-management",
-        List[DpkgMaintscriptHelperCommand],
+        list[DpkgMaintscriptHelperCommand],
         _unpack_list,
         expected_debputy_integration_mode=not_integrations(
             INTEGRATION_MODE_DH_DEBPUTY_RRR
         ),
+        register_value=False,
     )
 
     api.pluggable_manifest_rule(
         OPARSER_PACKAGES,
-        "services",
-        List[ServiceRuleParsedFormat],
+        MK_SERVICES,
+        list[ServiceRuleParsedFormat],
         _process_service_rules,
-        source_format=List[ServiceRuleSourceFormat],
+        source_format=list[ServiceRuleSourceFormat],
         expected_debputy_integration_mode=not_integrations(
             INTEGRATION_MODE_DH_DEBPUTY_RRR
         ),
+        register_value=False,
     )
 
     api.pluggable_manifest_rule(
@@ -110,21 +129,145 @@ def register_binary_package_rules(api: DebputyPluginInitializerProvider) -> None
         _parse_clean_after_removal,
         # FIXME: debputy won't see the attributes for this one :'(
         # (update `debputy_docs.yaml` when fixed)
-        source_format=List[Any],
+        source_format=list[Any],
         expected_debputy_integration_mode=not_integrations(
             INTEGRATION_MODE_DH_DEBPUTY_RRR
         ),
+        register_value=False,
     )
 
     api.pluggable_manifest_rule(
         OPARSER_PACKAGES,
-        "installation-search-dirs",
+        MK_INSTALLATION_SEARCH_DIRS,
         InstallationSearchDirsParsedFormat,
         _parse_installation_search_dirs,
-        source_format=List[FileSystemExactMatchRule],
+        source_format=list[FileSystemExactMatchRule],
         expected_debputy_integration_mode=not_integrations(
             INTEGRATION_MODE_DH_DEBPUTY_RRR
         ),
+        register_value=False,
+    )
+
+    api.pluggable_manifest_rule(
+        rule_type=OPARSER_PACKAGES,
+        rule_name="built-using",
+        parsed_format=list[BuiltUsingParsedFormat],
+        handler=_parse_built_using,
+        expected_debputy_integration_mode=not_integrations("dh-sequence-zz-debputy-rrr"),
+        inline_reference_documentation=reference_documentation(
+            title="Built-Using dependency relations (`$RULE_NAME`)",
+            description=textwrap.dedent(
+                """\
+                Generate a `Built-Using` dependency relation on the
+                build dependencies selected by the `sources-for`, which
+                may contain a `*` wildcard matching any number of
+                arbitrary characters.
+
+                The `built-using` should be used for static linking
+                where license of dependency libraries require the
+                exact source to be retained. Usually these libraries
+                will be under the license terms like GNU GPL.
+
+                packages:
+                  PKG:
+                    $RULE_NAME:
+                    - sources-for: foo-*-source   # foo-3.1.0-source
+                    - sources-for: librust-*-dev  # several matches
+                    - sources-for: foo
+                      when:                       # foo is always installed
+                        arch-matches: amd64       # but only used on amd64
+
+                Either of these conditions prevents the generation:
+                * PKG is not part of the current build because of its
+                  `Architecture` or `Build-Profiles` fields.
+                * The match in `Build-Depends` carries an
+                  architecture or build profile restriction that does
+                  not match the current run.
+                * The match in `Build-Depends` is not installed.
+                  This should only happen inside alternatives, see below.
+                * The manifest item carries a `when:` condition that
+                  evaluates to false. This may be useful when the match
+                  must be installed for unrelated reasons.
+
+                Matches are searched in the `Build-Depends` field of
+                the source package, and either `Build-Depends-Indep`
+                or `Build-Depends-Arch` depending on PKG.
+
+                In alternatives like `a | b`, each option may match
+                separately.  This is a compromise between
+                reproducibility on automatic builders (where the set
+                of installed package is constant), and least surprise
+                during local builds (where `b` may be installed
+                alone). There seems to be no one-size fits all
+                solution when both are installed.
+
+                Architecture qualifiers and version restrictions in
+                `Build-Depends` are ignored. The only allowed
+                co-installations require a common source and version.
+                """,
+            ),
+        ),
+    )
+
+    api.pluggable_manifest_rule(
+        rule_type=OPARSER_PACKAGES,
+        rule_name="static-built-using",
+        parsed_format=list[BuiltUsingParsedFormat],
+        handler=_parse_static_built_using,
+        expected_debputy_integration_mode=not_integrations("dh-sequence-zz-debputy-rrr"),
+        inline_reference_documentation=reference_documentation(
+            title="Static-Built-Using dependency relations (`$RULE_NAME`)",
+            description=textwrap.dedent(
+                """\
+                Generate a `Static-Built-Using` dependency relation on the
+                build dependencies selected by the `sources-for`, which
+                may contain a `*` wildcard matching any number of
+                arbitrary characters.
+
+                The `static-built-using` should be used for static linking
+                where license of dependency libraries do not require the
+                exact source to be retained. This is usually libraries under
+                permissive libraries like Apache-2.0 or MIT/X11/Expat.
+
+                packages:
+                  PKG:
+                    $RULE_NAME:
+                    - sources-for: foo-*-source   # foo-3.1.0-source
+                    - sources-for: librust-*-dev  # several matches
+                    - sources-for: foo
+                      when:                       # foo is always installed
+                        arch-matches: amd64       # but only used on amd64
+
+                Either of these conditions prevents the generation:
+                * PKG is not part of the current build because of its
+                  `Architecture` or `Build-Profiles` fields.
+                * The match in `Build-Depends` carries an
+                  architecture or build profile restriction that does
+                  not match the current run.
+                * The match in `Build-Depends` is not installed.
+                  This should only happen inside alternatives, see below.
+                * The manifest item carries a `when:` condition that
+                  evaluates to false. This may be useful when the match
+                  must be installed for unrelated reasons.
+
+                Matches are searched in the `Build-Depends` field of
+                the source package, and either `Build-Depends-Indep`
+                or `Build-Depends-Arch` depending on PKG.
+
+                In alternatives like `a | b`, each option may match
+                separately.  This is a compromise between
+                reproducibility on automatic builders (where the set
+                of installed package is constant), and least surprise
+                during local builds (where `b` may be installed
+                alone). There seems to be no one-size fits all
+                solution when both are installed.
+
+                Architecture qualifiers and version restrictions in
+                `Build-Depends` are ignored. The only allowed
+                co-installations require a common source and version.
+                """,
+            ),
+        ),
     )
 
 
@@ -138,7 +281,7 @@ class ServiceRuleSourceFormat(TypedDict):
     service_manager: NotRequired[
         Annotated[str, DebputyParseHint.target_attribute("service_managers")]
     ]
-    service_managers: NotRequired[List[str]]
+    service_managers: NotRequired[list[str]]
 
 
 class ServiceRuleParsedFormat(DebputyParsedContent):
@@ -148,7 +291,7 @@ class ServiceRuleParsedFormat(DebputyParsedContent):
     enable_on_install: NotRequired[bool]
     start_on_install: NotRequired[bool]
     on_upgrade: NotRequired[ServiceUpgradeRule]
-    service_managers: NotRequired[List[str]]
+    service_managers: NotRequired[list[str]]
 
 
 @dataclasses.dataclass(slots=True, frozen=True)
@@ -157,10 +300,10 @@ class ServiceRule:
     service: str
     type_of_service: str
     service_scope: Literal["system", "user"]
-    enable_on_install: Optional[bool]
-    start_on_install: Optional[bool]
-    on_upgrade: Optional[ServiceUpgradeRule]
-    service_managers: Optional[FrozenSet[str]]
+    enable_on_install: bool | None
+    start_on_install: bool | None
+    on_upgrade: ServiceUpgradeRule | None
+    service_managers: frozenset[str] | None
 
     @classmethod
     def from_service_rule_parsed_format(
@@ -219,20 +362,26 @@ class BinaryVersionParsedFormat(DebputyParsedContent):
     binary_version: str
 
 
+class BuiltUsingParsedFormat(DebputyParsedContentStandardConditional):
+    """Also used for static-built-using."""
+
+    sources_for: str
+
+
 class ListParsedFormat(DebputyParsedContent):
-    elements: List[Any]
+    elements: list[Any]
 
 
 class ListOfTransformationRulesFormat(DebputyParsedContent):
-    elements: List[TransformationRule]
+    elements: list[TransformationRule]
 
 
 class ListOfDpkgMaintscriptHelperCommandFormat(DebputyParsedContent):
-    elements: List[DpkgMaintscriptHelperCommand]
+    elements: list[DpkgMaintscriptHelperCommand]
 
 
 class InstallationSearchDirsParsedFormat(DebputyParsedContent):
-    installation_search_dirs: List[FileSystemExactMatchRule]
+    installation_search_dirs: list[FileSystemExactMatchRule]
 
 
 def _parse_binary_version(
@@ -249,41 +398,139 @@ def _parse_installation_search_dirs(
     parsed_data: InstallationSearchDirsParsedFormat,
     _attribute_path: AttributePath,
     _parser_context: ParserContextData,
-) -> List[FileSystemExactMatchRule]:
+) -> list[FileSystemExactMatchRule]:
     return parsed_data["installation_search_dirs"]
 
 
 def _process_service_rules(
     _name: str,
-    parsed_data: List[ServiceRuleParsedFormat],
+    parsed_data: list[ServiceRuleParsedFormat],
     attribute_path: AttributePath,
     _parser_context: ParserContextData,
-) -> List[ServiceRule]:
+) -> list[ServiceRule]:
     return [
         ServiceRule.from_service_rule_parsed_format(x, attribute_path[i])
         for i, x in enumerate(parsed_data)
     ]
 
 
+def _parse_built_using(
+    name: str,
+    parsed_data: list[BuiltUsingParsedFormat],
+    attribute_path: AttributePath,
+    parser_context: ParserContextData,
+) -> list[BuiltUsing]:
+    return _built_using_handler(
+        BuiltUsing,
+        name,
+        parsed_data,
+        attribute_path,
+        parser_context,
+    )
+
+
+def _parse_static_built_using(
+    name: str,
+    parsed_data: list[BuiltUsingParsedFormat],
+    attribute_path: AttributePath,
+    parser_context: ParserContextData,
+) -> list[StaticBuiltUsing]:
+    return _built_using_handler(
+        StaticBuiltUsing,
+        name,
+        parsed_data,
+        attribute_path,
+        parser_context,
+    )
+
+
+_VALID_BUILT_USING_GLOB = re.compile("[a-z*][a-z0-9.+*-]*")
+_BUILT_USING_GLOB_TO_RE = str.maketrans({".": "[.]", "+": "[+]", "*": ".*"})
+
+
+def _validate_built_using[T: BuiltUsingBase](
+    cls: type[T],
+    parsed_data: BuiltUsingParsedFormat,
+    attribute_path: AttributePath,
+    parser_context: ParserContextData,
+) -> T:
+    # Process an item in a (static-)built-using list.
+    raw_glob = parsed_data["sources_for"]
+    if _VALID_BUILT_USING_GLOB.fullmatch(raw_glob) is None:
+        raise ManifestParseException(
+            f"The glob {raw_glob!r} defined at {attribute_path["sources_for"].path} contained invalid characters."
+            f" It must only characters valid in a package name plus the `*` character"
+        )
+    regex = re.compile(raw_glob.translate(_BUILT_USING_GLOB_TO_RE))
+
+    pkg = parser_context.current_binary_package_state.binary_package
+    if pkg.is_arch_all:
+        other = "Build-Depends-Indep"
+    else:
+        other = "Build-Depends-Arch"
+    # pylint: disable=too-many-nested-blocks
+    matched_packages = []
+    for bd_field in ("Build-Depends", other):
+        raw = parser_context.source_package.fields.get(bd_field)
+        if raw is not None:
+            for options in PkgRelation.parse_relations(raw):
+                options: list[PkgRelation.ParsedRelation]
+                for idx, relation in enumerate(options):
+                    name = relation["name"]
+                    if regex.fullmatch(name):
+                        matched_packages.append(
+                            MatchedBuiltUsingRelation(name, not idx, relation)
+                        )
+    if not matched_packages:
+        raise ManifestParseException(
+            f"The glob {raw_glob!r} defined at {attribute_path["sources_for"].path} does match at least one clause of Build-Depends or {other}."
+            " Either a Build-dependency is missing or the glob fails to match the intended build-dependency, or the glob superfluous and can be removed."
+        )
+    return cls(
+        raw_glob,
+        matched_packages,
+        parsed_data.get("when"),
+        attribute_path,
+    )
+
+
+def _built_using_handler[T: BuiltUsingBase](
+    cls: type[T],
+    _name: str,
+    parsed_data: list[BuiltUsingParsedFormat],
+    attribute_path: AttributePath,
+    parser_context: ParserContextData,
+) -> list[T]:
+    return [
+        _validate_built_using(
+            cls,
+            pd,
+            attribute_path[idx],
+            parser_context,
+        )
+        for idx, pd in enumerate(parsed_data)
+    ]
+
+
 def _unpack_list(
     _name: str,
-    parsed_data: List[Any],
+    parsed_data: list[Any],
     _attribute_path: AttributePath,
     _parser_context: ParserContextData,
-) -> List[Any]:
+) -> list[Any]:
     return parsed_data
 
 
 class CleanAfterRemovalRuleSourceFormat(TypedDict):
     path: NotRequired[Annotated[str, DebputyParseHint.target_attribute("paths")]]
-    paths: NotRequired[List[str]]
+    paths: NotRequired[list[str]]
     delete_on: NotRequired[Literal["purge", "removal"]]
     recursive: NotRequired[bool]
     ignore_non_empty_dir: NotRequired[bool]
 
 
 class CleanAfterRemovalRule(DebputyParsedContent):
-    paths: List[str]
+    paths: list[str]
     delete_on: NotRequired[Literal["purge", "removal"]]
     recursive: NotRequired[bool]
     ignore_non_empty_dir: NotRequired[bool]
@@ -293,7 +540,7 @@ class CleanAfterRemovalRule(DebputyParsedContent):
 #  complex types that is registered by plugins, so it will work for now.
 _CLEAN_AFTER_REMOVAL_RULE_PARSER = ParserGenerator().generate_parser(
     CleanAfterRemovalRule,
-    source_content=Union[CleanAfterRemovalRuleSourceFormat, str, List[str]],
+    source_content=Union[CleanAfterRemovalRuleSourceFormat, str, list[str]],
     inline_reference_documentation=reference_documentation(
         reference_documentation_url=manifest_format_doc(
             "remove-runtime-created-paths-on-purge-or-post-removal-clean-after-removal"
@@ -372,7 +619,7 @@ def _parse_clean_after_removal(
                     f" Please restrict this pattern at {definition_source.path} considerably."
                 )
             is_subdir_match = False
-            matched_directory: Optional[str]
+            matched_directory: str | None
             if isinstance(match_rule, ExactFileSystemPath):
                 matched_directory = (
                     os.path.dirname(match_rule.path)
