@@ -9,11 +9,6 @@ from typing import (
     Self,
     cast,
     Dict,
-    Mapping,
-    Sequence,
-    MutableMapping,
-    Iterable,
-    Container,
     List,
     Tuple,
     Union,
@@ -22,6 +17,7 @@ from typing import (
     Literal,
     Required,
 )
+from collections.abc import Mapping, Sequence, MutableMapping, Iterable, Container
 
 from debian.debian_support import Version
 
@@ -100,8 +96,9 @@ def register_build_keywords(api: DebputyPluginInitializerProvider) -> None:
     api.pluggable_manifest_rule(
         OPARSER_MANIFEST_ROOT,
         "build-environments",
-        List[NamedEnvironmentSourceFormat],
+        list[NamedEnvironmentSourceFormat],
         _parse_build_environments,
+        register_value=False,
         expected_debputy_integration_mode=only_integrations(INTEGRATION_MODE_FULL),
         inline_reference_documentation=reference_documentation(
             title="Build Environments (`build-environments`)",
@@ -196,6 +193,7 @@ def register_build_keywords(api: DebputyPluginInitializerProvider) -> None:
         "default-build-environment",
         EnvironmentSourceFormat,
         _parse_default_environment,
+        register_value=False,
         expected_debputy_integration_mode=only_integrations(INTEGRATION_MODE_FULL),
         inline_reference_documentation=reference_documentation(
             title="Default Build Environment (`default-build-environment`)",
@@ -274,8 +272,9 @@ def register_build_keywords(api: DebputyPluginInitializerProvider) -> None:
     api.pluggable_manifest_rule(
         OPARSER_MANIFEST_ROOT,
         MK_BUILDS,
-        List[BuildRule],
+        list[BuildRule],
         _handle_build_rules,
+        register_value=False,
         expected_debputy_integration_mode=only_integrations(INTEGRATION_MODE_FULL),
         inline_reference_documentation=reference_documentation(
             title=f"Build rules (`{MK_BUILDS}`)",
@@ -403,9 +402,9 @@ def register_build_rules(api: DebputyPluginInitializerProvider) -> None:
 
 
 class EnvironmentSourceFormat(TypedDict):
-    set: NotRequired[Dict[str, str]]
-    override: NotRequired[Dict[str, str]]
-    unset: NotRequired[List[str]]
+    set: NotRequired[dict[str, str]]
+    override: NotRequired[dict[str, str]]
+    unset: NotRequired[list[str]]
 
 
 class NamedEnvironmentSourceFormat(EnvironmentSourceFormat):
@@ -453,7 +452,7 @@ def _check_variables(
 
 
 def _no_overlap(
-    lhs: Iterable[Union[str, Tuple[int, str]]],
+    lhs: Iterable[str | tuple[int, str]],
     rhs: Container[str],
     lhs_key: str,
     rhs_key: str,
@@ -498,7 +497,7 @@ class ManifestProvidedBuildEnvironment(BuildEnvironmentDefinition):
         parser_context: ParserContextData,
         is_default: bool = False,
     ) -> Self:
-        reference_name: Optional[str]
+        reference_name: str | None
         if is_default:
             name = "default-env"
             reference_name = None
@@ -696,7 +695,7 @@ class MakefileBuildSystemRule(StepBasedBuildSystemRule):
     def _run_make_maybe_explicit_target(
         self,
         context: "BuildContext",
-        provided_target: Optional[str],
+        provided_target: str | None,
         fallback_targets: Sequence[str],
         *make_args: str,
     ) -> None:
@@ -760,7 +759,7 @@ class PerlBuildBuildSystemRule(StepBasedBuildSystemRule):
     @staticmethod
     def _perl_cross_build_env(
         context: "BuildContext",
-    ) -> Tuple[PerlConfigVars, Optional[EnvironmentModification]]:
+    ) -> tuple[PerlConfigVars, EnvironmentModification | None]:
         perl_config_data = resolve_perl_config(
             context.dpkg_architecture_variables,
             None,
@@ -1063,7 +1062,7 @@ class DebhelperBuildSystemRule(StepBasedBuildSystemRule):
         else:
             d = json.loads(v)
             build_system = d.get("build-system")
-            return build_system is not None
+            return build_system is not None and build_system != "none"
 
     @classmethod
     def characteristics(cls) -> BuildSystemCharacteristics:
@@ -1092,7 +1091,7 @@ class DebhelperBuildSystemRule(StepBasedBuildSystemRule):
                 f" `dh_assistant which-build-system -S{dh_build_system}`"
             ) from None
 
-    def _default_options(self) -> List[str]:
+    def _default_options(self) -> list[str]:
         default_options = []
         if self.dh_build_system is not None:
             default_options.append(f"-S{self.dh_build_system}")
@@ -1400,7 +1399,7 @@ class CMakeBuildSystemRule(StepBasedBuildSystemRule):
     @staticmethod
     def _compiler_and_cross_flags(
         context: "BuildContext",
-        cmake_flags: List[str],
+        cmake_flags: list[str],
     ) -> None:
 
         if "CC" in os.environ:
@@ -1672,7 +1671,7 @@ class MesonBuildSystemRule(StepBasedBuildSystemRule):
     @staticmethod
     def _cross_flags(
         context: "BuildContext",
-        meson_flags: List[str],
+        meson_flags: list[str],
     ) -> None:
         if not context.is_cross_compiling:
             return
@@ -1817,7 +1816,7 @@ class MesonBuildSystemRule(StepBasedBuildSystemRule):
         assert self.out_of_source_build
 
 
-def _add_qmake_flag(options: List[str], envvar: str, *, include_cppflags: bool) -> None:
+def _add_qmake_flag(options: list[str], envvar: str, *, include_cppflags: bool) -> None:
     value = os.environ.get(envvar)
     if value is None:
         return
@@ -1836,7 +1835,7 @@ class ParsedGenericQmakeBuildRuleDefinition(
     OptionalBuildDirectory,
     OptionalTestRule,
 ):
-    configure_args: NotRequired[List[str]]
+    configure_args: NotRequired[list[str]]
 
 
 class AbstractQmakeBuildSystemRule(StepBasedBuildSystemRule):
@@ -2133,7 +2132,7 @@ class ParsedAutoconfBuildRuleDefinition(
     OptionalBuildDirectory,
     OptionalTestRule,
 ):
-    configure_args: NotRequired[List[str]]
+    configure_args: NotRequired[list[str]]
 
 
 @debputy_build_system(
@@ -2186,7 +2185,7 @@ class ParsedCMakeBuildRuleDefinition(
     OptionalBuildDirectory,
     OptionalTestRule,
 ):
-    configure_args: NotRequired[List[str]]
+    configure_args: NotRequired[list[str]]
     target_build_system: Literal["make", "ninja"]
 
 
@@ -2227,7 +2226,7 @@ class ParsedMesonBuildRuleDefinition(
     OptionalBuildDirectory,
     OptionalTestRule,
 ):
-    configure_args: NotRequired[List[str]]
+    configure_args: NotRequired[list[str]]
 
 
 @debputy_build_system(
@@ -2270,7 +2269,7 @@ class ParsedPerlBuildBuildRuleDefinition(
     OptionalInstallDirectly,
     OptionalTestRule,
 ):
-    configure_args: NotRequired[List[str]]
+    configure_args: NotRequired[list[str]]
 
 
 @debputy_build_system(
@@ -2327,7 +2326,7 @@ class ParsedDebhelperBuildRuleDefinition(
     OptionalBuildDirectory,
     OptionalTestRule,
 ):
-    configure_args: NotRequired[List[str]]
+    configure_args: NotRequired[list[str]]
     dh_build_system: NotRequired[str]
 
 
@@ -2370,7 +2369,7 @@ class ParsedPerlMakeMakerBuildRuleDefinition(
     OptionalInstallDirectly,
     OptionalTestRule,
 ):
-    configure_args: NotRequired[List[str]]
+    configure_args: NotRequired[list[str]]
 
 
 @debputy_build_system(
@@ -2467,10 +2466,10 @@ def _parse_default_environment(
 
 def _parse_build_environments(
     _name: str,
-    parsed_data: List[NamedEnvironmentSourceFormat],
+    parsed_data: list[NamedEnvironmentSourceFormat],
     attribute_path: AttributePath,
     parser_context: ParserContextData,
-) -> List[ManifestProvidedBuildEnvironment]:
+) -> list[ManifestProvidedBuildEnvironment]:
     return [
         ManifestProvidedBuildEnvironment.from_environment_definition(
             value,
@@ -2484,8 +2483,8 @@ def _parse_build_environments(
 
 def _handle_build_rules(
     _name: str,
-    parsed_data: List[BuildRule],
+    parsed_data: list[BuildRule],
     _attribute_path: AttributePath,
     _parser_context: ParserContextData,
-) -> List[BuildRule]:
+) -> list[BuildRule]:
     return parsed_data
