@@ -1,9 +1,17 @@
+import dataclasses
 import itertools
 import os
 import re
+import subprocess
 import textwrap
-from collections.abc import Iterable, Iterator
+from collections.abc import Iterable, Iterator, Mapping
 
+from debian.deb822 import PkgRelation
+
+from debputy.bug1120283 import holds_with_profiles, holds_on_arch
+from debputy.manifest_conditions import ManifestCondition
+from debputy.manifest_parser.util import AttributePath
+from debputy.packages import BinaryPackage
 from debputy.packaging.alternatives import SYSTEM_DEFAULT_PATH_DIRS
 from debputy.plugin.api import (
     VirtualPath,
@@ -16,8 +24,13 @@ from debputy.plugins.debputy.paths import (
     GSETTINGS_SCHEMA_DIR,
     SYSTEMD_SYSUSERS_DIR,
 )
-from debputy.plugins.debputy.types import DebputyCapability
-from debputy.util import assume_not_none, _warn
+from debputy.plugins.debputy.types import (
+    DebputyCapability,
+    BuiltUsing,
+    StaticBuiltUsing,
+    BuiltUsingBase,
+)
+from debputy.util import assume_not_none, _debug_log, _error, _warn
 
 DPKG_ROOT = '"${DPKG_ROOT}"'
 DPKG_ROOT_UNQUOTED = "${DPKG_ROOT}"
@@ -566,3 +579,154 @@ def auto_depends_arch_any_solink(
         # If both packages are arch:all or both are arch:any, we can generate a tight dependency
         relation = f"{pkg_dep.name} (= ${{binary:Version}})"
         ctrl.substvars.add_dependency("misc:Depends", relation)
+
+
+def _built_using_enabled_match(
+    context: PackageProcessingContext,
+    pkg: BinaryPackage,
+    attribute_path: AttributePath,
+    manifest_condition: ManifestCondition,
+    relation: "PkgRelation.ParsedRelation",
+) -> bool:
+    # A logical conjunction of restrictions, with logging.
+    name = relation["name"]
+    host = pkg.resolved_architecture
+    table = context.dpkg_arch_query_table
+    if not holds_on_arch(relation, host, table):
+        field = attribute_path["sources"]
+        _debug_log(f"{field}: {name} is disabled by host architecture {host}.")
+        return False
+    profiles = context.deb_options_and_profiles.deb_build_profiles
+    if not holds_with_profiles(relation, profiles):
+        field = attribute_path["sources"]
+        _debug_log(f"{field}: {name} is disabled by profiles {' '.join(profiles)}.")
+        return False
+    if manifest_condition and not manifest_condition.evaluate(
+        context.condition_context(pkg)
+    ):
+        field = attribute_path["sources"]
+        _debug_log(f"{field}: {name} is disabled by its manifest condition.")
+        return False
+    return True
+
+
+@dataclasses.dataclass(slots=True)
+class _BuiltUsingTodo:
+    """For efficiency, a first pass constructs a todo list, then at
+    most one dpkg-query subprocess is spawn."""
+
+    pkg: BinaryPackage
+    field: str
+    dep: str
+    first_option: bool  # This relation was the first in its alternative.
+    attribute_path: AttributePath
+
+
+def _split_dpkg_output(dpkg_output: str) -> Iterable[tuple[str, str]]:
+    for line in dpkg_output.splitlines(keepends=False):
+        status, package_name, source_and_version = line.split("\x1f")
+        if status[1] == "i":
+            yield package_name, source_and_version
+
+
+def _sources_for(
+    deps: Iterable[str],
+) -> Mapping[str, str]:
+    """Map installed packages among deps to a "source (= version)"
+    relation, excluding unknown or not installed packages.
+
+    >>> r = _sources_for(("dpkg", "dpkg", "dpkg-dev", "gcc", "dummy"))
+    >>> r["dpkg"]  # doctest: +ELLIPSIS
+    'dpkg (= ...)'
+    >>> r["dpkg-dev"]  # doctest: +ELLIPSIS
+    'dpkg (= ...)'
+    >>> r["gcc"]  # doctest: +ELLIPSIS
+    'gcc-defaults (= ...)'
+    >>> "dummy" in r
+    False
+    """
+    cp = subprocess.run(
+        args=(
+            "dpkg-query",
+            "-Wf${db:Status-Abbrev}\x1f${Package}\x1f${source:Package} (= ${source:Version})\n",
+            *deps,
+        ),
+        capture_output=True,
+        check=False,
+        text=True,
+    )
+    # 0: OK    1: unknown package    2: other
+    if cp.returncode not in (0, 1):
+        _error(f"dpkg-query -W failed (code: {cp.returncode}): {cp.stderr.rstrip()}")
+    # For the example above, stdout is:
+    # "ii \x1fdpkg\x1fdpkg (= 1.22.21)\n"
+    # "ii \x1fdpkg-dev\x1fdpkg (= 1.22.21)\n"
+    # "ii \x1fgcc\x1fgcc-defaults (= 1.220)\n"
+    #   ^: the package is (i)nstalled
+    return dict(_split_dpkg_output(cp.stdout))
+
+
+def detect_built_using(
+    _fs_foot: VirtualPath,
+    ctrl: BinaryCtrlAccessor,
+    context: PackageProcessingContext,
+) -> None:
+    binary_package = context.binary_package
+    built_using_conf = context.manifest_configuration(binary_package, list[BuiltUsing])
+    static_built_using_conf = context.manifest_configuration(
+        binary_package, list[StaticBuiltUsing]
+    )
+
+    all_built_using_conf: list[BuiltUsingBase] = []
+
+    if built_using_conf:
+        all_built_using_conf.extend(built_using_conf)
+    if static_built_using_conf:
+        all_built_using_conf.extend(static_built_using_conf)
+
+    all_todos = []
+
+    for conf in all_built_using_conf:
+        for matched_package in conf.matched_packages:
+
+            if _built_using_enabled_match(
+                context,
+                binary_package,
+                conf.attribute_path,
+                conf.conditional,
+                matched_package.relationship,
+            ):
+                all_todos.append(
+                    _BuiltUsingTodo(
+                        binary_package,
+                        conf.field_name(),
+                        matched_package.name,
+                        matched_package.is_first,
+                        conf.attribute_path,
+                    )
+                )
+
+    if not all_todos:
+        return
+
+    relevant_sources = _sources_for(sorted({t.dep for t in all_todos}))
+    already_warned = set()
+    for t in all_todos:
+        if t.dep in relevant_sources:
+            ctrl.substvars.add_dependency(
+                f"debputy:{t.field}",
+                relevant_sources[t.dep],
+            )
+            continue
+        # With `Build-Depends: a | b`, in usual configurations,
+        # `a` is installed but `b` might not be.
+        field = t.attribute_path["sources"]
+        if t.first_option and t.dep not in already_warned:
+            _warn(
+                f"{field}: {t.dep} is not installed despite being the first or only option in a Build-Depends clause. It will be omitted from the built-using fields."
+            )
+            already_warned.add(t.dep)
+        else:
+            _debug_log(
+                f"{field}: {t.dep} is not installed and therefore not included in the built-using fields."
+            )
