@@ -20,15 +20,12 @@ from typing import (
     Optional,
     TypeVar,
     Dict,
-    Iterator,
-    Iterable,
     Literal,
     Tuple,
-    Sequence,
     List,
-    Mapping,
     Any,
 )
+from collections.abc import Iterator, Iterable, Sequence, Mapping
 
 from debian.deb822 import Deb822
 
@@ -108,9 +105,9 @@ _WORD_EQUAL = re.compile(r"^-*[\w_\-]+=")
 _DOUBLE_ESCAPEES = re.compile(r'([\n`$"\\])')
 _REGULAR_ESCAPEES = re.compile(r"""([\s!"$()*+#;<>?@'\[\]\\`|~])""")
 _PROFILE_GROUP_SPLIT = re.compile(r">\s+<")
-_DEFAULT_LOGGER: Optional[logging.Logger] = None
-_STDOUT_HANDLER: Optional[logging.StreamHandler[Any]] = None
-_STDERR_HANDLER: Optional[logging.StreamHandler[Any]] = None
+_DEFAULT_LOGGER: logging.Logger | None = None
+_STDOUT_HANDLER: logging.StreamHandler[Any] | None = None
+_STDERR_HANDLER: logging.StreamHandler[Any] | None = None
 PRINT_COMMAND = logging.INFO + 3
 PRINT_BUILD_SYSTEM_COMMAND = PRINT_COMMAND + 3
 TRACE_LOG = logging.DEBUG + 3
@@ -121,7 +118,7 @@ logging.addLevelName(PRINT_BUILD_SYSTEM_COMMAND, "_INFO")
 logging.addLevelName(TRACE_LOG, "TRACE")
 
 
-def assume_not_none(x: Optional[T]) -> T:
+def assume_not_none(x: T | None) -> T:
     if x is None:  # pragma: no cover
         raise ValueError(
             'Internal error: None was given, but the receiver assumed "not None" here'
@@ -172,7 +169,7 @@ def _debug_log(msg: str) -> None:
     # No fallback print for this level
 
 
-def _error(msg: str, *, prog: Optional[str] = None) -> "NoReturn":
+def _error(msg: str, *, prog: str | None = None) -> "NoReturn":
     global _DEFAULT_LOGGER
     logger = _DEFAULT_LOGGER
     if logger:
@@ -186,7 +183,7 @@ def _error(msg: str, *, prog: Optional[str] = None) -> "NoReturn":
     sys.exit(1)
 
 
-def _warn(msg: str, *, prog: Optional[str] = None) -> None:
+def _warn(msg: str, *, prog: str | None = None) -> None:
     global _DEFAULT_LOGGER
     logger = _DEFAULT_LOGGER
     if logger:
@@ -211,7 +208,7 @@ def ensure_dir(path: str) -> None:
         os.makedirs(path, mode=0o755, exist_ok=True)
 
 
-def _clean_path(orig_p: str) -> str:
+def _clean_path(orig_p: str, allow_and_keep_upward_segments: bool = False) -> str:
     p = SLASH_PRUNE.sub("/", orig_p)
     if "." in p:
         path_base = p
@@ -220,21 +217,38 @@ def _clean_path(orig_p: str) -> str:
         if path_base.startswith("./"):
             path_base = path_base[2:]
             assert path_base
-        for segment in path_base.split("/"):
-            if segment in (".", ".."):
-                raise ValueError(
-                    'Please provide paths that are normalized (i.e., no ".." or ".").'
-                    f' Offending input "{orig_p}"'
-                )
+        if allow_and_keep_upward_segments:
+            stack = []
+            for segment in path_base.split("/"):
+                if segment == ".":
+                    continue
+                stack.append(segment)
+            p = "/".join(stack)
+            if path_base.startswith("/"):
+                p = "/" + p
+        else:
+            for segment in path_base.split("/"):
+                if segment in (".", ".."):
+                    raise ValueError(
+                        'Please provide paths that are normalized (i.e., no ".." or ".").'
+                        f' Offending input "{orig_p}"'
+                    )
     return p
 
 
-def _normalize_path(path: str, with_prefix: bool = True) -> str:
+def _normalize_path(
+    path: str,
+    with_prefix: bool = True,
+    allow_and_keep_upward_segments: bool = False,
+) -> str:
     path = path.strip("/")
     if not path or path == ".":
         return "."
     if "//" in path or "." in path:
-        path = _clean_path(path)
+        path = _clean_path(
+            path,
+            allow_and_keep_upward_segments=allow_and_keep_upward_segments,
+        )
     if with_prefix ^ path.startswith("./"):
         if with_prefix:
             path = "./" + path
@@ -245,7 +259,7 @@ def _normalize_path(path: str, with_prefix: bool = True) -> str:
 
 def _normalize_link_target(link_target: str) -> str:
     link_target = SLASH_PRUNE.sub("/", link_target.lstrip("/"))
-    result: List[str] = []
+    result: list[str] = []
     for segment in link_target.split("/"):
         if segment in (".", ""):
             # Ignore these - the empty string is generally a trailing slash
@@ -287,7 +301,7 @@ def escape_shell(*args: str) -> str:
 
 def render_command(
     *args: str,
-    cwd: Optional[str] = None,
+    cwd: str | None = None,
     env_mod: Optional["EnvironmentModification"] = None,
 ) -> str:
     env_mod_prefix = ""
@@ -310,7 +324,7 @@ def render_command(
 
 def print_command(
     *args: str,
-    cwd: Optional[str] = None,
+    cwd: str | None = None,
     env_mod: Optional["EnvironmentModification"] = None,
     print_at_log_level: int = PRINT_COMMAND,
 ) -> None:
@@ -335,8 +349,8 @@ def print_command(
 
 def run_command(
     *args: str,
-    cwd: Optional[str] = None,
-    env: Optional[Mapping[str, str]] = None,
+    cwd: str | None = None,
+    env: Mapping[str, str] | None = None,
     env_mod: Optional["EnvironmentModification"] = None,
     print_at_log_level: int = PRINT_COMMAND,
     raise_file_not_found_on_missing_command: bool = False,
@@ -371,8 +385,8 @@ def run_command(
 
 def run_build_system_command(
     *args: str,
-    cwd: Optional[str] = None,
-    env: Optional[Mapping[str, str]] = None,
+    cwd: str | None = None,
+    env: Mapping[str, str] | None = None,
     env_mod: Optional["EnvironmentModification"] = None,
     print_at_log_level: int = PRINT_BUILD_SYSTEM_COMMAND,
     raise_file_not_found_on_missing_command: bool = False,
@@ -464,9 +478,10 @@ def glob_escape(replacement_value: str) -> str:
 
 
 # TODO: This logic should probably be moved to `python-debian`
+# See bug1120283.py.
 def active_profiles_match(
     profiles_raw: str,
-    active_build_profiles: Union[Set[str], FrozenSet[str]],
+    active_build_profiles: set[str] | frozenset[str],
 ) -> bool:
     profiles_raw = profiles_raw.strip()
     if profiles_raw[0] != "<" or profiles_raw[-1] != ">" or profiles_raw == "<>":
@@ -493,7 +508,7 @@ def active_profiles_match(
     return False
 
 
-def _parse_build_profiles(build_profiles_raw: str) -> FrozenSet[FrozenSet[str]]:
+def _parse_build_profiles(build_profiles_raw: str) -> frozenset[frozenset[str]]:
     profiles_raw = build_profiles_raw.strip()
     if profiles_raw[0] != "<" or profiles_raw[-1] != ">" or profiles_raw == "<>":
         raise ValueError(
@@ -504,7 +519,7 @@ def _parse_build_profiles(build_profiles_raw: str) -> FrozenSet[FrozenSet[str]]:
 
 
 def resolve_source_date_epoch(
-    command_line_value: Optional[int],
+    command_line_value: int | None,
     *,
     substitution: Optional["Substitution"] = None,
 ) -> int:
@@ -530,12 +545,15 @@ def resolve_source_date_epoch(
 
 
 def compute_output_filename(control_root_dir: str, is_udeb: bool) -> str:
-    with open(os.path.join(control_root_dir, "control"), "rt") as fd:
+    with open(os.path.join(control_root_dir, "control")) as fd:
         control_file = Deb822(fd)
 
     package_name = control_file["Package"]
     package_version = control_file["Version"]
-    package_architecture = control_file["Architecture"]
+    if "Architecture-Variant" in control_file:
+        package_architecture = control_file["Architecture-Variant"]
+    else:
+        package_architecture = control_file["Architecture"]
     extension = control_file.get("Package-Type") or "deb"
     if ":" in package_version:
         package_version = package_version.split(":", 1)[1]
@@ -584,13 +602,13 @@ def generated_root_directory(path: str, *, internal_only: bool = False) -> None:
         )
 
 
-_RUNTIME_CONTAINER_DIR_KEY: Optional[str] = None
+_RUNTIME_CONTAINER_DIR_KEY: str | None = None
 
 
 def generated_content_dir(
     *,
     package: Optional["BinaryPackage"] = None,
-    subdir_key: Optional[str] = None,
+    subdir_key: str | None = None,
 ) -> str:
     global _RUNTIME_CONTAINER_DIR_KEY
     container_dir = _RUNTIME_CONTAINER_DIR_KEY
@@ -626,7 +644,7 @@ PerlConfigVars = collections.namedtuple(
     "PerlConfigVars", ["vendorlib", "vendorarch", "cross_inc_dir", "ld", "path_sep"]
 )
 PerlConfigData = collections.namedtuple("PerlConfigData", ["version", "debian_abi"])
-_PERL_MODULE_DIRS: Dict[str, PerlConfigVars] = {}
+_PERL_MODULE_DIRS: dict[str, PerlConfigVars] = {}
 
 
 @functools.lru_cache(1)
@@ -729,7 +747,7 @@ def detect_fakeroot() -> bool:
 
 
 @functools.lru_cache(1)
-def _sc_arg_max() -> Optional[int]:
+def _sc_arg_max() -> int | None:
     try:
         return os.sysconf("SC_ARG_MAX")
     except RuntimeError:
@@ -742,7 +760,7 @@ def _split_xargs_args(
     max_args_byte_len: int,
     varargs: Iterable[str],
     reuse_list_ok: bool,
-) -> Iterator[List[str]]:
+) -> Iterator[list[str]]:
     static_cmd_len = len(static_cmd)
     remaining_len = max_args_byte_len
     pending_args = list(static_cmd)
@@ -772,9 +790,9 @@ def xargs(
     static_cmd: Sequence[str],
     varargs: Iterable[str],
     *,
-    env: Optional[Mapping[str, str]] = None,
+    env: Mapping[str, str] | None = None,
     reuse_list_ok: bool = False,
-) -> Iterator[List[str]]:
+) -> Iterator[list[str]]:
     max_args_bytes = _sc_arg_max()
     # len overshoots with one space explaining the -1.  The _split_xargs_args
     # will account for the space for the first argument
@@ -807,8 +825,8 @@ def grouper(
     n: int,
     *,
     incomplete: Literal["fill", "strict", "ignore"] = "fill",
-    fillvalue: Optional[T] = None,
-) -> Iterator[Tuple[T, ...]]:
+    fillvalue: T | None = None,
+) -> Iterator[tuple[T, ...]]:
     """Collect data into non-overlapping fixed-length chunks or blocks"""
     # grouper('ABCDEFG', 3, fillvalue='x') --> ABC DEF Gxx
     # grouper('ABCDEFG', 3, incomplete='strict') --> ABC DEF ValueError
@@ -827,7 +845,7 @@ def grouper(
 _LOGGING_SET_UP = False
 
 
-def _check_color() -> Tuple[bool, bool, Optional[str]]:
+def _check_color() -> tuple[bool, bool, str | None]:
     dpkg_or_default = os.environ.get(
         "DPKG_COLORS", "never" if "NO_COLOR" in os.environ else "auto"
     )
@@ -862,7 +880,7 @@ def program_name() -> str:
 def package_cross_check_precheck(
     pkg_a: "BinaryPackage",
     pkg_b: "BinaryPackage",
-) -> Tuple[bool, bool]:
+) -> tuple[bool, bool]:
     """Whether these two packages can do content cross-checks
 
     :param pkg_a: The first package
@@ -921,7 +939,7 @@ def change_log_level(
     logging.getLogger("").setLevel(log_level)
 
 
-def current_log_level() -> Optional[int]:
+def current_log_level() -> int | None:
     if _DEFAULT_LOGGER is not None:
         return _DEFAULT_LOGGER.level
     return None
@@ -939,7 +957,7 @@ def setup_logging(
             " Use reconfigure_logging=True if you need to reconfigure it"
         )
     stdout_color, stderr_color, bad_request = _check_color()
-    colors: Optional[Dict[str, str]] = None
+    colors: dict[str, str] | None = None
 
     if stdout_color or stderr_color:
         try:
