@@ -1,4 +1,5 @@
 import os
+import typing
 from collections.abc import Mapping
 
 import pytest
@@ -11,19 +12,22 @@ from debputy.architecture_support import (
     DpkgArchitectureBuildProcessValuesTable,
     faked_arch_table,
 )
-from debputy.filesystem_scan import FSROOverlay
+from debputy.exceptions import PureVirtualPathError
+from debputy.filesystem_scan import OSFSROOverlay
+from debputy.highlevel_manifest_parser import YAMLManifestParser
 from debputy.manifest_parser.util import AttributePath
-from debputy.packages import BinaryPackage, SourcePackage
-from debputy.plugins.debputy.debputy_plugin import initialize_debputy_features
-from debputy.plugin.api.impl_types import (
-    DebputyPluginMetadata,
-)
+from debputy.packages import BinaryPackage, SourcePackage, DctrlParser
+from debputy.plugin.api import VirtualPath
 from debputy.plugin.api.feature_set import PluginProvidedFeatureSet
 from debputy.plugin.api.impl import (
     DebputyPluginInitializerProvider,
     _resolve_bundled_plugin_docs_path,
     find_json_plugin,
 )
+from debputy.plugin.api.impl_types import (
+    DebputyPluginMetadata,
+)
+from debputy.plugins.debputy.debputy_plugin import initialize_debputy_features
 from debputy.substitution import (
     NULL_SUBSTITUTION,
     Substitution,
@@ -127,7 +131,7 @@ def amd64_substitution(
     amd64_dpkg_architecture_variables,
     _empty_debputy_plugin_feature_set,
 ) -> Substitution:
-    debian_dir = FSROOverlay.create_root_dir("debian", "debian")
+    debian_dir = OSFSROOverlay.create_root_dir("debian", "debian")
     variable_context = VariableContext(
         debian_dir,
     )
@@ -192,3 +196,71 @@ def debputy_plugin_feature_set(
 @pytest.fixture
 def attribute_path(request) -> AttributePath:
     return AttributePath.builtin_path()[request.node.nodeid]
+
+
+class ManifestParserFactory(typing.Protocol):
+
+    def __call__(
+        self,
+        debian_dir: VirtualPath,
+        *,
+        dctrl_parser: DctrlParser | None = None,
+    ) -> YAMLManifestParser: ...
+
+
+@pytest.fixture()
+def manifest_parser_amd64_factory(
+    amd64_dpkg_architecture_variables,
+    dpkg_arch_query,
+    amd64_substitution,
+    no_profiles_or_build_options,
+    debputy_plugin_feature_set,
+) -> ManifestParserFactory:
+
+    def _factory(
+        debian_dir: VirtualPath,
+        *,
+        dctrl_parser: DctrlParser | None = None,
+    ) -> YAMLManifestParser:
+        if dctrl_parser is None:
+            dctrl_parser = DctrlParser(
+                frozenset(),
+                frozenset(),
+                False,
+                False,
+                amd64_dpkg_architecture_variables,
+                dpkg_arch_query,
+                no_profiles_or_build_options,
+            )
+
+        dctrl = debian_dir.get("control")
+        try:
+            if dctrl is None:
+                raise PureVirtualPathError()
+
+            with dctrl.open() as fd:
+                _, source_package, binary_packages = (
+                    dctrl_parser.parse_source_debian_control(fd.readlines())
+                )
+        except PureVirtualPathError:
+            raise ValueError(
+                "The manifest_parser_amd64_factory requires `control` to be present and readable in the `debian_dir`"
+            ) from None
+
+        return YAMLManifestParser(
+            "debian/test-debputy.manifest",
+            source_package,
+            binary_packages,
+            amd64_substitution,
+            # To align with the substitution, we do not allow this field to
+            # be overwritten by the DctrlParser
+            amd64_dpkg_architecture_variables,
+            # Use the same instances
+            dctrl_parser.dpkg_arch_query_table,
+            dctrl_parser.deb_options_and_profiles,
+            debputy_plugin_feature_set,
+            "full",
+            debian_dir=debian_dir,
+        )
+
+    return _factory
