@@ -1,17 +1,13 @@
 import asyncio
 import collections
 from typing import (
-    List,
     Optional,
-    Mapping,
     Any,
-    Callable,
-    Sequence,
     TYPE_CHECKING,
-    Awaitable,
     cast,
     Union,
 )
+from collections.abc import Mapping, Callable, Sequence, Awaitable
 
 import pytest
 
@@ -21,6 +17,7 @@ from debputy.linting.lint_util import (
     LintStateImpl,
     LintState,
     AsyncLinterImpl,
+    te_range_to_lsp,
 )
 from debputy.lsp.config.config_options import (
     ALL_DEBPUTY_CONFIG_OPTIONS,
@@ -38,6 +35,9 @@ from debputy.lsp.text_edit import apply_text_edits
 from debputy.packages import DctrlParser
 from debputy.plugin.api.feature_set import PluginProvidedFeatureSet
 from debputy.util import T
+from debputy.lsp.vendoring._deb822_repro.locatable import (
+    Range as TERange,
+)
 
 if TYPE_CHECKING:
     from lsprotocol import types
@@ -58,7 +58,7 @@ LINTER_POSITION_CODEC = LinterPositionCodec()
 class TestDebputyConfig(DebputyConfig):
 
     def set_config_value(
-        self, config_option: DebputyConfigOption[T], value: Optional[T]
+        self, config_option: DebputyConfigOption[T], value: T | None
     ) -> None:
         self._config[config_option] = value
 
@@ -74,19 +74,19 @@ class BaseLintStateWrapper:
         dctrl_parser: DctrlParser,
     ) -> None:
         self._debputy_plugin_feature_set = debputy_plugin_feature_set
-        self.dctrl_lines: Optional[List[str]] = None
+        self.dctrl_lines: list[str] | None = None
         self.path = path
         self._dctrl_parser = dctrl_parser
-        self.source_root: Optional[VirtualPathBase] = None
+        self.source_root: VirtualPathBase | None = None
         self.maint_preference_table = MaintainerPreferenceTable({}, {})
-        self.effective_preference: Optional[EffectiveFormattingPreference] = None
+        self.effective_preference: EffectiveFormattingPreference | None = None
         self.debputy_config = TestDebputyConfig(
             {c: c.default_value for c in ALL_DEBPUTY_CONFIG_OPTIONS}
         )
 
     def _setup(
         self,
-        lines: List[str],
+        lines: list[str],
         *,
         lint_handler: Optional["AsyncLinterImpl"] = None,
     ) -> LintStateImpl:
@@ -128,7 +128,7 @@ class LintWrapper(BaseLintStateWrapper):
     ) -> None:
         super().__init__(path, debputy_plugin_feature_set, dctrl_parser)
         self._handler = handler
-        self._lint_state: Optional[LintStateImpl] = None
+        self._lint_state: LintStateImpl | None = None
 
     @property
     def lint_state(self) -> LintStateImpl:
@@ -138,7 +138,7 @@ class LintWrapper(BaseLintStateWrapper):
         ), "lint_state must be fetched after the diagnostics have been run"
         return lint_state
 
-    def __call__(self, lines: List[str]) -> List["types.Diagnostic"]:
+    def __call__(self, lines: list[str]) -> list["types.Diagnostic"]:
         self._lint_state = lint_state = self._setup(lines, lint_handler=self._handler)
         return check_diagnostics(asyncio.run(lint_state.gather_diagnostics()))
 
@@ -148,7 +148,7 @@ class ReformatWrapper(BaseLintStateWrapper):
     def __init__(
         self,
         path: str,
-        handler: Callable[[LintState], Optional[Sequence[types.TextEdit]]],
+        handler: Callable[[LintState], Sequence[types.TextEdit] | None],
         debputy_plugin_feature_set: PluginProvidedFeatureSet,
         dctrl_parser: DctrlParser,
         maint_preference_table: MaintainerPreferenceTable,
@@ -157,8 +157,8 @@ class ReformatWrapper(BaseLintStateWrapper):
         self._handler = handler
         self.maint_preference_table = maint_preference_table
 
-    def reformat(self, lines: List[str]) -> List["types.TextEdit"]:
-        def _style(lint_state: LintState) -> Optional[EffectiveFormattingPreference]:
+    def reformat(self, lines: list[str]) -> list["types.TextEdit"]:
+        def _style(lint_state: LintState) -> EffectiveFormattingPreference | None:
             pref_result = determine_effective_preference(
                 lint_state.maint_preference_table, lint_state.source_package, None
             )
@@ -167,17 +167,17 @@ class ReformatWrapper(BaseLintStateWrapper):
         return self._reformat(lines, _style)
 
     def reformat_with_named_style(
-        self, named_style: str, lines: List[str]
-    ) -> List["types.TextEdit"]:
+        self, named_style: str, lines: list[str]
+    ) -> list["types.TextEdit"]:
         return self._reformat(
             lines, lambda s: s.maint_preference_table.named_styles[named_style]
         )
 
     def _reformat(
         self,
-        lines: List[str],
-        style_selector: Callable[[LintState], Optional[EffectiveFormattingPreference]],
-    ) -> List["types.TextEdit"]:
+        lines: list[str],
+        style_selector: Callable[[LintState], EffectiveFormattingPreference | None],
+    ) -> list["types.TextEdit"]:
         lint_state = self._setup(lines, lint_handler=None)
         lint_state.effective_preference = style_selector(lint_state)
         reformat_result = self._handler(lint_state)
@@ -191,8 +191,8 @@ def requires_levenshtein(func: Any) -> Any:
 
 
 def check_diagnostics(
-    diagnostics: Optional[List["types.Diagnostic"]],
-) -> List["types.Diagnostic"]:
+    diagnostics: list["types.Diagnostic"] | None,
+) -> list["types.Diagnostic"]:
     if diagnostics:
         for diagnostic in diagnostics:
             assert diagnostic.severity is not None
@@ -209,8 +209,8 @@ def by_range_sort_key(diagnostic: types.Diagnostic) -> Any:
 
 
 def group_diagnostics_by_severity(
-    diagnostics: Optional[List["types.Diagnostic"]],
-) -> Mapping["types.DiagnosticSeverity", List["types.Diagnostic"]]:
+    diagnostics: list["types.Diagnostic"] | None,
+) -> Mapping["types.DiagnosticSeverity", list["types.Diagnostic"]]:
     if not diagnostics:
         return {}
 
@@ -239,6 +239,10 @@ def diag_range_to_text(lines: Sequence[str], range_: "types.Range") -> str:
     return "".join(parts)
 
 
+def te_range_to_text(lines: Sequence[str], range_: "TERange") -> str:
+    return diag_range_to_text(lines, te_range_to_lsp(range_))
+
+
 def standard_quick_fixes_for_diagnostic(
     lint_state: LintState,
     code_action_params: "types.CodeActionParams",
@@ -274,7 +278,7 @@ def standard_quick_fixes_for_diagnostic(
 
 
 def apply_code_action_edits(
-    doc_uri: str, lines: List[str], code_action: types.CodeAction
+    doc_uri: str, lines: list[str], code_action: types.CodeAction
 ) -> str:
     """Apply the text edits for a given doc URI from a code action
 
@@ -312,3 +316,23 @@ def apply_code_action_edits(
         edits = changes.get(doc_uri)
         assert edits
     return apply_text_edits(orig, lines, edits)
+
+
+def apply_formatting_edits(
+    lines: list[str],
+    edits: Sequence[types.TextEdit],
+) -> str:
+    orig = "".join(lines)
+    return apply_text_edits(orig, lines, edits)
+
+
+def code_actions_for(
+    uri: str,
+    text_range: "types.Range",
+) -> "types.CodeActionsParams":
+    return types.CodeActionParams(
+        types.TextDocumentIdentifier(uri),
+        text_range,
+        # Ignore the actual list of diagnostics, since we do not use them.
+        types.CodeActionContext([]),
+    )
