@@ -1,6 +1,6 @@
 from debputy.filesystem_scan import build_virtual_fs
 from debputy.plugin.api import virtual_path_def
-from debputy.plugin.debputy.package_processors import apply_compression
+from debputy.plugins.debputy.package_processors import apply_compression
 
 
 def test_apply_compression():
