@@ -1,6 +1,7 @@
 import dataclasses
 import textwrap
-from typing import Mapping, Dict, Self
+from typing import Dict, Self
+from collections.abc import Mapping
 
 import pytest
 from debian.deb822 import Deb822
@@ -87,7 +88,7 @@ class FlexibleManifestParser:
     no_profiles_or_build_options: DebBuildOptionsAndProfiles
     debputy_plugin_feature_set: PluginProvidedFeatureSet
     debian_dir: VirtualPath
-    binary_packages: Dict[str, BinaryPackage] = dataclasses.field(default_factory=dict)
+    binary_packages: dict[str, BinaryPackage] = dataclasses.field(default_factory=dict)
 
     def binary_package(
         self,
@@ -199,6 +200,7 @@ def test_install_rules(manifest_parser_pkg_foo) -> None:
 
     result = manifest.perform_installations(
         INTEGRATION_MODE_DH_DEBPUTY,
+        (),
         install_request_context=InstallSearchDirContext(
             [
                 SearchDir(debian_tmp_dir, all_pkgs),
@@ -289,6 +291,7 @@ def test_multi_dest_install_rules(manifest_parser_pkg_foo) -> None:
 
     result = manifest.perform_installations(
         INTEGRATION_MODE_DH_DEBPUTY,
+        (),
         install_request_context=InstallSearchDirContext(
             [
                 SearchDir(debian_source_root_dir, all_pkgs),
@@ -383,6 +386,7 @@ def test_install_rules_with_glob(manifest_parser_pkg_foo) -> None:
 
     result = manifest.perform_installations(
         INTEGRATION_MODE_DH_DEBPUTY,
+        (),
         install_request_context=InstallSearchDirContext(
             [
                 SearchDir(debian_tmp_dir, all_pkgs),
@@ -449,6 +453,7 @@ def test_install_rules_auto_discard_rules_dir(manifest_parser_pkg_foo) -> None:
 
     result = manifest.perform_installations(
         INTEGRATION_MODE_DH_DEBPUTY,
+        (),
         install_request_context=InstallSearchDirContext(
             [
                 SearchDir(debian_tmp_dir, all_pkgs),
@@ -509,6 +514,7 @@ def test_install_rules_auto_discard_rules_glob(manifest_parser_pkg_foo) -> None:
 
     result = manifest.perform_installations(
         INTEGRATION_MODE_DH_DEBPUTY,
+        (),
         install_request_context=InstallSearchDirContext(
             [
                 SearchDir(debian_tmp_dir, all_pkgs),
@@ -576,6 +582,7 @@ def test_install_rules_auto_discard_rules_overruled_by_explicit_install_rule(
 
     result = manifest.perform_installations(
         INTEGRATION_MODE_DH_DEBPUTY,
+        (),
         install_request_context=InstallSearchDirContext(
             [
                 SearchDir(debian_tmp_dir, all_pkgs),
@@ -646,6 +653,7 @@ def test_install_rules_install_as_with_var(manifest_parser_pkg_foo) -> None:
 
     result = manifest.perform_installations(
         INTEGRATION_MODE_DH_DEBPUTY,
+        (),
         install_request_context=InstallSearchDirContext(
             [
                 SearchDir(debian_tmp_dir, all_pkgs),
@@ -698,6 +706,7 @@ def test_install_rules_no_matches(manifest_parser_pkg_foo) -> None:
     with pytest.raises(NoMatchForInstallPatternError) as e_info:
         manifest.perform_installations(
             INTEGRATION_MODE_DH_DEBPUTY,
+            (),
             install_request_context=InstallSearchDirContext(
                 [
                     SearchDir(debian_tmp_dir, all_pkgs),
@@ -808,6 +817,7 @@ def test_install_rules_per_package_search_dirs(manifest_parser_pkg_foo_w_udeb) -
 
     result = manifest.perform_installations(
         INTEGRATION_MODE_DH_DEBPUTY,
+        (),
         install_request_context=InstallSearchDirContext(
             [
                 SearchDir(debian_tmp_deb_dir, all_deb_pkgs),
@@ -910,6 +920,7 @@ def test_install_rules_multi_into(manifest_parser_pkg_foo_w_udeb) -> None:
 
     result = manifest.perform_installations(
         INTEGRATION_MODE_DH_DEBPUTY,
+        (),
         install_request_context=InstallSearchDirContext(
             [
                 SearchDir(debian_source_root_dir, all_pkgs),
@@ -1030,6 +1041,7 @@ def test_auto_install_d_pkg(manifest_parser_pkg_foo_w_udeb) -> None:
 
     result = manifest.perform_installations(
         INTEGRATION_MODE_DH_DEBPUTY,
+        (),
         install_request_context=InstallSearchDirContext(
             [
                 SearchDir(debian_source_root_dir, all_pkgs),
@@ -1116,6 +1128,7 @@ def test_install_doc_rules_ignore_udeb(manifest_parser_pkg_foo_w_udeb) -> None:
 
     result = manifest.perform_installations(
         INTEGRATION_MODE_DH_DEBPUTY,
+        (),
         install_request_context=InstallSearchDirContext(
             [
                 SearchDir(debian_source_root_dir, all_pkgs),
@@ -1211,6 +1224,7 @@ def test_bts_1093680_crash_inactive_package(
 
     manifest.perform_installations(
         INTEGRATION_MODE_DH_DEBPUTY,
+        (),
         install_request_context=InstallSearchDirContext(
             [
                 SearchDir(debian_source_root_dir, all_pkgs),
@@ -1291,6 +1305,7 @@ def test_debputy_132(
 
     manifest.perform_installations(
         INTEGRATION_MODE_DH_DEBPUTY,
+        (),
         install_request_context=InstallSearchDirContext(
             [
                 SearchDir(debian_source_root_dir, all_pkgs),
@@ -1336,6 +1351,7 @@ def test_uncovered_installation(
     with pytest.raises(PathNotCoveredByInstallRulesError) as exec_info:
         manifest.perform_installations(
             INTEGRATION_MODE_DH_DEBPUTY,
+            (),
             install_request_context=InstallSearchDirContext(
                 [
                     SearchDir(debian_source_root_dir, all_pkgs),
