@@ -1,10 +1,10 @@
 import io
 import textwrap
-from typing import Optional, List, Tuple
 from collections.abc import Callable, Sequence
 
 import pytest
 
+from conftest import ManifestParserFactory
 from debputy.commands.debputy_cmd.output import no_fancy_output
 from debputy.dh_migration.migrators import Migrator
 from debputy.dh_migration.migrators_impl import (
@@ -27,11 +27,14 @@ from debputy.dh_migration.migrators_impl import (
     detect_dh_addons_zz_debputy_rrr,
     migrate_shell_completions,
     migrate_clean_file,
+    detect_modprobe_files,
+    migrate_dh_builtusing,
 )
 from debputy.dh_migration.models import (
     FeatureMigration,
     AcceptableMigrationIssues,
     UnsupportedFeature,
+    MigrationRequest,
 )
 from debputy.highlevel_manifest import HighLevelManifest
 from debputy.highlevel_manifest_parser import YAMLManifestParser
@@ -44,7 +47,6 @@ from debputy.plugin.api.test_api import (
     build_virtual_file_system,
 )
 
-
 DEBIAN_DIR_ENTRY = virtual_path_def(".", fs_path="/nowhere/debian")
 
 
@@ -97,20 +99,20 @@ def empty_manifest_pkg_foo(
 
 def run_migrator(
     migrator: Migrator,
-    path: VirtualPath,
+    debian_dir: VirtualPath,
     manifest: HighLevelManifest,
     acceptable_migration_issues: AcceptableMigrationIssues,
     *,
     migration_target=INTEGRATION_MODE_DH_DEBPUTY,
 ) -> FeatureMigration:
     feature_migration = FeatureMigration(migrator.__name__, no_fancy_output())
-    migrator(
-        path,
+    migration_request = MigrationRequest(
+        debian_dir,
         manifest,
         acceptable_migration_issues,
-        feature_migration,
         migration_target,
     )
+    migrator(migration_request, feature_migration)
     return feature_migration
 
 
@@ -500,6 +502,46 @@ def test_detect_pam_files(
     assert not migration.rename_paths_on_success
 
 
+def test_detect_modprobe_files(
+    empty_manifest_pkg_foo: HighLevelManifest,
+    accept_no_migration_issues: AcceptableMigrationIssues,
+) -> None:
+    migrator = detect_modprobe_files
+    empty_fs = build_virtual_file_system([DEBIAN_DIR_ENTRY])
+    pam_fs = build_virtual_file_system(
+        [
+            DEBIAN_DIR_ENTRY,
+            virtual_path_def("modprobe", fs_path="/nowhere/debian/foo.modprobe"),
+        ]
+    )
+
+    migration = run_migrator(
+        migrator,
+        empty_fs,
+        empty_manifest_pkg_foo,
+        accept_no_migration_issues,
+    )
+
+    assert not migration.anything_to_do
+    assert not migration.warnings
+    assert migration.assumed_compat is None
+    assert not migration.remove_paths_on_success
+    assert not migration.rename_paths_on_success
+
+    migration = run_migrator(
+        migrator,
+        pam_fs,
+        empty_manifest_pkg_foo,
+        accept_no_migration_issues,
+    )
+
+    assert not migration.anything_to_do
+    assert not migration.warnings
+    assert migration.assumed_compat == 14
+    assert not migration.remove_paths_on_success
+    assert not migration.rename_paths_on_success
+
+
 def test_migrate_doc_base_files(
     empty_manifest_pkg_foo: HighLevelManifest,
     accept_no_migration_issues: AcceptableMigrationIssues,
@@ -1923,3 +1965,86 @@ def test_migrate_zsh_completions_file_list(
         expected_manifest_content,
         expected_renamed_paths=expected_renames,
     )
+
+
+def test_migrate_dh_builtusing(
+    manifest_parser_amd64_factory: ManifestParserFactory,
+    accept_no_migration_issues: AcceptableMigrationIssues,
+) -> None:
+    dh_config_content = textwrap.dedent(
+        """\
+        Source: foo
+        Build-Depends: something
+
+        Package: foo
+        Architecture: any
+        # No warning; not a known substvar
+        Built-Using: ${foo:var}
+
+        Package: libfoo1
+        Architecture: any
+        Built-Using: ${dh-builtusing:var}
+    """
+    )
+    fs = build_virtual_file_system(
+        [
+            DEBIAN_DIR_ENTRY,
+            virtual_path_def(
+                "control",
+                fs_path=f"/nowhere/debian/control",
+                content=dh_config_content,
+            ),
+        ]
+    )
+    migration = run_migrator(
+        migrate_dh_builtusing,
+        fs,
+        manifest_parser_amd64_factory(fs).build_manifest(),
+        accept_no_migration_issues,
+    )
+    msg = (
+        "Migrate all `${dh-builtusing:custom-pattern}` instances in the (Static-)Built-Using of libfoo1"
+        " to `packages.libfoo1.(static-)built-using.sources: glob-pattern` in `debian/debputy.manifest`"
+    )
+    assert not migration.rename_paths_on_success
+    assert not migration.remove_paths_on_success
+    assert migration.warnings == [msg]
+    assert not migration.required_plugins
+
+
+def test_migrate_dh_builtusing_nothing_by_default(
+    manifest_parser_amd64_factory: ManifestParserFactory,
+    accept_no_migration_issues: AcceptableMigrationIssues,
+) -> None:
+    dh_config_content = textwrap.dedent(
+        """\
+        Source: foo
+        Build-Depends: something
+
+        Package: foo
+        Architecture: any
+
+        Package: libfoo1
+        Architecture: any
+    """
+    )
+    fs = build_virtual_file_system(
+        [
+            DEBIAN_DIR_ENTRY,
+            virtual_path_def(
+                "control",
+                fs_path=f"/nowhere/debian/control",
+                content=dh_config_content,
+            ),
+        ]
+    )
+    migration = run_migrator(
+        migrate_dh_builtusing,
+        fs,
+        manifest_parser_amd64_factory(fs).build_manifest(),
+        accept_no_migration_issues,
+    )
+    assert not migration.rename_paths_on_success
+    assert not migration.remove_paths_on_success
+    assert not migration.warnings
+    assert not migration.required_plugins
