@@ -1,6 +1,8 @@
+import dataclasses
 import os
 import random
-from typing import cast, TYPE_CHECKING, Sequence, Optional, Mapping
+from typing import cast, TYPE_CHECKING, Optional
+from collections.abc import Sequence, Mapping
 
 import pytest
 
@@ -147,7 +149,7 @@ def test_packager_provided_files(
     binary_packages = binary_package_table(*binaries)
 
     ppf = detect_all_packager_provided_files(
-        debputy_plugin_feature_set.packager_provided_files,
+        debputy_plugin_feature_set,
         debian_dir,
         binary_packages,
     )
@@ -206,7 +208,7 @@ def test_packager_provided_files_priority(
 
         debian_dir = build_virtual_file_system(all_entries)
         ppf = detect_all_packager_provided_files(
-            debputy_plugin_feature_set.packager_provided_files,
+            debputy_plugin_feature_set,
             debian_dir,
             binary_packages,
         )
@@ -224,7 +226,11 @@ def test_packager_provided_files_priority(
         ("foo.bar", "foo.bar-doc"),
     ],
 )
-def test_detect_ppf_simple(main_package: str, secondary_package: str) -> None:
+def test_detect_ppf_simple(
+    main_package: str,
+    secondary_package: str,
+    debputy_plugin_feature_set: PluginProvidedFeatureSet,
+) -> None:
     plugin_metadata = plugin_metadata_for_debputys_own_plugin()
     binary_packages = binary_package_table(
         faked_binary_package(main_package, is_main_package=True),
@@ -239,7 +245,8 @@ def test_detect_ppf_simple(main_package: str, secondary_package: str) -> None:
             "copyright",
         ]
     )
-    ppf_defs = _ppfs(
+    ppf_defs = _replace_ppf_def(
+        debputy_plugin_feature_set,
         _fake_PPFClassSpec(
             plugin_metadata,
             "install",
@@ -278,7 +285,9 @@ def test_detect_ppf_simple(main_package: str, secondary_package: str) -> None:
     assert {m.path.name for m in sec_matches} == expected
 
 
-def test_detect_ppf_name_segment() -> None:
+def test_detect_ppf_name_segment(
+    debputy_plugin_feature_set: PluginProvidedFeatureSet,
+) -> None:
     plugin_metadata = plugin_metadata_for_debputys_own_plugin()
     binary_packages = binary_package_table(
         faked_binary_package("foo", is_main_package=True),
@@ -289,7 +298,8 @@ def test_detect_ppf_name_segment() -> None:
             "fuu.install",
         ]
     )
-    ppf_defs = _ppfs(
+    ppf_defs = _replace_ppf_def(
+        debputy_plugin_feature_set,
         _fake_PPFClassSpec(
             plugin_metadata,
             "install",
@@ -315,7 +325,8 @@ def test_detect_ppf_name_segment() -> None:
     assert "bar" in results
     assert not results["bar"].auto_installable
 
-    ppf_defs = _ppfs(
+    ppf_defs = _replace_ppf_def(
+        debputy_plugin_feature_set,
         _fake_PPFClassSpec(
             plugin_metadata,
             "install",
@@ -342,7 +353,9 @@ def test_detect_ppf_name_segment() -> None:
     assert not results["bar"].auto_installable
 
 
-def test_detect_ppf_fuzzy_match_bug_950723() -> None:
+def test_detect_ppf_fuzzy_match_bug_950723(
+    debputy_plugin_feature_set: PluginProvidedFeatureSet,
+) -> None:
     plugin_metadata = plugin_metadata_for_debputys_own_plugin()
     binary_packages = binary_package_table(
         faked_binary_package("foo", is_main_package=True),
@@ -354,7 +367,8 @@ def test_detect_ppf_fuzzy_match_bug_950723() -> None:
             "foo@.service",
         ]
     )
-    ppf_defs = _ppfs(
+    ppf_defs = _replace_ppf_def(
+        debputy_plugin_feature_set,
         _fake_PPFClassSpec(
             plugin_metadata,
             "service",
@@ -411,7 +425,7 @@ def test_detect_ppf_fuzzy_match_bug_950723() -> None:
 
 
 @requires_levenshtein
-def test_detect_ppf_typo() -> None:
+def test_detect_ppf_typo(debputy_plugin_feature_set: PluginProvidedFeatureSet) -> None:
     plugin_metadata = plugin_metadata_for_debputys_own_plugin()
     binary_packages = binary_package_table(
         faked_binary_package("foo", is_main_package=True),
@@ -423,7 +437,8 @@ def test_detect_ppf_typo() -> None:
             "bar.intsall",
         ]
     )
-    ppf_defs = _ppfs(
+    ppf_defs = _replace_ppf_def(
+        debputy_plugin_feature_set,
         _fake_PPFClassSpec(
             plugin_metadata,
             "install",
@@ -495,7 +510,9 @@ def test_detect_ppf_typo() -> None:
     assert os.path.basename(bar_logcheck.expected_path) == "bar.logcheck.violations.d"
 
 
-def test_debhelper_overlapping_stems() -> None:
+def test_debhelper_overlapping_stems(
+    debputy_plugin_feature_set: PluginProvidedFeatureSet,
+) -> None:
     plugin_metadata = plugin_metadata_for_debputys_own_plugin()
     binary_packages = binary_package_table(
         faked_binary_package("foo", is_main_package=True),
@@ -505,7 +522,8 @@ def test_debhelper_overlapping_stems() -> None:
             "foo.user.service",
         ]
     )
-    ppf_defs = _ppfs(
+    ppf_defs = _replace_ppf_def(
+        debputy_plugin_feature_set,
         _fake_PPFClassSpec(
             plugin_metadata,
             "service",
@@ -553,7 +571,9 @@ def test_debhelper_overlapping_stems() -> None:
 
 
 @requires_levenshtein
-def test_debhelper_overlapping_stems_typo_check() -> None:
+def test_debhelper_overlapping_stems_typo_check(
+    debputy_plugin_feature_set: PluginProvidedFeatureSet,
+) -> None:
     plugin_metadata = plugin_metadata_for_debputys_own_plugin()
     binary_packages = binary_package_table(
         faked_binary_package("foo", is_main_package=True),
@@ -563,7 +583,8 @@ def test_debhelper_overlapping_stems_typo_check() -> None:
             "foo.uxxr.service",
         ]
     )
-    ppf_defs = _ppfs(
+    ppf_defs = _replace_ppf_def(
+        debputy_plugin_feature_set,
         _fake_PPFClassSpec(
             plugin_metadata,
             "service",
@@ -603,7 +624,7 @@ def _fake_PPFClassSpec(
     debputy_plugin_metadata: DebputyPluginMetadata,
     stem: str,
     *,
-    default_priority: Optional[int] = None,
+    default_priority: int | None = None,
     packageless_is_fallback_for_all_packages: bool = False,
     bug_950723: bool = False,
     has_active_command: bool = False,
@@ -626,7 +647,11 @@ def _fake_PPFClassSpec(
     )
 
 
-def _ppfs(
+def _replace_ppf_def(
+    debputy_plugin_feature_set: PluginProvidedFeatureSet,
     *ppfs: PackagerProvidedFileClassSpec,
-) -> Mapping[str, PackagerProvidedFileClassSpec]:
-    return {ppf.stem: ppf for ppf in ppfs}
+) -> PluginProvidedFeatureSet:
+    ppf_def = {ppf.stem: ppf for ppf in ppfs}
+    return dataclasses.replace(
+        debputy_plugin_feature_set, packager_provided_files=ppf_def
+    )
