@@ -1,6 +1,9 @@
+import subprocess
+from functools import lru_cache
+
 import pytest
 
-from typing import Tuple, Any
+from typing import Any
 from collections.abc import Mapping
 
 from debian.deb822 import Deb822
@@ -12,6 +15,7 @@ from debputy.architecture_support import (
 )
 from debputy.packages import BinaryPackage
 from debputy.plugin.api.test_api import DEBPUTY_TEST_AGAINST_INSTALLED_PLUGINS
+from debputy.util import _error
 
 _DPKG_ARCHITECTURE_TABLE_NATIVE_AMD64 = None
 _DPKG_ARCH_QUERY_TABLE = None
@@ -83,3 +87,22 @@ def with_plugins_loaded(*plugin_names: str) -> Any:
         return func
 
     return _wrapper
+
+
+@lru_cache
+def is_pkg_installed(pkg: str) -> bool:
+    cp = subprocess.run(
+        args=(
+            "dpkg-query",
+            "-Wf${db:Status-Abbrev}\n",
+            pkg,
+        ),
+        capture_output=True,
+        check=False,
+        text=True,
+    )
+    # 0: OK    1: unknown package    2: other
+    if cp.returncode not in (0, 1):
+        _error(f"dpkg-query -W failed (code: {cp.returncode}): {cp.stderr.rstrip()}")
+
+    return len(cp.stdout) > 1 and cp.stdout[1] == "i"
